@@ -1,17 +1,19 @@
 from django.contrib import admin
-from django.contrib.flatpages.models import FlatPage
-from django.utils.translation import ugettext_lazy as _
 from django.contrib.flatpages.forms import FlatpageForm
+from django.contrib.flatpages.models import FlatPage
+from django.utils.translation import gettext_lazy as _
 
 
+@admin.register(FlatPage)
 class FlatPageAdmin(admin.ModelAdmin):
     form = FlatpageForm
     fieldsets = (
         (None, {'fields': ('url', 'title', 'content', 'sites')}),
-        (_('Advanced options'), {'classes': ('collapse',), 'fields': ('enable_comments', 'registration_required', 'template_name')}),
+        (_('Advanced options'), {
+            'classes': ('collapse',),
+            'fields': ('registration_required', 'template_name'),
+        }),
     )
     list_display = ('url', 'title')
-    list_filter = ('sites', 'enable_comments', 'registration_required')
+    list_filter = ('sites', 'registration_required')
     search_fields = ('url', 'title')
-
-admin.site.register(FlatPage, FlatPageAdmin)
