@@ -1,13 +1,10 @@
-from __future__ import unicode_literals
-
 import logging
 
 from django.conf import settings
 from django.contrib.gis import gdal
-from django.contrib.gis.geos import GEOSGeometry, GEOSException
+from django.contrib.gis.geometry import json_regex
+from django.contrib.gis.geos import GEOSException, GEOSGeometry
 from django.forms.widgets import Widget
-from django.template import loader
-from django.utils import six
 from django.utils import translation
 
 logger = logging.getLogger('django.contrib.gis')
@@ -16,7 +13,7 @@ logger = logging.getLogger('django.contrib.gis')
 class BaseGeometryWidget(Widget):
     """
     The base class for rich geometry widgets.
-    Renders a map using the WKT of the geometry.
+    Render a map using the WKT of the geometry.
     """
     geom_type = 'GEOMETRY'
     map_srid = 4326
@@ -39,82 +36,82 @@ class BaseGeometryWidget(Widget):
 
     def deserialize(self, value):
         try:
-            return GEOSGeometry(value, self.map_srid)
-        except (GEOSException, ValueError) as err:
-            logger.error(
-                "Error creating geometry from value '%s' (%s)" % (
-                    value, err)
-            )
+            return GEOSGeometry(value)
+        except (GEOSException, ValueError, TypeError) as err:
+            logger.error("Error creating geometry from value '%s' (%s)", value, err)
         return None
 
-    def render(self, name, value, attrs=None):
+    def get_context(self, name, value, attrs):
+        context = super().get_context(name, value, attrs)
         # If a string reaches here (via a validation error on another
         # field) then just reconstruct the Geometry.
-        if isinstance(value, six.string_types):
+        if value and isinstance(value, str):
             value = self.deserialize(value)
 
         if value:
             # Check that srid of value and map match
-            if value.srid != self.map_srid:
+            if value.srid and value.srid != self.map_srid:
                 try:
                     ogr = value.ogr
                     ogr.transform(self.map_srid)
                     value = ogr
-                except gdal.OGRException as err:
+                except gdal.GDALException as err:
                     logger.error(
-                        "Error transforming geometry from srid '%s' to srid '%s' (%s)" % (
-                            value.srid, self.map_srid, err)
+                        "Error transforming geometry from srid '%s' to srid '%s' (%s)",
+                        value.srid, self.map_srid, err
                     )
 
-        context = self.build_attrs(
-            attrs,
-            name=name,
-            module='geodjango_%s' % name.replace('-', '_'),  # JS-safe
-            serialized=self.serialize(value),
-            geom_type=gdal.OGRGeomType(self.attrs['geom_type']),
-            STATIC_URL=settings.STATIC_URL,
-            LANGUAGE_BIDI=translation.get_language_bidi(),
-        )
-        return loader.render_to_string(self.template_name, context)
+        context.update(self.build_attrs(self.attrs, {
+            'name': name,
+            'module': 'geodjango_%s' % name.replace('-', '_'),  # JS-safe
+            'serialized': self.serialize(value),
+            'geom_type': gdal.OGRGeomType(self.attrs['geom_type']),
+            'STATIC_URL': settings.STATIC_URL,
+            'LANGUAGE_BIDI': translation.get_language_bidi(),
+            **(attrs or {}),
+        }))
+        return context
 
 
 class OpenLayersWidget(BaseGeometryWidget):
     template_name = 'gis/openlayers.html'
+    map_srid = 3857
 
     class Media:
+        css = {
+            'all': (
+                'https://cdnjs.cloudflare.com/ajax/libs/ol3/4.6.5/ol.css',
+                'gis/css/ol3.css',
+            )
+        }
         js = (
-            'http://openlayers.org/api/2.13/OpenLayers.js',
+            'https://cdnjs.cloudflare.com/ajax/libs/ol3/4.6.5/ol.js',
             'gis/js/OLMapWidget.js',
         )
 
+    def serialize(self, value):
+        return value.json if value else ''
 
-class OSMWidget(BaseGeometryWidget):
+    def deserialize(self, value):
+        geom = super().deserialize(value)
+        # GeoJSON assumes WGS84 (4326). Use the map's SRID instead.
+        if geom and json_regex.match(value) and self.map_srid != 4326:
+            geom.srid = self.map_srid
+        return geom
+
+
+class OSMWidget(OpenLayersWidget):
     """
     An OpenLayers/OpenStreetMap-based widget.
     """
     template_name = 'gis/openlayers-osm.html'
     default_lon = 5
     default_lat = 47
-
-    class Media:
-        js = (
-            'http://openlayers.org/api/2.13/OpenLayers.js',
-            'http://www.openstreetmap.org/openlayers/OpenStreetMap.js',
-            'gis/js/OLMapWidget.js',
-        )
+    default_zoom = 12
 
     def __init__(self, attrs=None):
-        super(OSMWidget, self).__init__()
-        for key in ('default_lon', 'default_lat'):
+        super().__init__()
+        for key in ('default_lon', 'default_lat', 'default_zoom'):
             self.attrs[key] = getattr(self, key)
         if attrs:
             self.attrs.update(attrs)
-
-    @property
-    def map_srid(self):
-        # Use the official spherical mercator projection SRID on versions
-        # of GDAL that support it; otherwise, fallback to 900913.
-        if gdal.HAS_GDAL and gdal.GDAL_VERSION >= (1, 7):
-            return 3857
-        else:
-            return 900913
