@@ -8,13 +8,11 @@ from django.contrib.gis.gdal.field import (
     OFTDate, OFTDateTime, OFTInteger, OFTInteger64, OFTReal, OFTString,
     OFTTime,
 )
-from django.utils import six
-from django.utils.six.moves import zip
 
 
 def mapping(data_source, geom_name='geom', layer_key=0, multi_geom=False):
     """
-    Given a DataSource, generates a dictionary that may be used
+    Given a DataSource, generate a dictionary that may be used
     for invoking the LayerMapping utility.
 
     Keyword Arguments:
@@ -26,7 +24,7 @@ def mapping(data_source, geom_name='geom', layer_key=0, multi_geom=False):
 
      `multi_geom` => Boolean (default: False) - specify as multigeometry.
     """
-    if isinstance(data_source, six.string_types):
+    if isinstance(data_source, str):
         # Instantiating the DataSource from the string.
         data_source = DataSource(data_source)
     elif isinstance(data_source, DataSource):
@@ -62,7 +60,7 @@ def ogrinspect(*args, **kwargs):
 
     ...will print model definition to stout
 
-    or put this in a python script and use to redirect the output to a new
+    or put this in a Python script and use to redirect the output to a new
     model like:
 
     $ python generate_model.py > myapp/models.py
@@ -94,7 +92,7 @@ def ogrinspect(*args, **kwargs):
      `multi_geom` => Boolean (default: False) - specify as multigeometry.
 
      `name_field` => String - specifies a field name to return for the
-       `__unicode__`/`__str__` function (which will be generated if specified).
+       __str__() method (which will be generated if specified).
 
      `imports` => Boolean (default: True) - set to False to omit the
        `from django.contrib.gis.db import models` code from the
@@ -116,7 +114,7 @@ def ogrinspect(*args, **kwargs):
        give specific fields to have null, then a list/tuple of OGR field
        names may be used.
 
-    Note: This routine calls the _ogrinspect() helper to do the heavy lifting.
+    Note: Call the _ogrinspect() helper to do the heavy lifting.
     """
     return '\n'.join(s for s in _ogrinspect(*args, **kwargs))
 
@@ -129,7 +127,7 @@ def _ogrinspect(data_source, model_name, geom_name='geom', layer_key=0, srid=Non
     to the given data source.  See the `ogrinspect` docstring for more details.
     """
     # Getting the DataSource
-    if isinstance(data_source, six.string_types):
+    if isinstance(data_source, str):
         data_source = DataSource(data_source)
     elif isinstance(data_source, DataSource):
         pass
@@ -171,6 +169,7 @@ def _ogrinspect(data_source, model_name, geom_name='geom', layer_key=0, srid=Non
         yield '# This is an auto-generated Django model module created by ogrinspect.'
         yield 'from django.contrib.gis.db import models'
         yield ''
+        yield ''
 
     yield 'class %s(models.Model):' % model_name
 
@@ -235,5 +234,4 @@ def _ogrinspect(data_source, model_name, geom_name='geom', layer_key=0, srid=Non
 
     if name_field:
         yield ''
-        yield '    def __%s__(self): return self.%s' % (
-            'str' if six.PY3 else 'unicode', name_field)
+        yield '    def __str__(self): return self.%s' % name_field
