@@ -6,33 +6,34 @@ from django.contrib.postgres.validators import ArrayMaxLengthValidator
 from django.core import checks, exceptions
 from django.db.models import Field, IntegerField, Transform
 from django.db.models.lookups import Exact, In
-from django.utils import six
-from django.utils.translation import ugettext_lazy as _
+from django.utils.inspect import func_supports_parameter
+from django.utils.translation import gettext_lazy as _
 
 from ..utils import prefix_validation_error
+from .mixins import CheckFieldDefaultMixin
 from .utils import AttributeSetter
 
 __all__ = ['ArrayField']
 
 
-class ArrayField(Field):
+class ArrayField(CheckFieldDefaultMixin, Field):
     empty_strings_allowed = False
     default_error_messages = {
-        'item_invalid': _('Item %(nth)s in the array did not validate: '),
+        'item_invalid': _('Item %(nth)s in the array did not validate:'),
         'nested_array_mismatch': _('Nested arrays must have the same length.'),
     }
+    _default_hint = ('list', '[]')
 
     def __init__(self, base_field, size=None, **kwargs):
         self.base_field = base_field
         self.size = size
         if self.size:
-            self.default_validators = self.default_validators[:]
-            self.default_validators.append(ArrayMaxLengthValidator(self.size))
+            self.default_validators = [*self.default_validators, ArrayMaxLengthValidator(self.size)]
         # For performance, only add a from_db_value() method if the base field
         # implements it.
         if hasattr(self.base_field, 'from_db_value'):
             self.from_db_value = self._from_db_value
-        super(ArrayField, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     @property
     def model(self):
@@ -47,7 +48,7 @@ class ArrayField(Field):
         self.base_field.model = model
 
     def check(self, **kwargs):
-        errors = super(ArrayField, self).check(**kwargs)
+        errors = super().check(**kwargs)
         if self.base_field.remote_field:
             errors.append(
                 checks.Error(
@@ -71,7 +72,7 @@ class ArrayField(Field):
         return errors
 
     def set_attributes_from_name(self, name):
-        super(ArrayField, self).set_attributes_from_name(name)
+        super().set_attributes_from_name(name)
         self.base_field.set_attributes_from_name(name)
 
     @property
@@ -82,33 +83,38 @@ class ArrayField(Field):
         size = self.size or ''
         return '%s[%s]' % (self.base_field.db_type(connection), size)
 
+    def get_placeholder(self, value, compiler, connection):
+        return '%s::{}'.format(self.db_type(connection))
+
     def get_db_prep_value(self, value, connection, prepared=False):
-        if isinstance(value, list) or isinstance(value, tuple):
+        if isinstance(value, (list, tuple)):
             return [self.base_field.get_db_prep_value(i, connection, prepared=False) for i in value]
         return value
 
     def deconstruct(self):
-        name, path, args, kwargs = super(ArrayField, self).deconstruct()
+        name, path, args, kwargs = super().deconstruct()
         if path == 'django.contrib.postgres.fields.array.ArrayField':
             path = 'django.contrib.postgres.fields.ArrayField'
         kwargs.update({
-            'base_field': self.base_field,
+            'base_field': self.base_field.clone(),
             'size': self.size,
         })
         return name, path, args, kwargs
 
     def to_python(self, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             # Assume we're deserializing
             vals = json.loads(value)
             value = [self.base_field.to_python(val) for val in vals]
         return value
 
-    def _from_db_value(self, value, expression, connection, context):
+    def _from_db_value(self, value, expression, connection):
         if value is None:
             return value
         return [
-            self.base_field.from_db_value(item, expression, connection, context)
+            self.base_field.from_db_value(item, expression, connection, {})
+            if func_supports_parameter(self.base_field.from_db_value, 'context')  # RemovedInDjango30Warning
+            else self.base_field.from_db_value(item, expression, connection)
             for item in value
         ]
 
@@ -126,16 +132,17 @@ class ArrayField(Field):
         return json.dumps(values)
 
     def get_transform(self, name):
-        transform = super(ArrayField, self).get_transform(name)
+        transform = super().get_transform(name)
         if transform:
             return transform
-        try:
-            index = int(name)
-        except ValueError:
-            pass
-        else:
-            index += 1  # postgres uses 1-indexing
-            return IndexTransformFactory(index, self.base_field)
+        if '_' not in name:
+            try:
+                index = int(name)
+            except ValueError:
+                pass
+            else:
+                index += 1  # postgres uses 1-indexing
+                return IndexTransformFactory(index, self.base_field)
         try:
             start, end = name.split('_')
             start = int(start) + 1
@@ -146,7 +153,7 @@ class ArrayField(Field):
             return SliceTransformFactory(start, end)
 
     def validate(self, value, model_instance):
-        super(ArrayField, self).validate(value, model_instance)
+        super().validate(value, model_instance)
         for index, part in enumerate(value):
             try:
                 self.base_field.validate(part, model_instance)
@@ -155,7 +162,7 @@ class ArrayField(Field):
                     error,
                     prefix=self.error_messages['item_invalid'],
                     code='item_invalid',
-                    params={'nth': index},
+                    params={'nth': index + 1},
                 )
         if isinstance(self.base_field, ArrayField):
             if len({len(i) for i in value}) > 1:
@@ -165,7 +172,7 @@ class ArrayField(Field):
                 )
 
     def run_validators(self, value):
-        super(ArrayField, self).run_validators(value)
+        super().run_validators(value)
         for index, part in enumerate(value):
             try:
                 self.base_field.run_validators(part)
@@ -174,23 +181,22 @@ class ArrayField(Field):
                     error,
                     prefix=self.error_messages['item_invalid'],
                     code='item_invalid',
-                    params={'nth': index},
+                    params={'nth': index + 1},
                 )
 
     def formfield(self, **kwargs):
-        defaults = {
+        return super().formfield(**{
             'form_class': SimpleArrayField,
             'base_field': self.base_field.formfield(),
             'max_length': self.size,
-        }
-        defaults.update(kwargs)
-        return super(ArrayField, self).formfield(**defaults)
+            **kwargs,
+        })
 
 
 @ArrayField.register_lookup
 class ArrayContains(lookups.DataContains):
     def as_sql(self, qn, connection):
-        sql, params = super(ArrayContains, self).as_sql(qn, connection)
+        sql, params = super().as_sql(qn, connection)
         sql = '%s::%s' % (sql, self.lhs.output_field.db_type(connection))
         return sql, params
 
@@ -198,7 +204,7 @@ class ArrayContains(lookups.DataContains):
 @ArrayField.register_lookup
 class ArrayContainedBy(lookups.ContainedBy):
     def as_sql(self, qn, connection):
-        sql, params = super(ArrayContainedBy, self).as_sql(qn, connection)
+        sql, params = super().as_sql(qn, connection)
         sql = '%s::%s' % (sql, self.lhs.output_field.db_type(connection))
         return sql, params
 
@@ -206,7 +212,7 @@ class ArrayContainedBy(lookups.ContainedBy):
 @ArrayField.register_lookup
 class ArrayExact(Exact):
     def as_sql(self, qn, connection):
-        sql, params = super(ArrayExact, self).as_sql(qn, connection)
+        sql, params = super().as_sql(qn, connection)
         sql = '%s::%s' % (sql, self.lhs.output_field.db_type(connection))
         return sql, params
 
@@ -214,7 +220,7 @@ class ArrayExact(Exact):
 @ArrayField.register_lookup
 class ArrayOverlap(lookups.Overlap):
     def as_sql(self, qn, connection):
-        sql, params = super(ArrayOverlap, self).as_sql(qn, connection)
+        sql, params = super().as_sql(qn, connection)
         sql = '%s::%s' % (sql, self.lhs.output_field.db_type(connection))
         return sql, params
 
@@ -236,16 +242,25 @@ class ArrayLenTransform(Transform):
 @ArrayField.register_lookup
 class ArrayInLookup(In):
     def get_prep_lookup(self):
-        values = super(ArrayInLookup, self).get_prep_lookup()
+        values = super().get_prep_lookup()
+        if hasattr(self.rhs, '_prepare'):
+            # Subqueries don't need further preparation.
+            return values
         # In.process_rhs() expects values to be hashable, so convert lists
         # to tuples.
-        return [tuple(value) for value in values]
+        prepared_values = []
+        for value in values:
+            if hasattr(value, 'resolve_expression'):
+                prepared_values.append(value)
+            else:
+                prepared_values.append(tuple(value))
+        return prepared_values
 
 
 class IndexTransform(Transform):
 
     def __init__(self, index, base_field, *args, **kwargs):
-        super(IndexTransform, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.index = index
         self.base_field = base_field
 
@@ -258,7 +273,7 @@ class IndexTransform(Transform):
         return self.base_field
 
 
-class IndexTransformFactory(object):
+class IndexTransformFactory:
 
     def __init__(self, index, base_field):
         self.index = index
@@ -271,7 +286,7 @@ class IndexTransformFactory(object):
 class SliceTransform(Transform):
 
     def __init__(self, start, end, *args, **kwargs):
-        super(SliceTransform, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.start = start
         self.end = end
 
@@ -280,7 +295,7 @@ class SliceTransform(Transform):
         return '%s[%s:%s]' % (lhs, self.start, self.end), params
 
 
-class SliceTransformFactory(object):
+class SliceTransformFactory:
 
     def __init__(self, start, end):
         self.start = start
