@@ -1,17 +1,16 @@
 import json
 
 from django import forms
-from django.utils import six
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 __all__ = ['JSONField']
 
 
-class InvalidJSONInput(six.text_type):
+class InvalidJSONInput(str):
     pass
 
 
-class JSONString(six.text_type):
+class JSONString(str):
     pass
 
 
@@ -30,13 +29,13 @@ class JSONField(forms.CharField):
             return value
         try:
             converted = json.loads(value)
-        except ValueError:
+        except json.JSONDecodeError:
             raise forms.ValidationError(
                 self.error_messages['invalid'],
                 code='invalid',
                 params={'value': value},
             )
-        if isinstance(converted, six.text_type):
+        if isinstance(converted, str):
             return JSONString(converted)
         else:
             return converted
@@ -46,10 +45,18 @@ class JSONField(forms.CharField):
             return initial
         try:
             return json.loads(data)
-        except ValueError:
+        except json.JSONDecodeError:
             return InvalidJSONInput(data)
 
     def prepare_value(self, value):
         if isinstance(value, InvalidJSONInput):
             return value
         return json.dumps(value)
+
+    def has_changed(self, initial, data):
+        if super().has_changed(initial, data):
+            return True
+        # For purposes of seeing whether something has changed, True isn't the
+        # same as 1 and the order of keys doesn't matter.
+        data = self.to_python(data)
+        return json.dumps(initial, sort_keys=True) != json.dumps(data, sort_keys=True)
