@@ -1,41 +1,48 @@
+import functools
 import os
+from collections import OrderedDict
+
+from django.apps import apps
 from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured
-from django.core.files.storage import default_storage, Storage, FileSystemStorage
-from django.utils.datastructures import SortedDict
-from django.utils.functional import empty, memoize, LazyObject
-from django.utils.module_loading import import_by_path
-from django.utils._os import safe_join
-from django.utils import six
-
 from django.contrib.staticfiles import utils
-from django.contrib.staticfiles.storage import AppStaticStorage
+from django.core.checks import Error
+from django.core.exceptions import ImproperlyConfigured
+from django.core.files.storage import (
+    FileSystemStorage, Storage, default_storage,
+)
+from django.utils._os import safe_join
+from django.utils.functional import LazyObject, empty
+from django.utils.module_loading import import_string
 
-_finders = SortedDict()
+# To keep track on which directories the finder has searched the static files.
+searched_locations = []
 
 
-class BaseFinder(object):
+class BaseFinder:
     """
     A base file finder to be used for custom staticfiles finder classes.
     """
+    def check(self, **kwargs):
+        raise NotImplementedError(
+            'subclasses may provide a check() method to verify the finder is '
+            'configured correctly.'
+        )
+
     def find(self, path, all=False):
         """
-        Given a relative file path this ought to find an
-        absolute file path.
+        Given a relative file path, find an absolute file path.
 
-        If the ``all`` parameter is ``False`` (default) only
-        the first found file path will be returned; if set
-        to ``True`` a list of all found files paths is returned.
+        If the ``all`` parameter is False (default) return only the first found
+        file path; if True, return a list of all found files paths.
         """
-        raise NotImplementedError()
+        raise NotImplementedError('subclasses of BaseFinder must provide a find() method')
 
     def list(self, ignore_patterns):
         """
-        Given an optional list of paths to ignore, this should return
-        a two item iterable consisting of the relative path and storage
-        instance.
+        Given an optional list of paths to ignore, return a two item iterable
+        consisting of the relative path and storage instance.
         """
-        raise NotImplementedError()
+        raise NotImplementedError('subclasses of BaseFinder must provide a list() method')
 
 
 class FileSystemFinder(BaseFinder):
@@ -43,39 +50,57 @@ class FileSystemFinder(BaseFinder):
     A static files finder that uses the ``STATICFILES_DIRS`` setting
     to locate files.
     """
-    def __init__(self, apps=None, *args, **kwargs):
+    def __init__(self, app_names=None, *args, **kwargs):
         # List of locations with static files
         self.locations = []
         # Maps dir paths to an appropriate storage instance
-        self.storages = SortedDict()
-        if not isinstance(settings.STATICFILES_DIRS, (list, tuple)):
-            raise ImproperlyConfigured(
-                "Your STATICFILES_DIRS setting is not a tuple or list; "
-                "perhaps you forgot a trailing comma?")
+        self.storages = OrderedDict()
         for root in settings.STATICFILES_DIRS:
             if isinstance(root, (list, tuple)):
                 prefix, root = root
             else:
                 prefix = ''
-            if settings.STATIC_ROOT and os.path.abspath(settings.STATIC_ROOT) == os.path.abspath(root):
-                raise ImproperlyConfigured(
-                    "The STATICFILES_DIRS setting should "
-                    "not contain the STATIC_ROOT setting")
             if (prefix, root) not in self.locations:
                 self.locations.append((prefix, root))
         for prefix, root in self.locations:
             filesystem_storage = FileSystemStorage(location=root)
             filesystem_storage.prefix = prefix
             self.storages[root] = filesystem_storage
-        super(FileSystemFinder, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
+
+    def check(self, **kwargs):
+        errors = []
+        if not isinstance(settings.STATICFILES_DIRS, (list, tuple)):
+            errors.append(Error(
+                'The STATICFILES_DIRS setting is not a tuple or list.',
+                hint='Perhaps you forgot a trailing comma?',
+                id='staticfiles.E001',
+            ))
+        for root in settings.STATICFILES_DIRS:
+            if isinstance(root, (list, tuple)):
+                prefix, root = root
+                if prefix.endswith('/'):
+                    errors.append(Error(
+                        'The prefix %r in the STATICFILES_DIRS setting must '
+                        'not end with a slash.' % prefix,
+                        id='staticfiles.E003',
+                    ))
+            if settings.STATIC_ROOT and os.path.abspath(settings.STATIC_ROOT) == os.path.abspath(root):
+                errors.append(Error(
+                    'The STATICFILES_DIRS setting should not contain the '
+                    'STATIC_ROOT setting.',
+                    id='staticfiles.E002',
+                ))
+        return errors
 
     def find(self, path, all=False):
         """
-        Looks for files in the extra locations
-        as defined in ``STATICFILES_DIRS``.
+        Look for files in the extra locations as defined in STATICFILES_DIRS.
         """
         matches = []
         for prefix, root in self.locations:
+            if root not in searched_locations:
+                searched_locations.append(root)
             matched_path = self.find_location(root, path, prefix)
             if matched_path:
                 if not all:
@@ -85,7 +110,7 @@ class FileSystemFinder(BaseFinder):
 
     def find_location(self, root, path, prefix=None):
         """
-        Finds a requested static file in a location, returning the found
+        Find a requested static file in a location and return the found
         absolute path (or ``None`` if no match).
         """
         if prefix:
@@ -110,40 +135,47 @@ class FileSystemFinder(BaseFinder):
 class AppDirectoriesFinder(BaseFinder):
     """
     A static files finder that looks in the directory of each app as
-    specified in the source_dir attribute of the given storage class.
+    specified in the source_dir attribute.
     """
-    storage_class = AppStaticStorage
+    storage_class = FileSystemStorage
+    source_dir = 'static'
 
-    def __init__(self, apps=None, *args, **kwargs):
+    def __init__(self, app_names=None, *args, **kwargs):
         # The list of apps that are handled
         self.apps = []
-        # Mapping of app module paths to storage instances
-        self.storages = SortedDict()
-        if apps is None:
-            apps = settings.INSTALLED_APPS
-        for app in apps:
-            app_storage = self.storage_class(app)
+        # Mapping of app names to storage instances
+        self.storages = OrderedDict()
+        app_configs = apps.get_app_configs()
+        if app_names:
+            app_names = set(app_names)
+            app_configs = [ac for ac in app_configs if ac.name in app_names]
+        for app_config in app_configs:
+            app_storage = self.storage_class(
+                os.path.join(app_config.path, self.source_dir))
             if os.path.isdir(app_storage.location):
-                self.storages[app] = app_storage
-                if app not in self.apps:
-                    self.apps.append(app)
-        super(AppDirectoriesFinder, self).__init__(*args, **kwargs)
+                self.storages[app_config.name] = app_storage
+                if app_config.name not in self.apps:
+                    self.apps.append(app_config.name)
+        super().__init__(*args, **kwargs)
 
     def list(self, ignore_patterns):
         """
         List all files in all app storages.
         """
-        for storage in six.itervalues(self.storages):
+        for storage in self.storages.values():
             if storage.exists(''):  # check if storage location exists
                 for path in utils.get_files(storage, ignore_patterns):
                     yield path, storage
 
     def find(self, path, all=False):
         """
-        Looks for files in the app directories.
+        Look for files in the app directories.
         """
         matches = []
         for app in self.apps:
+            app_location = self.storages[app].location
+            if app_location not in searched_locations:
+                searched_locations.append(app_location)
             match = self.find_in_app(app, path)
             if match:
                 if not all:
@@ -155,13 +187,8 @@ class AppDirectoriesFinder(BaseFinder):
         """
         Find a requested static file in an app's static locations.
         """
-        storage = self.storages.get(app, None)
+        storage = self.storages.get(app)
         if storage:
-            if storage.prefix:
-                prefix = '%s%s' % (storage.prefix, os.sep)
-                if not path.startswith(prefix):
-                    return None
-                path = path[len(prefix):]
             # only try to find a file if the source dir actually exists
             if storage.exists(path):
                 matched_path = storage.path(path)
@@ -183,20 +210,22 @@ class BaseStorageFinder(BaseFinder):
             raise ImproperlyConfigured("The staticfiles storage finder %r "
                                        "doesn't have a storage class "
                                        "assigned." % self.__class__)
-        # Make sure we have an storage instance here.
+        # Make sure we have a storage instance here.
         if not isinstance(self.storage, (Storage, LazyObject)):
             self.storage = self.storage()
-        super(BaseStorageFinder, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def find(self, path, all=False):
         """
-        Looks for files in the default file storage, if it's local.
+        Look for files in the default file storage, if it's local.
         """
         try:
             self.storage.path('')
         except NotImplementedError:
             pass
         else:
+            if self.storage.location not in searched_locations:
+                searched_locations.append(self.storage.location)
             if self.storage.exists(path):
                 match = self.storage.path(path)
                 if all:
@@ -219,7 +248,7 @@ class DefaultStorageFinder(BaseStorageFinder):
     storage = default_storage
 
     def __init__(self, *args, **kwargs):
-        super(DefaultStorageFinder, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         base_location = getattr(self.storage, 'base_location', empty)
         if not base_location:
             raise ImproperlyConfigured("The storage backend of the "
@@ -234,6 +263,7 @@ def find(path, all=False):
     If ``all`` is ``False`` (default), return the first matching
     absolute path (or ``None`` if no match). Otherwise return a list.
     """
+    searched_locations[:] = []
     matches = []
     for finder in get_finders():
         result = finder.find(path, all=all)
@@ -253,14 +283,14 @@ def get_finders():
         yield get_finder(finder_path)
 
 
-def _get_finder(import_path):
+@functools.lru_cache(maxsize=None)
+def get_finder(import_path):
     """
-    Imports the staticfiles finder class described by import_path, where
+    Import the staticfiles finder class described by import_path, where
     import_path is the full Python path to the class.
     """
-    Finder = import_by_path(import_path)
+    Finder = import_string(import_path)
     if not issubclass(Finder, BaseFinder):
         raise ImproperlyConfigured('Finder "%s" is not a subclass of "%s"' %
                                    (Finder, BaseFinder))
     return Finder()
-get_finder = memoize(_get_finder, _finders, 1)
