@@ -1,29 +1,24 @@
 "File-based cache backend"
-import errno
 import glob
 import hashlib
-import io
 import os
+import pickle
 import random
 import tempfile
 import time
 import zlib
 
 from django.core.cache.backends.base import DEFAULT_TIMEOUT, BaseCache
+from django.core.files import locks
 from django.core.files.move import file_move_safe
-from django.utils.encoding import force_bytes
-
-try:
-    from django.utils.six.moves import cPickle as pickle
-except ImportError:
-    import pickle
 
 
 class FileBasedCache(BaseCache):
     cache_suffix = '.djcache'
+    pickle_protocol = pickle.HIGHEST_PROTOCOL
 
     def __init__(self, dir, params):
-        super(FileBasedCache, self).__init__(params)
+        super().__init__(params)
         self._dir = os.path.abspath(dir)
         self._createdir()
 
@@ -36,14 +31,18 @@ class FileBasedCache(BaseCache):
     def get(self, key, default=None, version=None):
         fname = self._key_to_file(key, version)
         try:
-            with io.open(fname, 'rb') as f:
+            with open(fname, 'rb') as f:
                 if not self._is_expired(f):
                     return pickle.loads(zlib.decompress(f.read()))
-        except IOError as e:
-            if e.errno != errno.ENOENT:
-                raise
+        except FileNotFoundError:
+            pass
         return default
 
+    def _write_content(self, file, timeout, value):
+        expiry = self.get_backend_timeout(timeout)
+        file.write(pickle.dumps(expiry, self.pickle_protocol))
+        file.write(zlib.compress(pickle.dumps(value, self.pickle_protocol)))
+
     def set(self, key, value, timeout=DEFAULT_TIMEOUT, version=None):
         self._createdir()  # Cache dir can be deleted at any time.
         fname = self._key_to_file(key, version)
@@ -51,16 +50,31 @@ class FileBasedCache(BaseCache):
         fd, tmp_path = tempfile.mkstemp(dir=self._dir)
         renamed = False
         try:
-            with io.open(fd, 'wb') as f:
-                expiry = self.get_backend_timeout(timeout)
-                f.write(pickle.dumps(expiry, pickle.HIGHEST_PROTOCOL))
-                f.write(zlib.compress(pickle.dumps(value, pickle.HIGHEST_PROTOCOL)))
+            with open(fd, 'wb') as f:
+                self._write_content(f, timeout, value)
             file_move_safe(tmp_path, fname, allow_overwrite=True)
             renamed = True
         finally:
             if not renamed:
                 os.remove(tmp_path)
 
+    def touch(self, key, timeout=DEFAULT_TIMEOUT, version=None):
+        try:
+            with open(self._key_to_file(key, version), 'r+b') as f:
+                try:
+                    locks.lock(f, locks.LOCK_EX)
+                    if self._is_expired(f):
+                        return False
+                    else:
+                        previous_value = pickle.loads(zlib.decompress(f.read()))
+                        f.seek(0)
+                        self._write_content(f, timeout, previous_value)
+                        return True
+                finally:
+                    locks.unlock(f)
+        except FileNotFoundError:
+            return False
+
     def delete(self, key, version=None):
         self._delete(self._key_to_file(key, version))
 
@@ -69,22 +83,20 @@ class FileBasedCache(BaseCache):
             return
         try:
             os.remove(fname)
-        except OSError as e:
-            # ENOENT can happen if the cache file is removed (by another
-            # process) after the os.path.exists check.
-            if e.errno != errno.ENOENT:
-                raise
+        except FileNotFoundError:
+            # The file may have been removed by another process.
+            pass
 
     def has_key(self, key, version=None):
         fname = self._key_to_file(key, version)
         if os.path.exists(fname):
-            with io.open(fname, 'rb') as f:
+            with open(fname, 'rb') as f:
                 return not self._is_expired(f)
         return False
 
     def _cull(self):
         """
-        Removes random cache entries if max_entries is reached at a ratio
+        Remove random cache entries if max_entries is reached at a ratio
         of num_entries / cull_frequency. A value of 0 for CULL_FREQUENCY means
         that the entire cache will be purged.
         """
@@ -102,13 +114,15 @@ class FileBasedCache(BaseCache):
 
     def _createdir(self):
         if not os.path.exists(self._dir):
+            # Set the umask because os.makedirs() doesn't apply the "mode" argument
+            # to intermediate-level directories.
+            old_umask = os.umask(0o077)
             try:
                 os.makedirs(self._dir, 0o700)
-            except OSError as e:
-                if e.errno != errno.EEXIST:
-                    raise EnvironmentError(
-                        "Cache directory '%s' does not exist "
-                        "and could not be created'" % self._dir)
+            except FileExistsError:
+                pass
+            finally:
+                os.umask(old_umask)
 
     def _key_to_file(self, key, version=None):
         """
@@ -118,7 +132,7 @@ class FileBasedCache(BaseCache):
         key = self.make_key(key, version=version)
         self.validate_key(key)
         return os.path.join(self._dir, ''.join(
-            [hashlib.md5(force_bytes(key)).hexdigest(), self.cache_suffix]))
+            [hashlib.md5(key.encode()).hexdigest(), self.cache_suffix]))
 
     def clear(self):
         """
@@ -131,10 +145,12 @@ class FileBasedCache(BaseCache):
 
     def _is_expired(self, f):
         """
-        Takes an open cache file and determines if it has expired,
-        deletes the file if it is has passed its expiry time.
+        Take an open cache file `f` and delete it if it's expired.
         """
-        exp = pickle.load(f)
+        try:
+            exp = pickle.load(f)
+        except EOFError:
+            exp = 0  # An empty file is considered expired.
         if exp is not None and exp < time.time():
             f.close()  # On Windows a file has to be closed before deleting
             self._delete(f.name)
