@@ -1,11 +1,10 @@
 import os
 import select
 import sys
-import warnings
+import traceback
 
 from django.core.management import BaseCommand, CommandError
 from django.utils.datastructures import OrderedSet
-from django.utils.deprecation import RemovedInDjango20Warning
 
 
 class Command(BaseCommand):
@@ -20,20 +19,15 @@ class Command(BaseCommand):
 
     def add_arguments(self, parser):
         parser.add_argument(
-            '--plain', action='store_true', dest='plain',
-            help='Tells Django to use plain Python, not IPython or bpython. '
-                 'Deprecated, use the `-i python` or `--interface python` option instead.',
-        )
-        parser.add_argument(
-            '--no-startup', action='store_true', dest='no_startup',
+            '--no-startup', action='store_true',
             help='When using plain Python, ignore the PYTHONSTARTUP environment variable and ~/.pythonrc.py script.',
         )
         parser.add_argument(
-            '-i', '--interface', choices=self.shells, dest='interface',
+            '-i', '--interface', choices=self.shells,
             help='Specify an interactive interpreter interface. Available options: "ipython", "bpython", and "python"',
         )
         parser.add_argument(
-            '-c', '--command', dest='command',
+            '-c', '--command',
             help='Instead of opening an interactive shell, run a command as Django and exit.',
         )
 
@@ -60,7 +54,7 @@ class Command(BaseCommand):
             import rlcompleter
             readline.set_completer(rlcompleter.Completer(imported_objects).complete)
             # Enable tab completion on systems using libedit (e.g. macOS).
-            # These lines are copied from Lib/site.py on Python 3.4.
+            # These lines are copied from Python's Lib/site.py.
             readline_doc = getattr(readline, '__doc__', '')
             if readline_doc is not None and 'libedit' in readline_doc:
                 readline.parse_and_bind("bind ^I rl_complete")
@@ -75,21 +69,18 @@ class Command(BaseCommand):
                     continue
                 if not os.path.isfile(pythonrc):
                     continue
+                with open(pythonrc) as handle:
+                    pythonrc_code = handle.read()
+                # Match the behavior of the cpython shell where an error in
+                # PYTHONSTARTUP prints an exception and continues.
                 try:
-                    with open(pythonrc) as handle:
-                        exec(compile(handle.read(), pythonrc, 'exec'), imported_objects)
-                except NameError:
-                    pass
+                    exec(compile(pythonrc_code, pythonrc, 'exec'), imported_objects)
+                except Exception:
+                    traceback.print_exc()
+
         code.interact(local=imported_objects)
 
     def handle(self, **options):
-        if options['plain']:
-            warnings.warn(
-                "The --plain option is deprecated in favor of the -i python or --interface python option.",
-                RemovedInDjango20Warning
-            )
-            options['interface'] = 'python'
-
         # Execute the command and exit.
         if options['command']:
             exec(options['command'])
@@ -97,7 +88,7 @@ class Command(BaseCommand):
 
         # Execute stdin if it has anything to read and exit.
         # Not supported on Windows due to select.select() limitations.
-        if sys.platform != 'win32' and select.select([sys.stdin], [], [], 0)[0]:
+        if sys.platform != 'win32' and not sys.stdin.isatty() and select.select([sys.stdin], [], [], 0)[0]:
             exec(sys.stdin.read())
             return
 
