@@ -1,24 +1,29 @@
+import copy
+import threading
 import time
 import warnings
 from collections import deque
 from contextlib import contextmanager
 
+import _thread
+import pytz
+
 from django.conf import settings
+from django.core.exceptions import ImproperlyConfigured
 from django.db import DEFAULT_DB_ALIAS
 from django.db.backends import utils
+from django.db.backends.base.validation import BaseDatabaseValidation
 from django.db.backends.signals import connection_created
 from django.db.transaction import TransactionManagementError
 from django.db.utils import DatabaseError, DatabaseErrorWrapper
+from django.utils import timezone
 from django.utils.functional import cached_property
-from django.utils.six.moves import _thread as thread
 
 NO_DB_ALIAS = '__no_db__'
 
 
-class BaseDatabaseWrapper(object):
-    """
-    Represents a database connection.
-    """
+class BaseDatabaseWrapper:
+    """Represent a database connection."""
     # Mapping of Field objects to their column types.
     data_types = {}
     # Mapping of Field objects to their SQL suffix such as AUTOINCREMENT.
@@ -27,12 +32,19 @@ class BaseDatabaseWrapper(object):
     data_type_check_constraints = {}
     ops = None
     vendor = 'unknown'
+    display_name = 'unknown'
     SchemaEditorClass = None
+    # Classes instantiated in __init__().
+    client_class = None
+    creation_class = None
+    features_class = None
+    introspection_class = None
+    ops_class = None
+    validation_class = BaseDatabaseValidation
 
     queries_limit = 9000
 
-    def __init__(self, settings_dict, alias=DEFAULT_DB_ALIAS,
-                 allow_thread_sharing=False):
+    def __init__(self, settings_dict, alias=DEFAULT_DB_ALIAS):
         # Connection related attributes.
         # The underlying database connection.
         self.connection = None
@@ -68,8 +80,70 @@ class BaseDatabaseWrapper(object):
         self.errors_occurred = False
 
         # Thread-safety related attributes.
-        self.allow_thread_sharing = allow_thread_sharing
-        self._thread_ident = thread.get_ident()
+        self._thread_sharing_lock = threading.Lock()
+        self._thread_sharing_count = 0
+        self._thread_ident = _thread.get_ident()
+
+        # A list of no-argument functions to run when the transaction commits.
+        # Each entry is an (sids, func) tuple, where sids is a set of the
+        # active savepoint IDs when this function was registered.
+        self.run_on_commit = []
+
+        # Should we run the on-commit hooks the next time set_autocommit(True)
+        # is called?
+        self.run_commit_hooks_on_set_autocommit_on = False
+
+        # A stack of wrappers to be invoked around execute()/executemany()
+        # calls. Each entry is a function taking five arguments: execute, sql,
+        # params, many, and context. It's the function's responsibility to
+        # call execute(sql, params, many, context).
+        self.execute_wrappers = []
+
+        self.client = self.client_class(self)
+        self.creation = self.creation_class(self)
+        self.features = self.features_class(self)
+        self.introspection = self.introspection_class(self)
+        self.ops = self.ops_class(self)
+        self.validation = self.validation_class(self)
+
+    def ensure_timezone(self):
+        """
+        Ensure the connection's timezone is set to `self.timezone_name` and
+        return whether it changed or not.
+        """
+        return False
+
+    @cached_property
+    def timezone(self):
+        """
+        Time zone for datetimes stored as naive values in the database.
+
+        Return a tzinfo object or None.
+
+        This is only needed when time zone support is enabled and the database
+        doesn't support time zones. (When the database supports time zones,
+        the adapter handles aware datetimes so Django doesn't need to.)
+        """
+        if not settings.USE_TZ:
+            return None
+        elif self.features.supports_timezones:
+            return None
+        elif self.settings_dict['TIME_ZONE'] is None:
+            return timezone.utc
+        else:
+            return pytz.timezone(self.settings_dict['TIME_ZONE'])
+
+    @cached_property
+    def timezone_name(self):
+        """
+        Name of the time zone of the database connection.
+        """
+        if not settings.USE_TZ:
+            return settings.TIME_ZONE
+        elif self.settings_dict['TIME_ZONE'] is None:
+            return 'UTC'
+        else:
+            return self.settings_dict['TIME_ZONE']
 
     @property
     def queries_logged(self):
@@ -86,25 +160,27 @@ class BaseDatabaseWrapper(object):
     # ##### Backend-specific methods for creating connections and cursors #####
 
     def get_connection_params(self):
-        """Returns a dict of parameters suitable for get_new_connection."""
+        """Return a dict of parameters suitable for get_new_connection."""
         raise NotImplementedError('subclasses of BaseDatabaseWrapper may require a get_connection_params() method')
 
     def get_new_connection(self, conn_params):
-        """Opens a connection to the database."""
+        """Open a connection to the database."""
         raise NotImplementedError('subclasses of BaseDatabaseWrapper may require a get_new_connection() method')
 
     def init_connection_state(self):
-        """Initializes the database connection settings."""
+        """Initialize the database connection settings."""
         raise NotImplementedError('subclasses of BaseDatabaseWrapper may require an init_connection_state() method')
 
-    def create_cursor(self):
-        """Creates a cursor. Assumes that a connection is established."""
+    def create_cursor(self, name=None):
+        """Create a cursor. Assume that a connection is established."""
         raise NotImplementedError('subclasses of BaseDatabaseWrapper may require a create_cursor() method')
 
     # ##### Backend-specific methods for creating connections #####
 
     def connect(self):
-        """Connects to the database. Assumes that the connection is closed."""
+        """Connect to the database. Assume that the connection is closed."""
+        # Check for invalid configurations.
+        self.check_settings()
         # In case the previous connection was closed while in an atomic block
         self.in_atomic_block = False
         self.savepoint_ids = []
@@ -121,20 +197,42 @@ class BaseDatabaseWrapper(object):
         self.init_connection_state()
         connection_created.send(sender=self.__class__, connection=self)
 
+        self.run_on_commit = []
+
+    def check_settings(self):
+        if self.settings_dict['TIME_ZONE'] is not None:
+            if not settings.USE_TZ:
+                raise ImproperlyConfigured(
+                    "Connection '%s' cannot set TIME_ZONE because USE_TZ is "
+                    "False." % self.alias)
+            elif self.features.supports_timezones:
+                raise ImproperlyConfigured(
+                    "Connection '%s' cannot set TIME_ZONE because its engine "
+                    "handles time zones conversions natively." % self.alias)
+
     def ensure_connection(self):
-        """
-        Guarantees that a connection to the database is established.
-        """
+        """Guarantee that a connection to the database is established."""
         if self.connection is None:
             with self.wrap_database_errors:
                 self.connect()
 
     # ##### Backend-specific wrappers for PEP-249 connection methods #####
 
-    def _cursor(self):
+    def _prepare_cursor(self, cursor):
+        """
+        Validate the connection is usable and perform database cursor wrapping.
+        """
+        self.validate_thread_sharing()
+        if self.queries_logged:
+            wrapped_cursor = self.make_debug_cursor(cursor)
+        else:
+            wrapped_cursor = self.make_cursor(cursor)
+        return wrapped_cursor
+
+    def _cursor(self, name=None):
         self.ensure_connection()
         with self.wrap_database_errors:
-            return self.create_cursor()
+            return self._prepare_cursor(self.create_cursor(name))
 
     def _commit(self):
         if self.connection is not None:
@@ -154,41 +252,33 @@ class BaseDatabaseWrapper(object):
     # ##### Generic wrappers for PEP-249 connection methods #####
 
     def cursor(self):
-        """
-        Creates a cursor, opening a connection if necessary.
-        """
-        self.validate_thread_sharing()
-        if self.queries_logged:
-            cursor = self.make_debug_cursor(self._cursor())
-        else:
-            cursor = self.make_cursor(self._cursor())
-        return cursor
+        """Create a cursor, opening a connection if necessary."""
+        return self._cursor()
 
     def commit(self):
-        """
-        Commits a transaction and resets the dirty flag.
-        """
+        """Commit a transaction and reset the dirty flag."""
         self.validate_thread_sharing()
         self.validate_no_atomic_block()
         self._commit()
         # A successful commit means that the database connection works.
         self.errors_occurred = False
+        self.run_commit_hooks_on_set_autocommit_on = True
 
     def rollback(self):
-        """
-        Rolls back a transaction and resets the dirty flag.
-        """
+        """Roll back a transaction and reset the dirty flag."""
         self.validate_thread_sharing()
         self.validate_no_atomic_block()
         self._rollback()
         # A successful rollback means that the database connection works.
         self.errors_occurred = False
+        self.needs_rollback = False
+        self.run_on_commit = []
 
     def close(self):
-        """
-        Closes the connection to the database.
-        """
+        """Close the connection to the database."""
         self.validate_thread_sharing()
+        self.run_on_commit = []
+
         # Don't call validate_no_atomic_block() to avoid making it difficult
         # to get rid of a connection in an invalid state. The next connect()
         # will reset the transaction state anyway.
@@ -225,14 +315,14 @@ class BaseDatabaseWrapper(object):
 
     def savepoint(self):
         """
-        Creates a savepoint inside the current transaction. Returns an
+        Create a savepoint inside the current transaction. Return an
         identifier for the savepoint that will be used for the subsequent
-        rollback or commit. Does nothing if savepoints are not supported.
+        rollback or commit. Do nothing if savepoints are not supported.
         """
         if not self._savepoint_allowed():
             return
 
-        thread_ident = thread.get_ident()
+        thread_ident = _thread.get_ident()
         tid = str(thread_ident).replace('-', '')
 
         self.savepoint_state += 1
@@ -245,7 +335,7 @@ class BaseDatabaseWrapper(object):
 
     def savepoint_rollback(self, sid):
         """
-        Rolls back to a savepoint. Does nothing if savepoints are not supported.
+        Roll back to a savepoint. Do nothing if savepoints are not supported.
         """
         if not self._savepoint_allowed():
             return
@@ -253,9 +343,14 @@ class BaseDatabaseWrapper(object):
         self.validate_thread_sharing()
         self._savepoint_rollback(sid)
 
+        # Remove any callbacks registered while this savepoint was active.
+        self.run_on_commit = [
+            (sids, func) for (sids, func) in self.run_on_commit if sid not in sids
+        ]
+
     def savepoint_commit(self, sid):
         """
-        Releases a savepoint. Does nothing if savepoints are not supported.
+        Release a savepoint. Do nothing if savepoints are not supported.
         """
         if not self._savepoint_allowed():
             return
@@ -265,7 +360,7 @@ class BaseDatabaseWrapper(object):
 
     def clean_savepoints(self):
         """
-        Resets the counter used to generate unique savepoint ids in this thread.
+        Reset the counter used to generate unique savepoint ids in this thread.
         """
         self.savepoint_state = 0
 
@@ -280,25 +375,43 @@ class BaseDatabaseWrapper(object):
     # ##### Generic transaction management methods #####
 
     def get_autocommit(self):
-        """
-        Check the autocommit state.
-        """
+        """Get the autocommit state."""
         self.ensure_connection()
         return self.autocommit
 
-    def set_autocommit(self, autocommit):
+    def set_autocommit(self, autocommit, force_begin_transaction_with_broken_autocommit=False):
         """
         Enable or disable autocommit.
+
+        The usual way to start a transaction is to turn autocommit off.
+        SQLite does not properly start a transaction when disabling
+        autocommit. To avoid this buggy behavior and to actually enter a new
+        transaction, an explcit BEGIN is required. Using
+        force_begin_transaction_with_broken_autocommit=True will issue an
+        explicit BEGIN with SQLite. This option will be ignored for other
+        backends.
         """
         self.validate_no_atomic_block()
         self.ensure_connection()
-        self._set_autocommit(autocommit)
+
+        start_transaction_under_autocommit = (
+            force_begin_transaction_with_broken_autocommit and not autocommit and
+            hasattr(self, '_start_transaction_under_autocommit')
+        )
+
+        if start_transaction_under_autocommit:
+            self._start_transaction_under_autocommit()
+        else:
+            self._set_autocommit(autocommit)
+
         self.autocommit = autocommit
 
+        if autocommit and self.run_commit_hooks_on_set_autocommit_on:
+            self.run_and_clear_commit_hooks()
+            self.run_commit_hooks_on_set_autocommit_on = False
+
     def get_rollback(self):
-        """
-        Get the "needs rollback" flag -- for *advanced use* only.
-        """
+        """Get the "needs rollback" flag -- for *advanced use* only."""
         if not self.in_atomic_block:
             raise TransactionManagementError(
                 "The rollback flag doesn't work outside of an 'atomic' block.")
@@ -314,9 +427,7 @@ class BaseDatabaseWrapper(object):
         self.needs_rollback = rollback
 
     def validate_no_atomic_block(self):
-        """
-        Raise an error if an atomic block is active.
-        """
+        """Raise an error if an atomic block is active."""
         if self.in_atomic_block:
             raise TransactionManagementError(
                 "This is forbidden when an 'atomic' block is active.")
@@ -332,7 +443,7 @@ class BaseDatabaseWrapper(object):
     @contextmanager
     def constraint_checks_disabled(self):
         """
-        Context manager that disables foreign key constraint checking.
+        Disable foreign key constraint checking.
         """
         disabled = self.disable_constraint_checking()
         try:
@@ -368,9 +479,9 @@ class BaseDatabaseWrapper(object):
 
     def is_usable(self):
         """
-        Tests if the database connection is usable.
+        Test if the database connection is usable.
 
-        This function may assume that self.connection is not None.
+        This method may assume that self.connection is not None.
 
         Actual implementations should take care not to raise exceptions
         as that may prevent Django from recycling unusable connections.
@@ -380,7 +491,7 @@ class BaseDatabaseWrapper(object):
 
     def close_if_unusable_or_obsolete(self):
         """
-        Closes the current connection if unrecoverable errors have occurred,
+        Close the current connection if unrecoverable errors have occurred
         or if it outlived its maximum age.
         """
         if self.connection is not None:
@@ -405,20 +516,36 @@ class BaseDatabaseWrapper(object):
 
     # ##### Thread safety handling #####
 
+    @property
+    def allow_thread_sharing(self):
+        with self._thread_sharing_lock:
+            return self._thread_sharing_count > 0
+
+    def inc_thread_sharing(self):
+        with self._thread_sharing_lock:
+            self._thread_sharing_count += 1
+
+    def dec_thread_sharing(self):
+        with self._thread_sharing_lock:
+            if self._thread_sharing_count <= 0:
+                raise RuntimeError('Cannot decrement the thread sharing count below zero.')
+            self._thread_sharing_count -= 1
+
     def validate_thread_sharing(self):
         """
-        Validates that the connection isn't accessed by another thread than the
+        Validate that the connection isn't accessed by another thread than the
         one which originally created it, unless the connection was explicitly
-        authorized to be shared between threads (via the `allow_thread_sharing`
-        property). Raises an exception if the validation fails.
+        authorized to be shared between threads (via the `inc_thread_sharing()`
+        method). Raise an exception if the validation fails.
         """
-        if not (self.allow_thread_sharing
-                or self._thread_ident == thread.get_ident()):
-            raise DatabaseError("DatabaseWrapper objects created in a "
+        if not (self.allow_thread_sharing or self._thread_ident == _thread.get_ident()):
+            raise DatabaseError(
+                "DatabaseWrapper objects created in a "
                 "thread can only be used in that same thread. The object "
                 "with alias '%s' was created in thread id %s and this is "
                 "thread id %s."
-                % (self.alias, self._thread_ident, thread.get_ident()))
+                % (self.alias, self._thread_ident, _thread.get_ident())
+            )
 
     # ##### Miscellaneous #####
 
@@ -437,16 +564,19 @@ class BaseDatabaseWrapper(object):
         """
         return DatabaseErrorWrapper(self)
 
+    def chunked_cursor(self):
+        """
+        Return a cursor that tries to avoid caching in the database (if
+        supported by the database), otherwise return a regular cursor.
+        """
+        return self.cursor()
+
     def make_debug_cursor(self, cursor):
-        """
-        Creates a cursor that logs all queries in self.queries_log.
-        """
+        """Create a cursor that logs all queries in self.queries_log."""
         return utils.CursorDebugWrapper(cursor, self)
 
     def make_cursor(self, cursor):
-        """
-        Creates a cursor without debug logging.
-        """
+        """Create a cursor without debug logging."""
         return utils.CursorWrapper(cursor, self)
 
     @contextmanager
@@ -456,48 +586,74 @@ class BaseDatabaseWrapper(object):
         if it opened one, closes it to avoid leaving a dangling connection.
         This is useful for operations outside of the request-response cycle.
 
-        Provides a cursor: with self.temporary_connection() as cursor: ...
+        Provide a cursor: with self.temporary_connection() as cursor: ...
         """
         must_close = self.connection is None
-        cursor = self.cursor()
         try:
-            yield cursor
+            with self.cursor() as cursor:
+                yield cursor
         finally:
-            cursor.close()
             if must_close:
                 self.close()
 
-    @cached_property
+    @property
     def _nodb_connection(self):
         """
-        Alternative connection to be used when there is no need to access
-        the main database, specifically for test db creation/deletion.
+        Return an alternative connection to be used when there is no need to
+        access the main database, specifically for test db creation/deletion.
         This also prevents the production database from being exposed to
         potential child threads while (or after) the test database is destroyed.
         Refs #10868, #17786, #16969.
         """
-        settings_dict = self.settings_dict.copy()
-        settings_dict['NAME'] = None
-        nodb_connection = self.__class__(
-            settings_dict,
-            alias=NO_DB_ALIAS,
-            allow_thread_sharing=False)
-        return nodb_connection
-
-    def _start_transaction_under_autocommit(self):
-        """
-        Only required when autocommits_when_autocommit_is_off = True.
-        """
-        raise NotImplementedError(
-            'subclasses of BaseDatabaseWrapper may require a '
-            '_start_transaction_under_autocommit() method'
-        )
+        return self.__class__({**self.settings_dict, 'NAME': None}, alias=NO_DB_ALIAS)
 
     def schema_editor(self, *args, **kwargs):
         """
-        Returns a new instance of this backend's SchemaEditor.
+        Return a new instance of this backend's SchemaEditor.
         """
         if self.SchemaEditorClass is None:
             raise NotImplementedError(
                 'The SchemaEditorClass attribute of this database wrapper is still None')
         return self.SchemaEditorClass(self, *args, **kwargs)
+
+    def on_commit(self, func):
+        if self.in_atomic_block:
+            # Transaction in progress; save for execution on commit.
+            self.run_on_commit.append((set(self.savepoint_ids), func))
+        elif not self.get_autocommit():
+            raise TransactionManagementError('on_commit() cannot be used in manual transaction management')
+        else:
+            # No transaction in progress and in autocommit mode; execute
+            # immediately.
+            func()
+
+    def run_and_clear_commit_hooks(self):
+        self.validate_no_atomic_block()
+        current_run_on_commit = self.run_on_commit
+        self.run_on_commit = []
+        while current_run_on_commit:
+            sids, func = current_run_on_commit.pop(0)
+            func()
+
+    @contextmanager
+    def execute_wrapper(self, wrapper):
+        """
+        Return a context manager under which the wrapper is applied to suitable
+        database query executions.
+        """
+        self.execute_wrappers.append(wrapper)
+        try:
+            yield
+        finally:
+            self.execute_wrappers.pop()
+
+    def copy(self, alias=None):
+        """
+        Return a copy of this connection.
+
+        For tests that require two connections to the same database.
+        """
+        settings_dict = copy.deepcopy(self.settings_dict)
+        if alias is None:
+            alias = self.alias
+        return type(self)(settings_dict, alias)
