@@ -6,8 +6,8 @@ import os
 import pickle
 import textwrap
 import unittest
-import warnings
 from importlib import import_module
+from io import StringIO
 
 from django.core.management import call_command
 from django.db import connections
@@ -17,8 +17,6 @@ from django.test.utils import (
     teardown_databases as _teardown_databases, teardown_test_environment,
 )
 from django.utils.datastructures import OrderedSet
-from django.utils.deprecation import RemovedInDjango21Warning
-from django.utils.six import StringIO
 
 try:
     import tblib.pickling_support
@@ -30,16 +28,16 @@ class DebugSQLTextTestResult(unittest.TextTestResult):
     def __init__(self, stream, descriptions, verbosity):
         self.logger = logging.getLogger('django.db.backends')
         self.logger.setLevel(logging.DEBUG)
-        super(DebugSQLTextTestResult, self).__init__(stream, descriptions, verbosity)
+        super().__init__(stream, descriptions, verbosity)
 
     def startTest(self, test):
         self.debug_sql_stream = StringIO()
         self.handler = logging.StreamHandler(self.debug_sql_stream)
         self.logger.addHandler(self.handler)
-        super(DebugSQLTextTestResult, self).startTest(test)
+        super().startTest(test)
 
     def stopTest(self, test):
-        super(DebugSQLTextTestResult, self).stopTest(test)
+        super().stopTest(test)
         self.logger.removeHandler(self.handler)
         if self.showAll:
             self.debug_sql_stream.seek(0)
@@ -47,26 +45,33 @@ class DebugSQLTextTestResult(unittest.TextTestResult):
             self.stream.writeln(self.separator2)
 
     def addError(self, test, err):
-        super(DebugSQLTextTestResult, self).addError(test, err)
+        super().addError(test, err)
         self.debug_sql_stream.seek(0)
         self.errors[-1] = self.errors[-1] + (self.debug_sql_stream.read(),)
 
     def addFailure(self, test, err):
-        super(DebugSQLTextTestResult, self).addFailure(test, err)
+        super().addFailure(test, err)
         self.debug_sql_stream.seek(0)
         self.failures[-1] = self.failures[-1] + (self.debug_sql_stream.read(),)
 
+    def addSubTest(self, test, subtest, err):
+        super().addSubTest(test, subtest, err)
+        if err is not None:
+            self.debug_sql_stream.seek(0)
+            errors = self.failures if issubclass(err[0], test.failureException) else self.errors
+            errors[-1] = errors[-1] + (self.debug_sql_stream.read(),)
+
     def printErrorList(self, flavour, errors):
         for test, err, sql_debug in errors:
             self.stream.writeln(self.separator1)
             self.stream.writeln("%s: %s" % (flavour, self.getDescription(test)))
             self.stream.writeln(self.separator2)
-            self.stream.writeln("%s" % err)
+            self.stream.writeln(err)
             self.stream.writeln(self.separator2)
-            self.stream.writeln("%s" % sql_debug)
+            self.stream.writeln(sql_debug)
 
 
-class RemoteTestResult(object):
+class RemoteTestResult:
     """
     Record information about which tests have succeeded and which have failed.
 
@@ -230,7 +235,7 @@ failure and get a correct traceback.
         self.stop_if_failfast()
 
 
-class RemoteTestRunner(object):
+class RemoteTestRunner:
     """
     Run tests and record everything but don't display anything.
 
@@ -253,13 +258,10 @@ class RemoteTestRunner(object):
 
 
 def default_test_processes():
-    """
-    Default number of test processes when using the --parallel option.
-    """
+    """Default number of test processes when using the --parallel option."""
     # The current implementation of the parallel test runner requires
     # multiprocessing to start subprocesses with fork().
-    # On Python 3.4+: if multiprocessing.get_start_method() != 'fork':
-    if not hasattr(os, 'fork'):
+    if multiprocessing.get_start_method() != 'fork':
         return 1
     try:
         return int(os.environ['DJANGO_TEST_PROCESSES'])
@@ -286,7 +288,7 @@ def _init_worker(counter):
 
     for alias in connections:
         connection = connections[alias]
-        settings_dict = connection.creation.get_test_db_clone_settings(_worker_id)
+        settings_dict = connection.creation.get_test_db_clone_settings(str(_worker_id))
         # connection.settings_dict must be updated in place for changes to be
         # reflected in django.db.connections. If the following line assigned
         # connection.settings_dict = settings_dict, new threads would connect
@@ -333,7 +335,7 @@ class ParallelTestSuite(unittest.TestSuite):
         self.subsuites = partition_suite_by_case(suite)
         self.processes = processes
         self.failfast = failfast
-        super(ParallelTestSuite, self).__init__()
+        super().__init__()
 
     def run(self, result):
         """
@@ -348,13 +350,14 @@ class ParallelTestSuite(unittest.TestSuite):
         - make tracebacks picklable with tblib, if available
 
         Even with tblib, errors may still occur for dynamically created
-        exception classes such Model.DoesNotExist which cannot be unpickled.
+        exception classes which cannot be unpickled.
         """
         counter = multiprocessing.Value(ctypes.c_int, 0)
         pool = multiprocessing.Pool(
             processes=self.processes,
             initializer=self.init_worker.__func__,
-            initargs=[counter])
+            initargs=[counter],
+        )
         args = [
             (self.runner_class, index, subsuite, self.failfast)
             for index, subsuite in enumerate(self.subsuites)
@@ -388,11 +391,12 @@ class ParallelTestSuite(unittest.TestSuite):
 
         return result
 
+    def __iter__(self):
+        return iter(self.subsuites)
 
-class DiscoverRunner(object):
-    """
-    A Django test runner that uses unittest2 test discovery.
-    """
+
+class DiscoverRunner:
+    """A Django test runner that uses unittest2 test discovery."""
 
     test_suite = unittest.TestSuite
     parallel_test_suite = ParallelTestSuite
@@ -421,31 +425,31 @@ class DiscoverRunner(object):
     @classmethod
     def add_arguments(cls, parser):
         parser.add_argument(
-            '-t', '--top-level-directory', action='store', dest='top_level', default=None,
+            '-t', '--top-level-directory', dest='top_level',
             help='Top level of project for unittest discovery.',
         )
         parser.add_argument(
-            '-p', '--pattern', action='store', dest='pattern', default="test*.py",
+            '-p', '--pattern', default="test*.py",
             help='The test matching pattern. Defaults to test*.py.',
         )
         parser.add_argument(
-            '-k', '--keepdb', action='store_true', dest='keepdb', default=False,
+            '-k', '--keepdb', action='store_true',
             help='Preserves the test DB between runs.'
         )
         parser.add_argument(
-            '-r', '--reverse', action='store_true', dest='reverse', default=False,
+            '-r', '--reverse', action='store_true',
             help='Reverses test cases order.',
         )
         parser.add_argument(
-            '--debug-mode', action='store_true', dest='debug_mode', default=False,
+            '--debug-mode', action='store_true',
             help='Sets settings.DEBUG to True.',
         )
         parser.add_argument(
-            '-d', '--debug-sql', action='store_true', dest='debug_sql', default=False,
+            '-d', '--debug-sql', action='store_true',
             help='Prints logged SQL queries on failure.',
         )
         parser.add_argument(
-            '--parallel', dest='parallel', nargs='?', default=1, type=int,
+            '--parallel', nargs='?', default=1, type=int,
             const=default_test_processes(), metavar='N',
             help='Run tests using up to N parallel processes.',
         )
@@ -522,6 +526,11 @@ class DiscoverRunner(object):
             suite.addTest(test)
 
         if self.tags or self.exclude_tags:
+            if self.verbosity >= 2:
+                if self.tags:
+                    print('Including test tag(s): %s.' % ', '.join(sorted(self.tags)))
+                if self.exclude_tags:
+                    print('Excluding test tag(s): %s.' % ', '.join(sorted(self.exclude_tags)))
             suite = filter_tests_by_tags(suite, self.tags, self.exclude_tags)
         suite = reorder_suite(suite, self.reorder_by, self.reverse)
 
@@ -531,8 +540,7 @@ class DiscoverRunner(object):
             # Since tests are distributed across processes on a per-TestCase
             # basis, there's no need for more processes than TestCases.
             parallel_units = len(parallel_suite.subsuites)
-            if self.parallel > parallel_units:
-                self.parallel = parallel_units
+            self.parallel = min(self.parallel, parallel_units)
 
             # If there's only one TestCase, parallelization isn't needed.
             if self.parallel > 1:
@@ -567,9 +575,7 @@ class DiscoverRunner(object):
         return runner.run(suite)
 
     def teardown_databases(self, old_config, **kwargs):
-        """
-        Destroys all the non-mirror databases.
-        """
+        """Destroy all the non-mirror databases."""
         _teardown_databases(
             old_config,
             verbosity=self.verbosity,
@@ -584,6 +590,27 @@ class DiscoverRunner(object):
     def suite_result(self, suite, result, **kwargs):
         return len(result.failures) + len(result.errors)
 
+    def _get_databases(self, suite):
+        databases = set()
+        for test in suite:
+            if isinstance(test, unittest.TestCase):
+                test_databases = getattr(test, 'databases', None)
+                if test_databases == '__all__':
+                    return set(connections)
+                if test_databases:
+                    databases.update(test_databases)
+            else:
+                databases.update(self._get_databases(test))
+        return databases
+
+    def get_databases(self, suite):
+        databases = self._get_databases(suite)
+        if self.verbosity >= 2:
+            unused_databases = [alias for alias in connections if alias not in databases]
+            if unused_databases:
+                print('Skipping setup of unused database(s): %s.' % ', '.join(sorted(unused_databases)))
+        return databases
+
     def run_tests(self, test_labels, extra_tests=None, **kwargs):
         """
         Run the unit tests for all the test labels in the provided list.
@@ -594,21 +621,34 @@ class DiscoverRunner(object):
         A list of 'extra' tests may also be provided; these tests
         will be added to the test suite.
 
-        Returns the number of tests that failed.
+        Return the number of tests that failed.
         """
         self.setup_test_environment()
         suite = self.build_suite(test_labels, extra_tests)
-        old_config = self.setup_databases()
-        self.run_checks()
-        result = self.run_suite(suite)
-        self.teardown_databases(old_config)
-        self.teardown_test_environment()
+        databases = self.get_databases(suite)
+        old_config = self.setup_databases(aliases=databases)
+        run_failed = False
+        try:
+            self.run_checks()
+            result = self.run_suite(suite)
+        except Exception:
+            run_failed = True
+            raise
+        finally:
+            try:
+                self.teardown_databases(old_config)
+                self.teardown_test_environment()
+            except Exception:
+                # Silence teardown exceptions if an exception was raised during
+                # runs to avoid shadowing it.
+                if not run_failed:
+                    raise
         return self.suite_result(suite, result)
 
 
 def is_discoverable(label):
     """
-    Check if a test label points to a python package or file directory.
+    Check if a test label points to a Python package or file directory.
 
     Relative labels like "." and ".." are seen as directories.
     """
@@ -624,15 +664,15 @@ def is_discoverable(label):
 
 def reorder_suite(suite, classes, reverse=False):
     """
-    Reorders a test suite by test type.
+    Reorder a test suite by test type.
 
     `classes` is a sequence of types
 
     All tests of type classes[0] are placed first, then tests of type
     classes[1], etc. Tests with no match in classes are placed last.
 
-    If `reverse` is True, tests within classes are sorted in opposite order,
-    but test classes are not reversed.
+    If `reverse` is True, sort tests within classes in opposite order but
+    don't reverse test classes.
     """
     class_count = len(classes)
     suite_class = type(suite)
@@ -646,7 +686,7 @@ def reorder_suite(suite, classes, reverse=False):
 
 def partition_suite_by_type(suite, classes, bins, reverse=False):
     """
-    Partitions a test suite by test type. Also prevents duplicated tests.
+    Partition a test suite by test type. Also prevent duplicated tests.
 
     classes is a sequence of types
     bins is a sequence of TestSuites, one more than classes
@@ -671,9 +711,7 @@ def partition_suite_by_type(suite, classes, bins, reverse=False):
 
 
 def partition_suite_by_case(suite):
-    """
-    Partitions a test suite by test case, preserving the order of tests.
-    """
+    """Partition a test suite by test case, preserving the order of tests."""
     groups = []
     suite_class = type(suite)
     for test_type, test_group in itertools.groupby(suite, type):
@@ -685,16 +723,6 @@ def partition_suite_by_case(suite):
     return groups
 
 
-def setup_databases(*args, **kwargs):
-    warnings.warn(
-        '`django.test.runner.setup_databases()` has moved to '
-        '`django.test.utils.setup_databases()`.',
-        RemovedInDjango21Warning,
-        stacklevel=2,
-    )
-    return _setup_databases(*args, **kwargs)
-
-
 def filter_tests_by_tags(suite, tags, exclude_tags):
     suite_class = type(suite)
     filtered_suite = suite_class()
