@@ -1,11 +1,9 @@
 """JsLex: a lexer for Javascript"""
 # Originally from https://bitbucket.org/ned/jslex
-from __future__ import unicode_literals
-
 import re
 
 
-class Tok(object):
+class Tok:
     """
     A specification for a token class.
     """
@@ -29,7 +27,7 @@ def literals(choices, prefix="", suffix=""):
     return "|".join(prefix + re.escape(c) + suffix for c in choices.split())
 
 
-class Lexer(object):
+class Lexer:
     """
     A generic multi-state regex-based lexer.
     """
@@ -52,7 +50,7 @@ class Lexer(object):
         """
         Lexically analyze `text`.
 
-        Yields pairs (`name`, `tokentext`).
+        Yield pairs (`name`, `tokentext`).
         """
         end = len(text)
         state = self.state
@@ -181,7 +179,7 @@ class JsLexer(Lexer):
     }
 
     def __init__(self):
-        super(JsLexer, self).__init__(self.states, 'reg')
+        super().__init__(self.states, 'reg')
 
 
 def prepare_js_for_gettext(js):
