@@ -1,16 +1,13 @@
-from __future__ import unicode_literals
-
 from decimal import Decimal
 
 from django.conf import settings
-from django.utils import six
 from django.utils.safestring import mark_safe
 
 
 def format(number, decimal_sep, decimal_pos=None, grouping=0, thousand_sep='',
-           force_grouping=False):
+           force_grouping=False, use_l10n=None):
     """
-    Gets a number (as a number or string), and returns it as a string,
+    Get a number (as a number or string), and return it as a string,
     using formats defined as arguments:
 
     * decimal_sep: Decimal separator symbol (for example ".")
@@ -21,12 +18,12 @@ def format(number, decimal_sep, decimal_pos=None, grouping=0, thousand_sep='',
         module in locale.localeconv() LC_NUMERIC grouping (e.g. (3, 2, 0)).
     * thousand_sep: Thousand separator symbol (for example ",")
     """
-    use_grouping = settings.USE_L10N and settings.USE_THOUSAND_SEPARATOR
+    use_grouping = (use_l10n or (use_l10n is None and settings.USE_L10N)) and settings.USE_THOUSAND_SEPARATOR
     use_grouping = use_grouping or force_grouping
     use_grouping = use_grouping and grouping != 0
     # Make the common case fast
     if isinstance(number, int) and not use_grouping and not decimal_pos:
-        return mark_safe(six.text_type(number))
+        return mark_safe(number)
     # sign
     sign = ''
     if isinstance(number, Decimal):
@@ -39,13 +36,13 @@ def format(number, decimal_sep, decimal_pos=None, grouping=0, thousand_sep='',
             # Format the coefficient.
             coefficient = format(
                 coefficient, decimal_sep, decimal_pos, grouping,
-                thousand_sep, force_grouping,
+                thousand_sep, force_grouping, use_l10n,
             )
             return '{}e{}'.format(coefficient, exponent)
         else:
             str_number = '{:f}'.format(number)
     else:
-        str_number = six.text_type(number)
+        str_number = str(number)
     if str_number[0] == '-':
         sign = '-'
         str_number = str_number[1:]
@@ -58,8 +55,7 @@ def format(number, decimal_sep, decimal_pos=None, grouping=0, thousand_sep='',
         int_part, dec_part = str_number, ''
     if decimal_pos is not None:
         dec_part = dec_part + ('0' * (decimal_pos - len(dec_part)))
-    if dec_part:
-        dec_part = decimal_sep + dec_part
+    dec_part = dec_part and decimal_sep + dec_part
     # grouping
     if use_grouping:
         try:
