@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import datetime
 import os
 import tempfile
@@ -14,10 +11,8 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.core.files.storage import FileSystemStorage
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Section(models.Model):
     """
     A simple section that links to articles, to test linking to related items
@@ -36,7 +31,6 @@ class Section(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Article(models.Model):
     """
     A simple article to test admin views. Test backwards compatibility.
@@ -62,7 +56,6 @@ class Article(models.Model):
     model_year_reversed.short_description = ''
 
 
-@python_2_unicode_compatible
 class Book(models.Model):
     """
     A simple book that has chapters.
@@ -73,7 +66,6 @@ class Book(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Promo(models.Model):
     name = models.CharField(max_length=100, verbose_name='¿Name?')
     book = models.ForeignKey(Book, models.CASCADE)
@@ -83,21 +75,19 @@ class Promo(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Chapter(models.Model):
     title = models.CharField(max_length=100, verbose_name='¿Title?')
     content = models.TextField()
     book = models.ForeignKey(Book, models.CASCADE)
 
-    def __str__(self):
-        return self.title
-
     class Meta:
         # Use a utf-8 bytestring to ensure it works (see #11710)
         verbose_name = '¿Chapter?'
 
+    def __str__(self):
+        return self.title
+
 
-@python_2_unicode_compatible
 class ChapterXtra1(models.Model):
     chap = models.OneToOneField(Chapter, models.CASCADE, verbose_name='¿Chap?')
     xtra = models.CharField(max_length=100, verbose_name='¿Xtra?')
@@ -107,7 +97,6 @@ class ChapterXtra1(models.Model):
         return '¿Xtra1: %s' % self.xtra
 
 
-@python_2_unicode_compatible
 class ChapterXtra2(models.Model):
     chap = models.OneToOneField(Chapter, models.CASCADE, verbose_name='¿Chap?')
     xtra = models.CharField(max_length=100, verbose_name='¿Xtra?')
@@ -125,7 +114,6 @@ class CustomArticle(models.Model):
     date = models.DateTimeField()
 
 
-@python_2_unicode_compatible
 class ModelWithStringPrimaryKey(models.Model):
     string_pk = models.CharField(max_length=255, primary_key=True)
 
@@ -136,7 +124,6 @@ class ModelWithStringPrimaryKey(models.Model):
         return '/dummy/%s/' % self.string_pk
 
 
-@python_2_unicode_compatible
 class Color(models.Model):
     value = models.CharField(max_length=10)
     warm = models.BooleanField(default=False)
@@ -151,7 +138,6 @@ class Color2(Color):
         proxy = True
 
 
-@python_2_unicode_compatible
 class Thing(models.Model):
     title = models.CharField(max_length=20)
     color = models.ForeignKey(Color, models.CASCADE, limit_choices_to={'warm': True})
@@ -161,7 +147,6 @@ class Thing(models.Model):
         return self.title
 
 
-@python_2_unicode_compatible
 class Actor(models.Model):
     name = models.CharField(max_length=50)
     age = models.IntegerField()
@@ -171,7 +156,6 @@ class Actor(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Inquisition(models.Model):
     expected = models.BooleanField(default=False)
     leader = models.ForeignKey(Actor, models.CASCADE)
@@ -181,7 +165,6 @@ class Inquisition(models.Model):
         return "by %s from %s" % (self.leader, self.country)
 
 
-@python_2_unicode_compatible
 class Sketch(models.Model):
     title = models.CharField(max_length=100)
     inquisition = models.ForeignKey(
@@ -218,7 +201,6 @@ def today_callable_q():
     return models.Q(last_action__gte=datetime.datetime.today())
 
 
-@python_2_unicode_compatible
 class Character(models.Model):
     username = models.CharField(max_length=100)
     last_action = models.DateTimeField()
@@ -227,7 +209,6 @@ class Character(models.Model):
         return self.username
 
 
-@python_2_unicode_compatible
 class StumpJoke(models.Model):
     variation = models.CharField(max_length=100)
     most_recently_fooled = models.ForeignKey(
@@ -253,7 +234,6 @@ class Fabric(models.Model):
     surface = models.CharField(max_length=20, choices=NG_CHOICES)
 
 
-@python_2_unicode_compatible
 class Person(models.Model):
     GENDER_CHOICES = (
         (1, "Male"),
@@ -268,7 +248,6 @@ class Person(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Persona(models.Model):
     """
     A simple persona associated with accounts, to test inlining of related
@@ -280,7 +259,6 @@ class Persona(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Account(models.Model):
     """
     A simple, generic account encapsulating the information shared by all
@@ -304,7 +282,6 @@ class BarAccount(Account):
     servicename = 'bar'
 
 
-@python_2_unicode_compatible
 class Subscriber(models.Model):
     name = models.CharField(blank=False, max_length=80)
     email = models.EmailField(blank=False, max_length=175)
@@ -354,7 +331,6 @@ class Child(models.Model):
             raise ValidationError('invalid')
 
 
-@python_2_unicode_compatible
 class EmptyModel(models.Model):
     def __str__(self):
         return "Primary key = %s" % self.id
@@ -438,7 +414,6 @@ class FancyDoodad(Doodad):
     expensive = models.BooleanField(default=True)
 
 
-@python_2_unicode_compatible
 class Category(models.Model):
     collector = models.ForeignKey(Collector, models.CASCADE)
     order = models.PositiveIntegerField()
@@ -481,7 +456,7 @@ class Post(models.Model):
         default=datetime.date.today,
         help_text="Some help text for the date (with unicode ŠĐĆŽćžšđ)"
     )
-    public = models.NullBooleanField()
+    public = models.BooleanField(null=True, blank=True)
 
     def awesomeness_level(self):
         return "Very awesome."
@@ -494,7 +469,6 @@ class FieldOverridePost(Post):
         proxy = True
 
 
-@python_2_unicode_compatible
 class Gadget(models.Model):
     name = models.CharField(max_length=100)
 
@@ -502,7 +476,6 @@ class Gadget(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Villain(models.Model):
     name = models.CharField(max_length=100)
 
@@ -514,7 +487,6 @@ class SuperVillain(Villain):
     pass
 
 
-@python_2_unicode_compatible
 class FunkyTag(models.Model):
     "Because we all know there's only one real use case for GFKs."
     name = models.CharField(max_length=25)
@@ -526,7 +498,6 @@ class FunkyTag(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Plot(models.Model):
     name = models.CharField(max_length=100)
     team_leader = models.ForeignKey(Villain, models.CASCADE, related_name='lead_plots')
@@ -537,7 +508,6 @@ class Plot(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class PlotDetails(models.Model):
     details = models.CharField(max_length=100)
     plot = models.OneToOneField(Plot, models.CASCADE, null=True, blank=True)
@@ -551,7 +521,6 @@ class PlotProxy(Plot):
         proxy = True
 
 
-@python_2_unicode_compatible
 class SecretHideout(models.Model):
     """ Secret! Not registered with the admin! """
     location = models.CharField(max_length=100)
@@ -561,7 +530,6 @@ class SecretHideout(models.Model):
         return self.location
 
 
-@python_2_unicode_compatible
 class SuperSecretHideout(models.Model):
     """ Secret! Not registered with the admin! """
     location = models.CharField(max_length=100)
@@ -571,7 +539,6 @@ class SuperSecretHideout(models.Model):
         return self.location
 
 
-@python_2_unicode_compatible
 class Bookmark(models.Model):
     name = models.CharField(max_length=60)
     tag = GenericRelation(FunkyTag, related_query_name='bookmark')
@@ -580,7 +547,6 @@ class Bookmark(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class CyclicOne(models.Model):
     name = models.CharField(max_length=25)
     two = models.ForeignKey('CyclicTwo', models.CASCADE)
@@ -589,7 +555,6 @@ class CyclicOne(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class CyclicTwo(models.Model):
     name = models.CharField(max_length=25)
     one = models.ForeignKey(CyclicOne, models.CASCADE)
@@ -598,7 +563,6 @@ class CyclicTwo(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Topping(models.Model):
     name = models.CharField(max_length=20)
 
@@ -618,6 +582,14 @@ class ReadablePizza(Pizza):
         proxy = True
 
 
+# No default permissions are created for this model and both name and toppings
+# are readonly for this model's admin.
+class ReadOnlyPizza(Pizza):
+    class Meta:
+        proxy = True
+        default_permissions = ()
+
+
 class Album(models.Model):
     owner = models.ForeignKey(User, models.SET_NULL, null=True, blank=True)
     title = models.CharField(max_length=30)
@@ -635,9 +607,13 @@ class WorkHour(models.Model):
 class Question(models.Model):
     question = models.CharField(max_length=20)
     posted = models.DateField(default=datetime.date.today)
+    expires = models.DateTimeField(null=True, blank=True)
+    related_questions = models.ManyToManyField('self')
+
+    def __str__(self):
+        return self.question
 
 
-@python_2_unicode_compatible
 class Answer(models.Model):
     question = models.ForeignKey(Question, models.PROTECT)
     answer = models.CharField(max_length=20)
@@ -646,24 +622,26 @@ class Answer(models.Model):
         return self.answer
 
 
+class Answer2(Answer):
+    class Meta:
+        proxy = True
+
+
 class Reservation(models.Model):
     start_date = models.DateTimeField()
     price = models.IntegerField()
 
 
-DRIVER_CHOICES = (
-    ('bill', 'Bill G'),
-    ('steve', 'Steve J'),
-)
-
-RESTAURANT_CHOICES = (
-    ('indian', 'A Taste of India'),
-    ('thai', 'Thai Pography'),
-    ('pizza', 'Pizza Mama'),
-)
-
-
 class FoodDelivery(models.Model):
+    DRIVER_CHOICES = (
+        ('bill', 'Bill G'),
+        ('steve', 'Steve J'),
+    )
+    RESTAURANT_CHOICES = (
+        ('indian', 'A Taste of India'),
+        ('thai', 'Thai Pography'),
+        ('pizza', 'Pizza Mama'),
+    )
     reference = models.CharField(max_length=100)
     driver = models.CharField(max_length=100, choices=DRIVER_CHOICES, blank=True)
     restaurant = models.CharField(max_length=100, choices=RESTAURANT_CHOICES, blank=True)
@@ -672,7 +650,6 @@ class FoodDelivery(models.Model):
         unique_together = (("driver", "restaurant"),)
 
 
-@python_2_unicode_compatible
 class CoverLetter(models.Model):
     author = models.CharField(max_length=30)
     date_written = models.DateField(null=True, blank=True)
@@ -691,7 +668,6 @@ class ShortMessage(models.Model):
     timestamp = models.DateTimeField(null=True, blank=True)
 
 
-@python_2_unicode_compatible
 class Telegram(models.Model):
     title = models.CharField(max_length=30)
     date_sent = models.DateField(null=True, blank=True)
@@ -713,7 +689,7 @@ class OtherStory(models.Model):
 class ComplexSortedPerson(models.Model):
     name = models.CharField(max_length=100)
     age = models.PositiveIntegerField()
-    is_employee = models.NullBooleanField()
+    is_employee = models.BooleanField(null=True)
 
 
 class PluggableSearchPerson(models.Model):
@@ -757,7 +733,6 @@ class AdminOrderedCallable(models.Model):
     stuff = models.CharField(max_length=200)
 
 
-@python_2_unicode_compatible
 class Report(models.Model):
     title = models.CharField(max_length=100)
 
@@ -780,6 +755,8 @@ class MainPrepopulated(models.Model):
 class RelatedPrepopulated(models.Model):
     parent = models.ForeignKey(MainPrepopulated, models.CASCADE)
     name = models.CharField(max_length=75)
+    fk = models.ForeignKey('self', models.CASCADE, blank=True, null=True)
+    m2m = models.ManyToManyField('self', blank=True)
     pubdate = models.DateField()
     status = models.CharField(
         max_length=20,
@@ -854,7 +831,7 @@ class DependentChild(models.Model):
 
 class _Manager(models.Manager):
     def get_queryset(self):
-        return super(_Manager, self).get_queryset().filter(pk__gt=1)
+        return super().get_queryset().filter(pk__gt=1)
 
 
 class FilteredManager(models.Model):
@@ -940,7 +917,6 @@ class InlineReference(models.Model):
     )
 
 
-# Models for #23604 and #23915
 class Recipe(models.Model):
     rname = models.CharField(max_length=20, unique=True)
 
@@ -991,3 +967,18 @@ class ParentWithUUIDPK(models.Model):
 
 class RelatedWithUUIDPKModel(models.Model):
     parent = models.ForeignKey(ParentWithUUIDPK, on_delete=models.SET_NULL, null=True, blank=True)
+
+
+class Author(models.Model):
+    pass
+
+
+class Authorship(models.Model):
+    book = models.ForeignKey(Book, models.CASCADE)
+    author = models.ForeignKey(Author, models.CASCADE)
+
+
+class UserProxy(User):
+    """Proxy a model with a different app_label."""
+    class Meta:
+        proxy = True
