@@ -22,6 +22,7 @@ from django.utils import translation
 from .models import (
     Advisor, Album, Band, Bee, Car, Company, Event, Honeycomb, Individual,
     Inventory, Member, MyFileField, Profile, School, Student,
+    UnsafeLimitChoicesTo,
 )
 from .widgetadmin import site as widget_admin_site
 
@@ -586,6 +587,19 @@ class ForeignKeyRawIdWidgetTest(TestCase):
             'Hidden</a></strong>' % {'pk': hidden.pk}
         )
 
+    def test_render_unsafe_limit_choices_to(self):
+        rel = UnsafeLimitChoicesTo._meta.get_field('band').remote_field
+        w = widgets.ForeignKeyRawIdWidget(rel, widget_admin_site)
+        parameters = w.url_parameters()
+        parameters['name'] = '%22%26%3E%3Cescapeme'
+        self.assertHTMLEqual(
+            w.render('test', None),
+            '<input type="text" name="test" class="vForeignKeyRawIdAdminField">'
+            '<a class="related-lookup" href="/admin_widgets/band/?%s" '
+            'id="lookup_id_test" title="Lookup"></a>'
+            % '&amp;'.join('%s=%s' % (k, v) for k, v in parameters.items())
+        )
+
 
 @override_settings(ROOT_URLCONF='admin_widgets.urls')
 class ManyToManyRawIdWidgetTest(TestCase):
