@@ -1,9 +1,10 @@
 import uuid
 
 from django.contrib.auth.models import AbstractUser
-from django.contrib.auth.tests.custom_user import RemoveGroupsAndPermissions
 from django.db import models
 
+from .custom_user import RemoveGroupsAndPermissions
+
 with RemoveGroupsAndPermissions():
     class UUIDUser(AbstractUser):
         """A user with a UUID as primary key"""
