@@ -1,5 +1,5 @@
 """
-21. Specifying 'choices' for a field
+Specifying 'choices' for a field
 
 Most fields take a ``choices`` parameter, which should be a tuple of tuples
 specifying which are the valid values for that field.
@@ -10,17 +10,14 @@ field. This method returns the "human-readable" value of the field.
 """
 
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
+from django.utils.translation import gettext_lazy as _
 
 
-GENDER_CHOICES = (
-    ('M', 'Male'),
-    ('F', 'Female'),
-)
-
-
-@python_2_unicode_compatible
 class Person(models.Model):
+    GENDER_CHOICES = (
+        ('M', _('Male')),
+        ('F', _('Female')),
+    )
     name = models.CharField(max_length=20)
     gender = models.CharField(max_length=1, choices=GENDER_CHOICES)
 
