@@ -1,8 +1,6 @@
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Author(models.Model):
     name = models.CharField(max_length=20)
     age = models.IntegerField(null=True)
@@ -11,3 +9,14 @@ class Author(models.Model):
 
     def __str__(self):
         return self.name
+
+
+class Article(models.Model):
+    author = models.ForeignKey(Author, on_delete=models.CASCADE)
+
+
+class MySQLUnixTimestamp(models.Model):
+    timestamp = models.PositiveIntegerField()
+
+    def __str__(self):
+        return self.name
