@@ -1,5 +1,5 @@
 """
-23. Giving models a custom manager
+Giving models a custom manager
 
 You can use a custom ``Manager`` in a particular model by extending the base
 ``Manager`` class and instantiating your custom ``Manager`` in your model.
@@ -9,34 +9,32 @@ There are two reasons you might want to customize a ``Manager``: to add extra
 returns.
 """
 
-from __future__ import unicode_literals
-
 from django.contrib.contenttypes.fields import (
-    GenericForeignKey, GenericRelation
+    GenericForeignKey, GenericRelation,
 )
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
-
-# An example of a custom manager called "objects".
 
 
 class PersonManager(models.Manager):
     def get_fun_people(self):
         return self.filter(fun=True)
 
-# An example of a custom manager that sets get_queryset().
-
 
 class PublishedBookManager(models.Manager):
     def get_queryset(self):
-        return super(PublishedBookManager, self).get_queryset().filter(is_published=True)
+        return super().get_queryset().filter(is_published=True)
 
-# An example of a custom queryset that copies its methods onto the manager.
+
+class AnnotatedBookManager(models.Manager):
+    def get_queryset(self):
+        return super().get_queryset().annotate(
+            favorite_avg=models.Avg('favorite_books__favorite_thing_id')
+        )
 
 
 class CustomQuerySet(models.QuerySet):
     def filter(self, *args, **kwargs):
-        queryset = super(CustomQuerySet, self).filter(fun=True)
+        queryset = super().filter(fun=True)
         queryset._filter_CustomQuerySet = True
         return queryset
 
@@ -57,38 +55,50 @@ class CustomQuerySet(models.QuerySet):
 
 class BaseCustomManager(models.Manager):
     def __init__(self, arg):
-        super(BaseCustomManager, self).__init__()
+        super().__init__()
         self.init_arg = arg
 
     def filter(self, *args, **kwargs):
-        queryset = super(BaseCustomManager, self).filter(fun=True)
+        queryset = super().filter(fun=True)
         queryset._filter_CustomManager = True
         return queryset
 
     def manager_only(self):
         return self.all()
 
+
 CustomManager = BaseCustomManager.from_queryset(CustomQuerySet)
 
 
+class CustomInitQuerySet(models.QuerySet):
+    # QuerySet with an __init__() method that takes an additional argument.
+    def __init__(self, custom_optional_arg=None, model=None, query=None, using=None, hints=None):
+        super().__init__(model=model, query=query, using=using, hints=hints)
+
+
+class DeconstructibleCustomManager(BaseCustomManager.from_queryset(CustomQuerySet)):
+
+    def __init__(self, a, b, c=1, d=2):
+        super().__init__(a)
+
+
 class FunPeopleManager(models.Manager):
     def get_queryset(self):
-        return super(FunPeopleManager, self).get_queryset().filter(fun=True)
+        return super().get_queryset().filter(fun=True)
 
 
 class BoringPeopleManager(models.Manager):
     def get_queryset(self):
-        return super(BoringPeopleManager, self).get_queryset().filter(fun=False)
+        return super().get_queryset().filter(fun=False)
 
 
-@python_2_unicode_compatible
 class Person(models.Model):
     first_name = models.CharField(max_length=30)
     last_name = models.CharField(max_length=30)
     fun = models.BooleanField(default=False)
 
-    favorite_book = models.ForeignKey('Book', null=True, related_name='favorite_books')
-    favorite_thing_type = models.ForeignKey('contenttypes.ContentType', null=True)
+    favorite_book = models.ForeignKey('Book', models.SET_NULL, null=True, related_name='favorite_books')
+    favorite_thing_type = models.ForeignKey('contenttypes.ContentType', models.SET_NULL, null=True)
     favorite_thing_id = models.IntegerField(null=True)
     favorite_thing = GenericForeignKey('favorite_thing_type', 'favorite_thing_id')
 
@@ -98,19 +108,23 @@ class Person(models.Model):
 
     custom_queryset_default_manager = CustomQuerySet.as_manager()
     custom_queryset_custom_manager = CustomManager('hello')
+    custom_init_queryset_manager = CustomInitQuerySet.as_manager()
 
     def __str__(self):
         return "%s %s" % (self.first_name, self.last_name)
 
 
-@python_2_unicode_compatible
 class FunPerson(models.Model):
     first_name = models.CharField(max_length=30)
     last_name = models.CharField(max_length=30)
     fun = models.BooleanField(default=True)
-
-    favorite_book = models.ForeignKey('Book', null=True, related_name='fun_people_favorite_books')
-    favorite_thing_type = models.ForeignKey('contenttypes.ContentType', null=True)
+    favorite_book = models.ForeignKey(
+        'Book',
+        models.SET_NULL,
+        null=True,
+        related_name='fun_people_favorite_books',
+    )
+    favorite_thing_type = models.ForeignKey('contenttypes.ContentType', models.SET_NULL, null=True)
     favorite_thing_id = models.IntegerField(null=True)
     favorite_thing = GenericForeignKey('favorite_thing_type', 'favorite_thing_id')
 
@@ -120,33 +134,38 @@ class FunPerson(models.Model):
         return "%s %s" % (self.first_name, self.last_name)
 
 
-@python_2_unicode_compatible
 class Book(models.Model):
     title = models.CharField(max_length=50)
     author = models.CharField(max_length=30)
     is_published = models.BooleanField(default=False)
-    published_objects = PublishedBookManager()
     authors = models.ManyToManyField(Person, related_name='books')
     fun_authors = models.ManyToManyField(FunPerson, related_name='books')
+    favorite_things = GenericRelation(
+        Person,
+        content_type_field='favorite_thing_type',
+        object_id_field='favorite_thing_id',
+    )
+    fun_people_favorite_things = GenericRelation(
+        FunPerson,
+        content_type_field='favorite_thing_type',
+        object_id_field='favorite_thing_id',
+    )
 
-    favorite_things = GenericRelation(Person,
-        content_type_field='favorite_thing_type', object_id_field='favorite_thing_id')
+    published_objects = PublishedBookManager()
+    annotated_objects = AnnotatedBookManager()
 
-    fun_people_favorite_things = GenericRelation(FunPerson,
-        content_type_field='favorite_thing_type', object_id_field='favorite_thing_id')
+    class Meta:
+        base_manager_name = 'annotated_objects'
 
     def __str__(self):
         return self.title
 
-# An example of providing multiple custom managers.
-
 
 class FastCarManager(models.Manager):
     def get_queryset(self):
-        return super(FastCarManager, self).get_queryset().filter(top_speed__gt=150)
+        return super().get_queryset().filter(top_speed__gt=150)
 
 
-@python_2_unicode_compatible
 class Car(models.Model):
     name = models.CharField(max_length=10)
     mileage = models.IntegerField()
@@ -156,3 +175,63 @@ class Car(models.Model):
 
     def __str__(self):
         return self.name
+
+
+class FastCarAsBase(Car):
+    class Meta:
+        proxy = True
+        base_manager_name = 'fast_cars'
+
+
+class FastCarAsDefault(Car):
+    class Meta:
+        proxy = True
+        default_manager_name = 'fast_cars'
+
+
+class RestrictedManager(models.Manager):
+    def get_queryset(self):
+        return super().get_queryset().filter(is_public=True)
+
+
+class RelatedModel(models.Model):
+    name = models.CharField(max_length=50)
+
+    def __str__(self):
+        return self.name
+
+
+class RestrictedModel(models.Model):
+    name = models.CharField(max_length=50)
+    is_public = models.BooleanField(default=False)
+    related = models.ForeignKey(RelatedModel, models.CASCADE)
+
+    objects = RestrictedManager()
+    plain_manager = models.Manager()
+
+    def __str__(self):
+        return self.name
+
+
+class OneToOneRestrictedModel(models.Model):
+    name = models.CharField(max_length=50)
+    is_public = models.BooleanField(default=False)
+    related = models.OneToOneField(RelatedModel, models.CASCADE)
+
+    objects = RestrictedManager()
+    plain_manager = models.Manager()
+
+    def __str__(self):
+        return self.name
+
+
+class AbstractPerson(models.Model):
+    abstract_persons = models.Manager()
+    objects = models.CharField(max_length=30)
+
+    class Meta:
+        abstract = True
+
+
+class PersonFromAbstract(AbstractPerson):
+    pass
