@@ -8,13 +8,9 @@ from django.contrib.auth.decorators import (
 from django.http import HttpRequest, HttpResponse, HttpResponseNotAllowed
 from django.middleware.clickjacking import XFrameOptionsMiddleware
 from django.test import SimpleTestCase
-from django.utils import six
 from django.utils.decorators import method_decorator
-from django.utils.deprecation import RemovedInDjango20Warning
-from django.utils.encoding import force_text
-from django.utils.functional import allow_lazy, keep_lazy, keep_lazy_text, lazy
+from django.utils.functional import keep_lazy, keep_lazy_text, lazy
 from django.utils.safestring import mark_safe
-from django.utils.translation import ugettext_lazy
 from django.views.decorators.cache import (
     cache_control, cache_page, never_cache,
 )
@@ -113,10 +109,10 @@ class DecoratorsTest(TestCase):
         callback = user_passes_test(test1)(callback)
         callback = user_passes_test(test2)(callback)
 
-        class DummyUser(object):
+        class DummyUser:
             pass
 
-        class DummyRequest(object):
+        class DummyRequest:
             pass
 
         request = DummyRequest()
@@ -155,15 +151,6 @@ class DecoratorsTest(TestCase):
         request.method = 'DELETE'
         self.assertIsInstance(my_safe_view(request), HttpResponseNotAllowed)
 
-    def test_deprecated_allow_lazy(self):
-        with self.assertRaises(RemovedInDjango20Warning):
-            def noop_text(text):
-                return force_text(text)
-            noop_text = allow_lazy(noop_text, six.text_type)
-            rendered = noop_text(ugettext_lazy("I am a text"))
-            self.assertEqual(type(rendered), six.text_type)
-            self.assertEqual(rendered, "I am a text")
-
 
 # For testing method_decorator, a decorator that assumes a single argument.
 # We will get type arguments if there is a mismatch in the number of arguments.
@@ -181,7 +168,7 @@ def myattr_dec(func):
     def wrapper(*args, **kwargs):
         return func(*args, **kwargs)
     wrapper.myattr = True
-    return wraps(func)(wrapper)
+    return wrapper
 
 
 myattr_dec_m = method_decorator(myattr_dec)
@@ -191,13 +178,13 @@ def myattr2_dec(func):
     def wrapper(*args, **kwargs):
         return func(*args, **kwargs)
     wrapper.myattr2 = True
-    return wraps(func)(wrapper)
+    return wrapper
 
 
 myattr2_dec_m = method_decorator(myattr2_dec)
 
 
-class ClsDec(object):
+class ClsDec:
     def __init__(self, myattr):
         self.myattr = myattr
 
@@ -213,7 +200,7 @@ class MethodDecoratorTests(SimpleTestCase):
     Tests for method_decorator
     """
     def test_preserve_signature(self):
-        class Test(object):
+        class Test:
             @simple_dec_m
             def say(self, arg):
                 return arg
@@ -222,16 +209,26 @@ class MethodDecoratorTests(SimpleTestCase):
 
     def test_preserve_attributes(self):
         # Sanity check myattr_dec and myattr2_dec
+        @myattr_dec
+        def func():
+            pass
+        self.assertIs(getattr(func, 'myattr', False), True)
+
+        @myattr2_dec
+        def func():
+            pass
+        self.assertIs(getattr(func, 'myattr2', False), True)
+
         @myattr_dec
         @myattr2_dec
         def func():
             pass
 
         self.assertIs(getattr(func, 'myattr', False), True)
-        self.assertIs(getattr(func, 'myattr2', False), True)
+        self.assertIs(getattr(func, 'myattr2', False), False)
 
         # Decorate using method_decorator() on the method.
-        class TestPlain(object):
+        class TestPlain:
             @myattr_dec_m
             @myattr2_dec_m
             def method(self):
@@ -242,44 +239,66 @@ class MethodDecoratorTests(SimpleTestCase):
         # The decorators applied to the methods are applied before the ones
         # applied to the class.
         @method_decorator(myattr_dec_m, "method")
-        class TestMethodAndClass(object):
+        class TestMethodAndClass:
             @method_decorator(myattr2_dec_m)
             def method(self):
                 "A method"
                 pass
 
-        # Decorate using an iterable of decorators.
-        decorators = (myattr_dec_m, myattr2_dec_m)
-
-        @method_decorator(decorators, "method")
-        class TestIterable(object):
+        # Decorate using an iterable of function decorators.
+        @method_decorator((myattr_dec, myattr2_dec), 'method')
+        class TestFunctionIterable:
             def method(self):
                 "A method"
                 pass
 
-        for Test in (TestPlain, TestMethodAndClass, TestIterable):
-            self.assertIs(getattr(Test().method, 'myattr', False), True)
-            self.assertIs(getattr(Test().method, 'myattr2', False), True)
+        # Decorate using an iterable of method decorators.
+        decorators = (myattr_dec_m, myattr2_dec_m)
 
-            self.assertIs(getattr(Test.method, 'myattr', False), True)
-            self.assertIs(getattr(Test.method, 'myattr2', False), True)
+        @method_decorator(decorators, "method")
+        class TestMethodIterable:
+            def method(self):
+                "A method"
+                pass
 
-            self.assertEqual(Test.method.__doc__, 'A method')
-            self.assertEqual(Test.method.__name__, 'method')
+        tests = (TestPlain, TestMethodAndClass, TestFunctionIterable, TestMethodIterable)
+        for Test in tests:
+            with self.subTest(Test=Test):
+                self.assertIs(getattr(Test().method, 'myattr', False), True)
+                self.assertIs(getattr(Test().method, 'myattr2', False), True)
+                self.assertIs(getattr(Test.method, 'myattr', False), True)
+                self.assertIs(getattr(Test.method, 'myattr2', False), True)
+                self.assertEqual(Test.method.__doc__, 'A method')
+                self.assertEqual(Test.method.__name__, 'method')
+
+    def test_new_attribute(self):
+        """A decorator that sets a new attribute on the method."""
+        def decorate(func):
+            func.x = 1
+            return func
+
+        class MyClass:
+            @method_decorator(decorate)
+            def method(self):
+                return True
+
+        obj = MyClass()
+        self.assertEqual(obj.method.x, 1)
+        self.assertIs(obj.method(), True)
 
     def test_bad_iterable(self):
         decorators = {myattr_dec_m, myattr2_dec_m}
-        # The rest of the exception message differs between Python 2 and 3.
-        with self.assertRaisesMessage(TypeError, "'set' object"):
+        msg = "'set' object is not subscriptable"
+        with self.assertRaisesMessage(TypeError, msg):
             @method_decorator(decorators, "method")
-            class TestIterable(object):
+            class TestIterable:
                 def method(self):
                     "A method"
                     pass
 
     # Test for argumented decorator
     def test_argumented(self):
-        class Test(object):
+        class Test:
             @method_decorator(ClsDec(False))
             def method(self):
                 return True
@@ -296,7 +315,7 @@ class MethodDecoratorTests(SimpleTestCase):
 
         method_dec = method_decorator(original_dec)
 
-        class bound_wrapper(object):
+        class bound_wrapper:
             def __init__(self, wrapped):
                 self.wrapped = wrapped
                 self.__name__ = wrapped.__name__
@@ -307,7 +326,7 @@ class MethodDecoratorTests(SimpleTestCase):
             def __get__(self, instance, cls=None):
                 return self
 
-        class descriptor_wrapper(object):
+        class descriptor_wrapper:
             def __init__(self, wrapped):
                 self.wrapped = wrapped
                 self.__name__ = wrapped.__name__
@@ -315,7 +334,7 @@ class MethodDecoratorTests(SimpleTestCase):
             def __get__(self, instance, cls=None):
                 return bound_wrapper(self.wrapped.__get__(instance, cls))
 
-        class Test(object):
+        class Test:
             @method_dec
             @descriptor_wrapper
             def method(self, arg):
@@ -333,7 +352,7 @@ class MethodDecoratorTests(SimpleTestCase):
             return _wrapper
 
         @method_decorator(deco, name="method")
-        class Test(object):
+        class Test:
             def method(self):
                 return False
 
@@ -362,11 +381,11 @@ class MethodDecoratorTests(SimpleTestCase):
         decorators = (add_exclamation_mark, add_question_mark)
 
         @method_decorator(decorators, name="method")
-        class TestFirst(object):
+        class TestFirst:
             def method(self):
                 return "hello world"
 
-        class TestSecond(object):
+        class TestSecond:
             @method_decorator(decorators)
             def method(self):
                 return "hello world"
@@ -384,7 +403,7 @@ class MethodDecoratorTests(SimpleTestCase):
         )
         with self.assertRaisesMessage(TypeError, msg):
             @method_decorator(lambda: None, name="prop")
-            class Test(object):
+            class Test:
                 prop = 1
 
                 @classmethod
@@ -397,11 +416,11 @@ class MethodDecoratorTests(SimpleTestCase):
         """
         msg = (
             "The keyword argument `name` must be the name of a method of the "
-            "decorated class: <class 'Test'>. Got 'non_existing_method' instead"
+            "decorated class: <class 'Test'>. Got 'nonexistent_method' instead"
         )
         with self.assertRaisesMessage(ValueError, msg):
-            @method_decorator(lambda: None, name="non_existing_method")
-            class Test(object):
+            @method_decorator(lambda: None, name='nonexistent_method')
+            class Test:
                 @classmethod
                 def __module__(cls):
                     return "tests"
