@@ -1,29 +1,23 @@
-from __future__ import unicode_literals
-
 import copy
 import datetime
 
 from django.contrib.auth.models import User
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class RevisionableModel(models.Model):
-    base = models.ForeignKey('self', null=True)
+    base = models.ForeignKey('self', models.SET_NULL, null=True)
     title = models.CharField(blank=True, max_length=255)
     when = models.DateTimeField(default=datetime.datetime.now)
 
     def __str__(self):
         return "%s (%s, %s)" % (self.title, self.id, self.base.id)
 
-    def save(self, *args, **kwargs):
-        super(RevisionableModel, self).save(*args, **kwargs)
+    def save(self, *args, force_insert=None, force_update=None, **kwargs):
+        super().save(*args, force_insert=force_insert, force_update=force_update, **kwargs)
         if not self.base:
             self.base = self
-            kwargs.pop('force_insert', None)
-            kwargs.pop('force_update', None)
-            super(RevisionableModel, self).save(*args, **kwargs)
+            super().save(*args, **kwargs)
 
     def new_revision(self):
         new_revision = copy.copy(self)
@@ -32,11 +26,10 @@ class RevisionableModel(models.Model):
 
 
 class Order(models.Model):
-    created_by = models.ForeignKey(User)
+    created_by = models.ForeignKey(User, models.CASCADE)
     text = models.TextField()
 
 
-@python_2_unicode_compatible
 class TestObject(models.Model):
     first = models.CharField(max_length=20)
     second = models.CharField(max_length=20)
