@@ -1,4 +1,3 @@
-# coding: utf-8
 """
 Callable defaults
 
@@ -13,10 +12,8 @@ field.
 from datetime import datetime
 
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Article(models.Model):
     headline = models.CharField(max_length=100, default='Default headline')
     pub_date = models.DateTimeField(default=datetime.now)
