@@ -14,34 +14,30 @@ from django.contrib.auth.models import Permission
 from django.contrib.contenttypes.fields import GenericForeignKey
 from django.contrib.contenttypes.models import ContentType
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Category(models.Model):
     title = models.CharField(max_length=100)
     description = models.TextField()
 
-    def __str__(self):
-        return self.title
-
     class Meta:
         ordering = ('title',)
 
+    def __str__(self):
+        return self.title
+
 
-@python_2_unicode_compatible
 class Article(models.Model):
     headline = models.CharField(max_length=100, default='Default headline')
     pub_date = models.DateTimeField()
 
-    def __str__(self):
-        return self.headline
-
     class Meta:
         ordering = ('-pub_date', 'headline')
 
+    def __str__(self):
+        return self.headline
+
 
-@python_2_unicode_compatible
 class Blog(models.Model):
     name = models.CharField(max_length=100)
     featured = models.ForeignKey(Article, models.CASCADE, related_name='fixtures_featured_set')
@@ -52,7 +48,6 @@ class Blog(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Tag(models.Model):
     name = models.CharField(max_length=100)
     tagged_type = models.ForeignKey(ContentType, models.CASCADE, related_name="fixtures_tag_set")
@@ -69,24 +64,23 @@ class PersonManager(models.Manager):
         return self.get(name=name)
 
 
-@python_2_unicode_compatible
 class Person(models.Model):
     objects = PersonManager()
     name = models.CharField(max_length=100)
 
-    def __str__(self):
-        return self.name
-
     class Meta:
         ordering = ('name',)
 
+    def __str__(self):
+        return self.name
+
     def natural_key(self):
         return (self.name,)
 
 
 class SpyManager(PersonManager):
     def get_queryset(self):
-        return super(SpyManager, self).get_queryset().filter(cover_blown=False)
+        return super().get_queryset().filter(cover_blown=False)
 
 
 class Spy(Person):
@@ -99,7 +93,6 @@ class ProxySpy(Spy):
         proxy = True
 
 
-@python_2_unicode_compatible
 class Visa(models.Model):
     person = models.ForeignKey(Person, models.CASCADE)
     permissions = models.ManyToManyField(Permission, blank=True)
@@ -109,18 +102,35 @@ class Visa(models.Model):
                           ', '.join(p.name for p in self.permissions.all()))
 
 
-@python_2_unicode_compatible
 class Book(models.Model):
     name = models.CharField(max_length=100)
     authors = models.ManyToManyField(Person)
 
+    class Meta:
+        ordering = ('name',)
+
     def __str__(self):
         authors = ' and '.join(a.name for a in self.authors.all())
         return '%s by %s' % (self.name, authors) if authors else self.name
 
-    class Meta:
-        ordering = ('name',)
-
 
 class PrimaryKeyUUIDModel(models.Model):
     id = models.UUIDField(primary_key=True, default=uuid.uuid4)
+
+
+class NaturalKeyThing(models.Model):
+    key = models.CharField(max_length=100)
+    other_thing = models.ForeignKey('NaturalKeyThing', on_delete=models.CASCADE, null=True)
+    other_things = models.ManyToManyField('NaturalKeyThing', related_name='thing_m2m_set')
+
+    class Manager(models.Manager):
+        def get_by_natural_key(self, key):
+            return self.get(key=key)
+
+    objects = Manager()
+
+    def natural_key(self):
+        return (self.key,)
+
+    def __str__(self):
+        return self.key
