@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.forms import (
     CharField, HiddenInput, PasswordInput, Textarea, TextInput,
     ValidationError,
@@ -75,7 +73,8 @@ class CharFieldTest(FormFieldAssertionsMixin, SimpleTestCase):
             CharField(min_length='a')
         with self.assertRaises(ValueError):
             CharField(max_length='a')
-        with self.assertRaises(ValueError):
+        msg = '__init__() takes 1 positional argument but 2 were given'
+        with self.assertRaisesMessage(TypeError, msg):
             CharField('a')
 
     def test_charfield_widget_attrs(self):
@@ -121,6 +120,35 @@ class CharFieldTest(FormFieldAssertionsMixin, SimpleTestCase):
         self.assertEqual(f.clean(' 1'), ' 1')
         self.assertEqual(f.clean('1 '), '1 ')
 
+    def test_strip_before_checking_empty(self):
+        """
+        A whitespace-only value, ' ', is stripped to an empty string and then
+        converted to the empty value, None.
+        """
+        f = CharField(required=False, empty_value=None)
+        self.assertIsNone(f.clean(' '))
+
+    def test_clean_non_string(self):
+        """CharField.clean() calls str(value) before stripping it."""
+        class StringWrapper:
+            def __init__(self, v):
+                self.v = v
+
+            def __str__(self):
+                return self.v
+
+        value = StringWrapper(' ')
+        f1 = CharField(required=False, empty_value=None)
+        self.assertIsNone(f1.clean(value))
+        f2 = CharField(strip=False)
+        self.assertEqual(f2.clean(value), ' ')
+
     def test_charfield_disabled(self):
         f = CharField(disabled=True)
-        self.assertWidgetRendersTo(f, '<input type="text" name="f" id="id_f" disabled required />')
+        self.assertWidgetRendersTo(f, '<input type="text" name="f" id="id_f" disabled required>')
+
+    def test_null_characters_prohibited(self):
+        f = CharField()
+        msg = 'Null characters are not allowed.'
+        with self.assertRaisesMessage(ValidationError, msg):
+            f.clean('\x00something')
