@@ -16,6 +16,6 @@ class LiveWidgetTests(AdminSeleniumTestCase):
         """
         article = Article.objects.create(content="\nTst\n")
         self.selenium.get(self.live_server_url + reverse('article_form', args=[article.pk]))
-        self.selenium.find_element_by_id('submit').submit()
+        self.selenium.find_element_by_id('submit').click()
         article = Article.objects.get(pk=article.pk)
         self.assertEqual(article.content, "\r\nTst\r\n")
