@@ -7,10 +7,10 @@ class PasswordInputTest(WidgetTest):
     widget = PasswordInput()
 
     def test_render(self):
-        self.check_html(self.widget, 'password', '', html='<input type="password" name="password" />')
+        self.check_html(self.widget, 'password', '', html='<input type="password" name="password">')
 
     def test_render_ignore_value(self):
-        self.check_html(self.widget, 'password', 'secret', html='<input type="password" name="password" />')
+        self.check_html(self.widget, 'password', 'secret', html='<input type="password" name="password">')
 
     def test_render_value_true(self):
         """
@@ -18,9 +18,9 @@ class PasswordInputTest(WidgetTest):
         render its value. For security reasons, this is off by default.
         """
         widget = PasswordInput(render_value=True)
-        self.check_html(widget, 'password', '', html='<input type="password" name="password" />')
-        self.check_html(widget, 'password', None, html='<input type="password" name="password" />')
+        self.check_html(widget, 'password', '', html='<input type="password" name="password">')
+        self.check_html(widget, 'password', None, html='<input type="password" name="password">')
         self.check_html(
             widget, 'password', 'test@example.com',
-            html='<input type="password" name="password" value="test@example.com" />',
+            html='<input type="password" name="password" value="test@example.com">',
         )
