@@ -4,14 +4,12 @@ from django.contrib.contenttypes.fields import (
 from django.contrib.contenttypes.models import ContentType
 from django.db import models
 from django.db.models.deletion import ProtectedError
-from django.utils.encoding import python_2_unicode_compatible
 
 __all__ = ('Link', 'Place', 'Restaurant', 'Person', 'Address',
            'CharLink', 'TextLink', 'OddRelation1', 'OddRelation2',
            'Contact', 'Organization', 'Note', 'Company')
 
 
-@python_2_unicode_compatible
 class Link(models.Model):
     content_type = models.ForeignKey(ContentType, models.CASCADE)
     object_id = models.PositiveIntegerField()
@@ -26,23 +24,25 @@ class LinkProxy(Link):
         proxy = True
 
 
-@python_2_unicode_compatible
 class Place(models.Model):
     name = models.CharField(max_length=100)
-    links = GenericRelation(Link)
+    links = GenericRelation(Link, related_query_name='places')
     link_proxy = GenericRelation(LinkProxy)
 
     def __str__(self):
         return "Place: %s" % self.name
 
 
-@python_2_unicode_compatible
 class Restaurant(Place):
     def __str__(self):
         return "Restaurant: %s" % self.name
 
 
-@python_2_unicode_compatible
+class Cafe(Restaurant):
+    def __str__(self):
+        return "Cafe: %s" % self.name
+
+
 class Address(models.Model):
     street = models.CharField(max_length=80)
     city = models.CharField(max_length=50)
@@ -56,7 +56,6 @@ class Address(models.Model):
         return '%s %s, %s %s' % (self.street, self.city, self.state, self.zipcode)
 
 
-@python_2_unicode_compatible
 class Person(models.Model):
     account = models.IntegerField(primary_key=True)
     name = models.CharField(max_length=128)
@@ -105,7 +104,6 @@ class Organization(models.Model):
     contacts = models.ManyToManyField(Contact, related_name='organizations')
 
 
-@python_2_unicode_compatible
 class Company(models.Model):
     name = models.CharField(max_length=100)
     links = GenericRelation(Link)
@@ -119,7 +117,6 @@ class Developer(models.Model):
     name = models.CharField(max_length=15)
 
 
-@python_2_unicode_compatible
 class Team(models.Model):
     name = models.CharField(max_length=15)
     members = models.ManyToManyField(Developer)
@@ -135,9 +132,8 @@ class Guild(models.Model):
     name = models.CharField(max_length=15)
     members = models.ManyToManyField(Developer)
 
-    def __nonzero__(self):
-
-        return self.members.count()
+    def __bool__(self):
+        return False
 
 
 class Tag(models.Model):
@@ -153,7 +149,7 @@ class Board(models.Model):
 
 class SpecialGenericRelation(GenericRelation):
     def __init__(self, *args, **kwargs):
-        super(SpecialGenericRelation, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.editable = True
         self.save_form_data_calls = 0
 
@@ -162,7 +158,7 @@ class SpecialGenericRelation(GenericRelation):
 
 
 class HasLinks(models.Model):
-    links = SpecialGenericRelation(Link)
+    links = SpecialGenericRelation(Link, related_query_name='targets')
 
     class Meta:
         abstract = True
@@ -173,7 +169,7 @@ class HasLinkThing(HasLinks):
 
 
 class A(models.Model):
-    flag = models.NullBooleanField()
+    flag = models.BooleanField(null=True)
     content_type = models.ForeignKey(ContentType, models.CASCADE)
     object_id = models.PositiveIntegerField()
     content_object = GenericForeignKey('content_type', 'object_id')
