@@ -2,10 +2,8 @@ from django.db import models
 from django.db.models import QuerySet
 from django.db.models.manager import BaseManager
 from django.urls import reverse
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Artist(models.Model):
     name = models.CharField(max_length=100)
 
@@ -21,7 +19,6 @@ class Artist(models.Model):
         return reverse('artist_detail', kwargs={'pk': self.id})
 
 
-@python_2_unicode_compatible
 class Author(models.Model):
     name = models.CharField(max_length=100)
     slug = models.SlugField()
@@ -37,12 +34,12 @@ class DoesNotExistQuerySet(QuerySet):
     def get(self, *args, **kwargs):
         raise Author.DoesNotExist
 
+
 DoesNotExistBookManager = BaseManager.from_queryset(DoesNotExistQuerySet)
 
 
-@python_2_unicode_compatible
 class Book(models.Model):
-    name = models.CharField(max_length=300)
+    name = models.CharField(max_length=255)
     slug = models.SlugField()
     pages = models.IntegerField()
     authors = models.ManyToManyField(Author)
@@ -60,7 +57,7 @@ class Book(models.Model):
 
 class Page(models.Model):
     content = models.TextField()
-    template = models.CharField(max_length=300)
+    template = models.CharField(max_length=255)
 
 
 class BookSigning(models.Model):
