@@ -1,46 +1,5 @@
-"""
-gdalinfo tests/gis_tests/data/rasters/raster.tif:
-
-Driver: GTiff/GeoTIFF
-Files: tests/gis_tests/data/rasters/raster.tif
-Size is 163, 174
-Coordinate System is:
-PROJCS["NAD83 / Florida GDL Albers",
-    GEOGCS["NAD83",
-        DATUM["North_American_Datum_1983",
-            SPHEROID["GRS 1980",6378137,298.2572221010002,
-                AUTHORITY["EPSG","7019"]],
-            TOWGS84[0,0,0,0,0,0,0],
-            AUTHORITY["EPSG","6269"]],
-        PRIMEM["Greenwich",0],
-        UNIT["degree",0.0174532925199433],
-        AUTHORITY["EPSG","4269"]],
-    PROJECTION["Albers_Conic_Equal_Area"],
-    PARAMETER["standard_parallel_1",24],
-    PARAMETER["standard_parallel_2",31.5],
-    PARAMETER["latitude_of_center",24],
-    PARAMETER["longitude_of_center",-84],
-    PARAMETER["false_easting",400000],
-    PARAMETER["false_northing",0],
-    UNIT["metre",1,
-        AUTHORITY["EPSG","9001"]],
-    AUTHORITY["EPSG","3086"]]
-Origin = (511700.468070655711927,435103.377123198588379)
-Pixel Size = (100.000000000000000,-100.000000000000000)
-Metadata:
-  AREA_OR_POINT=Area
-Image Structure Metadata:
-  INTERLEAVE=BAND
-Corner Coordinates:
-Upper Left  (  511700.468,  435103.377) ( 82d51'46.16"W, 27d55' 1.53"N)
-Lower Left  (  511700.468,  417703.377) ( 82d51'52.04"W, 27d45'37.50"N)
-Upper Right (  528000.468,  435103.377) ( 82d41'48.81"W, 27d54'56.30"N)
-Lower Right (  528000.468,  417703.377) ( 82d41'55.54"W, 27d45'32.28"N)
-Center      (  519850.468,  426403.377) ( 82d46'50.64"W, 27d50'16.99"N)
-Band 1 Block=163x50 Type=Byte, ColorInterp=Gray
-  NoData Value=15
-"""
 import os
+import shutil
 import struct
 import tempfile
 
@@ -49,8 +8,6 @@ from django.contrib.gis.gdal.error import GDALException
 from django.contrib.gis.gdal.raster.band import GDALBand
 from django.contrib.gis.shortcuts import numpy
 from django.test import SimpleTestCase
-from django.utils import six
-from django.utils._os import upath
 
 from ..data.rasters.textrasters import JSON_RASTER
 
@@ -60,8 +17,7 @@ class GDALRasterTests(SimpleTestCase):
     Test a GDALRaster instance created from a file (GeoTiff).
     """
     def setUp(self):
-        self.rs_path = os.path.join(os.path.dirname(upath(__file__)),
-                                    '../data/rasters/raster.tif')
+        self.rs_path = os.path.join(os.path.dirname(__file__), '../data/rasters/raster.tif')
         self.rs = GDALRaster(self.rs_path)
 
     def test_rs_name_repr(self):
@@ -106,6 +62,9 @@ class GDALRasterTests(SimpleTestCase):
         self.assertEqual(self.rs.skew.y, 0)
         # Create in-memory rasters and change gtvalues
         rsmem = GDALRaster(JSON_RASTER)
+        # geotransform accepts both floats and ints
+        rsmem.geotransform = [0.0, 1.0, 2.0, 3.0, 4.0, 5.0]
+        self.assertEqual(rsmem.geotransform, [0.0, 1.0, 2.0, 3.0, 4.0, 5.0])
         rsmem.geotransform = range(6)
         self.assertEqual(rsmem.geotransform, [float(x) for x in range(6)])
         self.assertEqual(rsmem.origin, [0, 3])
@@ -120,6 +79,18 @@ class GDALRasterTests(SimpleTestCase):
         self.assertEqual(rsmem.width, 5)
         self.assertEqual(rsmem.height, 5)
 
+    def test_geotransform_bad_inputs(self):
+        rsmem = GDALRaster(JSON_RASTER)
+        error_geotransforms = [
+            [1, 2],
+            [1, 2, 3, 4, 5, 'foo'],
+            [1, 2, 3, 4, 5, 6, 'foo'],
+        ]
+        msg = 'Geotransform must consist of 6 numeric values.'
+        for geotransform in error_geotransforms:
+            with self.subTest(i=geotransform), self.assertRaisesMessage(ValueError, msg):
+                rsmem.geotransform = geotransform
+
     def test_rs_extent(self):
         self.assertEqual(
             self.rs.extent,
@@ -185,6 +156,69 @@ class GDALRasterTests(SimpleTestCase):
         else:
             self.assertEqual(restored_raster.bands[0].data(), self.rs.bands[0].data())
 
+    def test_vsi_raster_creation(self):
+        # Open a raster as a file object.
+        with open(self.rs_path, 'rb') as dat:
+            # Instantiate a raster from the file binary buffer.
+            vsimem = GDALRaster(dat.read())
+        # The data of the in-memory file is equal to the source file.
+        result = vsimem.bands[0].data()
+        target = self.rs.bands[0].data()
+        if numpy:
+            result = result.flatten().tolist()
+            target = target.flatten().tolist()
+        self.assertEqual(result, target)
+
+    def test_vsi_raster_deletion(self):
+        path = '/vsimem/raster.tif'
+        # Create a vsi-based raster from scratch.
+        vsimem = GDALRaster({
+            'name': path,
+            'driver': 'tif',
+            'width': 4,
+            'height': 4,
+            'srid': 4326,
+            'bands': [{
+                'data': range(16),
+            }],
+        })
+        # The virtual file exists.
+        rst = GDALRaster(path)
+        self.assertEqual(rst.width, 4)
+        # Delete GDALRaster.
+        del vsimem
+        del rst
+        # The virtual file has been removed.
+        msg = 'Could not open the datasource at "/vsimem/raster.tif"'
+        with self.assertRaisesMessage(GDALException, msg):
+            GDALRaster(path)
+
+    def test_vsi_invalid_buffer_error(self):
+        msg = 'Failed creating VSI raster from the input buffer.'
+        with self.assertRaisesMessage(GDALException, msg):
+            GDALRaster(b'not-a-raster-buffer')
+
+    def test_vsi_buffer_property(self):
+        # Create a vsi-based raster from scratch.
+        rast = GDALRaster({
+            'name': '/vsimem/raster.tif',
+            'driver': 'tif',
+            'width': 4,
+            'height': 4,
+            'srid': 4326,
+            'bands': [{
+                'data': range(16),
+            }],
+        })
+        # Do a round trip from raster to buffer to raster.
+        result = GDALRaster(rast.vsi_buffer).bands[0].data()
+        if numpy:
+            result = result.flatten().tolist()
+        # Band data is equal to nodata value except on input block of ones.
+        self.assertEqual(result, list(range(16)))
+        # The vsi buffer is None for rasters that are not vsi based.
+        self.assertIsNone(self.rs.vsi_buffer)
+
     def test_offset_size_and_shape_on_raster_creation(self):
         rast = GDALRaster({
             'datatype': 1,
@@ -223,7 +257,7 @@ class GDALRasterTests(SimpleTestCase):
         if numpy:
             result = result.flatten().tolist()
         # All band data is equal to nodata value.
-        self.assertEqual(result, [23, ] * 4)
+        self.assertEqual(result, [23] * 4)
 
     def test_set_nodata_none_on_raster_creation(self):
         if GDAL_VERSION < (2, 1):
@@ -243,6 +277,132 @@ class GDALRasterTests(SimpleTestCase):
         # Band data is equal to zero becaues no nodata value has been specified.
         self.assertEqual(result, [0] * 4)
 
+    def test_raster_metadata_property(self):
+        data = self.rs.metadata
+        self.assertEqual(data['DEFAULT'], {'AREA_OR_POINT': 'Area'})
+        self.assertEqual(data['IMAGE_STRUCTURE'], {'INTERLEAVE': 'BAND'})
+
+        # Create file-based raster from scratch
+        source = GDALRaster({
+            'datatype': 1,
+            'width': 2,
+            'height': 2,
+            'srid': 4326,
+            'bands': [{'data': range(4), 'nodata_value': 99}],
+        })
+        # Set metadata on raster and on a band.
+        metadata = {
+            'DEFAULT': {'OWNER': 'Django', 'VERSION': '1.0', 'AREA_OR_POINT': 'Point'},
+        }
+        source.metadata = metadata
+        source.bands[0].metadata = metadata
+        self.assertEqual(source.metadata['DEFAULT'], metadata['DEFAULT'])
+        self.assertEqual(source.bands[0].metadata['DEFAULT'], metadata['DEFAULT'])
+        # Update metadata on raster.
+        metadata = {
+            'DEFAULT': {'VERSION': '2.0'},
+        }
+        source.metadata = metadata
+        self.assertEqual(source.metadata['DEFAULT']['VERSION'], '2.0')
+        # Remove metadata on raster.
+        metadata = {
+            'DEFAULT': {'OWNER': None},
+        }
+        source.metadata = metadata
+        self.assertNotIn('OWNER', source.metadata['DEFAULT'])
+
+    def test_raster_info_accessor(self):
+        if GDAL_VERSION < (2, 1):
+            msg = 'GDAL ≥ 2.1 is required for using the info property.'
+            with self.assertRaisesMessage(ValueError, msg):
+                self.rs.info
+            return
+        gdalinfo = """
+        Driver: GTiff/GeoTIFF
+        Files: {0}
+        Size is 163, 174
+        Coordinate System is:
+        PROJCS["NAD83 / Florida GDL Albers",
+            GEOGCS["NAD83",
+                DATUM["North_American_Datum_1983",
+                    SPHEROID["GRS 1980",6378137,298.257222101,
+                        AUTHORITY["EPSG","7019"]],
+                    TOWGS84[0,0,0,0,0,0,0],
+                    AUTHORITY["EPSG","6269"]],
+                PRIMEM["Greenwich",0,
+                    AUTHORITY["EPSG","8901"]],
+                UNIT["degree",0.0174532925199433,
+                    AUTHORITY["EPSG","9122"]],
+                AUTHORITY["EPSG","4269"]],
+            PROJECTION["Albers_Conic_Equal_Area"],
+            PARAMETER["standard_parallel_1",24],
+            PARAMETER["standard_parallel_2",31.5],
+            PARAMETER["latitude_of_center",24],
+            PARAMETER["longitude_of_center",-84],
+            PARAMETER["false_easting",400000],
+            PARAMETER["false_northing",0],
+            UNIT["metre",1,
+                AUTHORITY["EPSG","9001"]],
+            AXIS["X",EAST],
+            AXIS["Y",NORTH],
+            AUTHORITY["EPSG","3086"]]
+        Origin = (511700.468070655711927,435103.377123198588379)
+        Pixel Size = (100.000000000000000,-100.000000000000000)
+        Metadata:
+          AREA_OR_POINT=Area
+        Image Structure Metadata:
+          INTERLEAVE=BAND
+        Corner Coordinates:
+        Upper Left  (  511700.468,  435103.377) ( 82d51'46.16"W, 27d55' 1.53"N)
+        Lower Left  (  511700.468,  417703.377) ( 82d51'52.04"W, 27d45'37.50"N)
+        Upper Right (  528000.468,  435103.377) ( 82d41'48.81"W, 27d54'56.30"N)
+        Lower Right (  528000.468,  417703.377) ( 82d41'55.54"W, 27d45'32.28"N)
+        Center      (  519850.468,  426403.377) ( 82d46'50.64"W, 27d50'16.99"N)
+        Band 1 Block=163x50 Type=Byte, ColorInterp=Gray
+          NoData Value=15
+        """.format(self.rs_path)
+        # Data
+        info_dyn = [line.strip() for line in self.rs.info.split('\n') if line.strip() != '']
+        info_ref = [line.strip() for line in gdalinfo.split('\n') if line.strip() != '']
+        self.assertEqual(info_dyn, info_ref)
+
+    def test_compressed_file_based_raster_creation(self):
+        rstfile = tempfile.NamedTemporaryFile(suffix='.tif')
+        # Make a compressed copy of an existing raster.
+        compressed = self.rs.warp({'papsz_options': {'compress': 'packbits'}, 'name': rstfile.name})
+        # Check physically if compression worked.
+        self.assertLess(os.path.getsize(compressed.name), os.path.getsize(self.rs.name))
+        # Create file-based raster with options from scratch.
+        compressed = GDALRaster({
+            'datatype': 1,
+            'driver': 'tif',
+            'name': rstfile.name,
+            'width': 40,
+            'height': 40,
+            'srid': 3086,
+            'origin': (500000, 400000),
+            'scale': (100, -100),
+            'skew': (0, 0),
+            'bands': [{
+                'data': range(40 ^ 2),
+                'nodata_value': 255,
+            }],
+            'papsz_options': {
+                'compress': 'packbits',
+                'pixeltype': 'signedbyte',
+                'blockxsize': 23,
+                'blockysize': 23,
+            }
+        })
+        # Check if options used on creation are stored in metadata.
+        # Reopening the raster ensures that all metadata has been written
+        # to the file.
+        compressed = GDALRaster(compressed.name)
+        self.assertEqual(compressed.metadata['IMAGE_STRUCTURE']['COMPRESSION'], 'PACKBITS',)
+        self.assertEqual(compressed.bands[0].metadata['IMAGE_STRUCTURE']['PIXELTYPE'], 'SIGNEDBYTE')
+        if GDAL_VERSION >= (2, 1):
+            self.assertIn('Block=40x23', compressed.info)
+
     def test_raster_warp(self):
         # Create in memory raster
         source = GDALRaster({
@@ -355,10 +515,10 @@ class GDALRasterTests(SimpleTestCase):
         self.assertEqual(target.width, 7)
         self.assertEqual(target.height, 7)
         self.assertEqual(target.bands[0].datatype(), source.bands[0].datatype())
-        self.assertAlmostEqual(target.origin[0], 9124842.791079799)
-        self.assertAlmostEqual(target.origin[1], 1589911.6476407414)
-        self.assertAlmostEqual(target.scale[0], 223824.82664250192)
-        self.assertAlmostEqual(target.scale[1], -223824.82664250192)
+        self.assertAlmostEqual(target.origin[0], 9124842.791079799, 3)
+        self.assertAlmostEqual(target.origin[1], 1589911.6476407414, 3)
+        self.assertAlmostEqual(target.scale[0], 223824.82664250192, 3)
+        self.assertAlmostEqual(target.scale[1], -223824.82664250192, 3)
         self.assertEqual(target.skew, [0, 0])
 
         result = target.bands[0].data()
@@ -382,52 +542,54 @@ class GDALRasterTests(SimpleTestCase):
 
 
 class GDALBandTests(SimpleTestCase):
-    def setUp(self):
-        self.rs_path = os.path.join(os.path.dirname(upath(__file__)), '../data/rasters/raster.tif')
-        rs = GDALRaster(self.rs_path)
-        self.band = rs.bands[0]
+    rs_path = os.path.join(os.path.dirname(__file__), '../data/rasters/raster.tif')
 
     def test_band_data(self):
-        pam_file = self.rs_path + '.aux.xml'
-        self.assertEqual(self.band.width, 163)
-        self.assertEqual(self.band.height, 174)
-        self.assertEqual(self.band.description, '')
-        self.assertEqual(self.band.datatype(), 1)
-        self.assertEqual(self.band.datatype(as_string=True), 'GDT_Byte')
-        self.assertEqual(self.band.nodata_value, 15)
+        rs = GDALRaster(self.rs_path)
+        band = rs.bands[0]
+        self.assertEqual(band.width, 163)
+        self.assertEqual(band.height, 174)
+        self.assertEqual(band.description, '')
+        self.assertEqual(band.datatype(), 1)
+        self.assertEqual(band.datatype(as_string=True), 'GDT_Byte')
+        self.assertEqual(band.color_interp(), 1)
+        self.assertEqual(band.color_interp(as_string=True), 'GCI_GrayIndex')
+        self.assertEqual(band.nodata_value, 15)
         if numpy:
-            data = self.band.data()
+            data = band.data()
             assert_array = numpy.loadtxt(
-                os.path.join(os.path.dirname(upath(__file__)), '../data/rasters/raster.numpy.txt')
+                os.path.join(os.path.dirname(__file__), '../data/rasters/raster.numpy.txt')
             )
             numpy.testing.assert_equal(data, assert_array)
-            self.assertEqual(data.shape, (self.band.height, self.band.width))
-        try:
-            smin, smax, smean, sstd = self.band.statistics(approximate=True)
+            self.assertEqual(data.shape, (band.height, band.width))
+
+    def test_band_statistics(self):
+        with tempfile.TemporaryDirectory() as tmp_dir:
+            rs_path = os.path.join(tmp_dir, 'raster.tif')
+            shutil.copyfile(self.rs_path, rs_path)
+            rs = GDALRaster(rs_path)
+            band = rs.bands[0]
+            pam_file = rs_path + '.aux.xml'
+            smin, smax, smean, sstd = band.statistics(approximate=True)
             self.assertEqual(smin, 0)
             self.assertEqual(smax, 9)
             self.assertAlmostEqual(smean, 2.842331288343558)
             self.assertAlmostEqual(sstd, 2.3965567248965356)
 
-            smin, smax, smean, sstd = self.band.statistics(approximate=False, refresh=True)
+            smin, smax, smean, sstd = band.statistics(approximate=False, refresh=True)
             self.assertEqual(smin, 0)
             self.assertEqual(smax, 9)
             self.assertAlmostEqual(smean, 2.828326634228898)
             self.assertAlmostEqual(sstd, 2.4260526986669095)
 
-            self.assertEqual(self.band.min, 0)
-            self.assertEqual(self.band.max, 9)
-            self.assertAlmostEqual(self.band.mean, 2.828326634228898)
-            self.assertAlmostEqual(self.band.std, 2.4260526986669095)
+            self.assertEqual(band.min, 0)
+            self.assertEqual(band.max, 9)
+            self.assertAlmostEqual(band.mean, 2.828326634228898)
+            self.assertAlmostEqual(band.std, 2.4260526986669095)
 
             # Statistics are persisted into PAM file on band close
-            self.band = None
+            rs = band = None
             self.assertTrue(os.path.isfile(pam_file))
-        finally:
-            # Close band and remove file if created
-            self.band = None
-            if os.path.isfile(pam_file):
-                os.remove(pam_file)
 
     def test_read_mode_error(self):
         # Open raster in read mode
@@ -499,7 +661,7 @@ class GDALBandTests(SimpleTestCase):
             self.assertEqual(result, block)
 
         # Set data from memoryview
-        bandmem.data(six.memoryview(packed_block), (1, 1), (2, 2))
+        bandmem.data(memoryview(packed_block), (1, 1), (2, 2))
         result = bandmem.data(offset=(1, 1), size=(2, 2))
         if numpy:
             numpy.testing.assert_equal(result, numpy.array(block).reshape(2, 2))
