@@ -1,18 +1,13 @@
-from django.utils.encoding import python_2_unicode_compatible
+from django.contrib.gis.db import models
 
-from ..models import models
 from ..utils import gisfield_may_be_null
 
 
-@python_2_unicode_compatible
 class NamedModel(models.Model):
     name = models.CharField(max_length=30)
 
-    objects = models.GeoManager()
-
     class Meta:
         abstract = True
-        required_db_features = ['gis_enabled']
 
     def __str__(self):
         return self.name
@@ -31,7 +26,6 @@ class City(NamedModel):
 
     class Meta:
         app_label = 'geoapp'
-        required_db_features = ['gis_enabled']
 
 
 # This is an inherited model from City
@@ -41,7 +35,6 @@ class PennsylvaniaCity(City):
 
     class Meta:
         app_label = 'geoapp'
-        required_db_features = ['gis_enabled']
 
 
 class State(NamedModel):
@@ -49,7 +42,6 @@ class State(NamedModel):
 
     class Meta:
         app_label = 'geoapp'
-        required_db_features = ['gis_enabled']
 
 
 class Track(NamedModel):
@@ -61,9 +53,6 @@ class MultiFields(NamedModel):
     point = models.PointField()
     poly = models.PolygonField()
 
-    class Meta:
-        required_db_features = ['gis_enabled']
-
 
 class UniqueTogetherModel(models.Model):
     city = models.CharField(max_length=30)
@@ -71,15 +60,12 @@ class UniqueTogetherModel(models.Model):
 
     class Meta:
         unique_together = ('city', 'point')
-        required_db_features = ['gis_enabled', 'supports_geometry_field_unique_index']
+        required_db_features = ['supports_geometry_field_unique_index']
 
 
 class Truth(models.Model):
     val = models.BooleanField(default=False)
 
-    class Meta:
-        required_db_features = ['gis_enabled']
-
 
 class Feature(NamedModel):
     geom = models.GeometryField()
@@ -88,9 +74,6 @@ class Feature(NamedModel):
 class MinusOneSRID(models.Model):
     geom = models.PointField(srid=-1)  # Minus one SRID.
 
-    class Meta:
-        required_db_features = ['gis_enabled']
-
 
 class NonConcreteField(models.IntegerField):
 
@@ -98,10 +81,16 @@ class NonConcreteField(models.IntegerField):
         return None
 
     def get_attname_column(self):
-        attname, column = super(NonConcreteField, self).get_attname_column()
+        attname, column = super().get_attname_column()
         return attname, None
 
 
 class NonConcreteModel(NamedModel):
     non_concrete = NonConcreteField()
     point = models.PointField(geography=True)
+
+
+class ManyPointModel(NamedModel):
+    point1 = models.PointField()
+    point2 = models.PointField()
+    point3 = models.PointField(srid=3857)
