@@ -1,17 +1,11 @@
-from django.utils.encoding import python_2_unicode_compatible
-
-from ..models import models
+from django.contrib.gis.db import models
 
 
-@python_2_unicode_compatible
 class NamedModel(models.Model):
     name = models.CharField(max_length=30)
 
-    objects = models.GeoManager()
-
     class Meta:
         abstract = True
-        required_db_features = ['gis_enabled']
 
     def __str__(self):
         return self.name
@@ -22,7 +16,6 @@ class City(NamedModel):
 
     class Meta:
         app_label = 'geogapp'
-        required_db_features = ['gis_enabled']
 
 
 class Zipcode(NamedModel):
@@ -36,7 +29,6 @@ class County(NamedModel):
 
     class Meta:
         app_label = 'geogapp'
-        required_db_features = ['gis_enabled']
 
     def __str__(self):
         return ' County, '.join([self.name, self.state])
