@@ -1,4 +1,4 @@
-from django.utils.translation import ugettext as _, ungettext
+from django.utils.translation import gettext as _, ngettext
 
 # Translators: This comment should be extracted
 dummy1 = _("This is a translatable string.")
@@ -9,7 +9,7 @@ dummy2 = _("This is another translatable string.")
 # This file has a literal with plural forms. When processed first, makemessages
 # shouldn't create a .po file with duplicate `Plural-Forms` headers
 number = 3
-dummy3 = ungettext("%(number)s Foo", "%(number)s Foos", number) % {'number': number}
+dummy3 = ngettext("%(number)s Foo", "%(number)s Foos", number) % {'number': number}
 
 dummy4 = _('Size')
 
