@@ -1,11 +1,10 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import gettext as gettext_module
 import os
 import stat
 import unittest
+from io import StringIO
 from subprocess import Popen
+from unittest import mock
 
 from django.core.management import (
     CommandError, call_command, execute_from_command_line,
@@ -14,12 +13,10 @@ from django.core.management.commands.makemessages import (
     Command as MakeMessagesCommand,
 )
 from django.core.management.utils import find_command
-from django.test import SimpleTestCase, mock, override_settings
+from django.test import SimpleTestCase, override_settings
 from django.test.utils import captured_stderr, captured_stdout
-from django.utils import six, translation
-from django.utils.encoding import force_text
-from django.utils.six import StringIO
-from django.utils.translation import ugettext
+from django.utils import translation
+from django.utils.translation import gettext
 
 from .utils import RunInTmpDirMixin, copytree
 
@@ -38,9 +35,10 @@ class PoFileTests(MessageCompilationTests):
     MO_FILE = 'locale/%s/LC_MESSAGES/django.mo' % LOCALE
 
     def test_bom_rejection(self):
-        with self.assertRaises(CommandError) as cm:
-            call_command('compilemessages', locale=[self.LOCALE], stdout=StringIO())
-        self.assertIn("file has a BOM (Byte Order Mark)", cm.exception.args[0])
+        stderr = StringIO()
+        with self.assertRaisesMessage(CommandError, 'compilemessages generated one or more errors.'):
+            call_command('compilemessages', locale=[self.LOCALE], stdout=StringIO(), stderr=stderr)
+        self.assertIn('file has a BOM (Byte Order Mark)', stderr.getvalue())
         self.assertFalse(os.path.exists(self.MO_FILE))
 
     def test_no_write_access(self):
@@ -50,9 +48,9 @@ class PoFileTests(MessageCompilationTests):
         old_mode = os.stat(mo_file_en).st_mode
         os.chmod(mo_file_en, stat.S_IREAD)
         try:
-            call_command('compilemessages', locale=['en'], stderr=err_buffer, verbosity=0)
-            err = err_buffer.getvalue()
-            self.assertIn("not writable location", force_text(err))
+            with self.assertRaisesMessage(CommandError, 'compilemessages generated one or more errors.'):
+                call_command('compilemessages', locale=['en'], stderr=err_buffer, verbosity=0)
+            self.assertIn('not writable location', err_buffer.getvalue())
         finally:
             os.chmod(mo_file_en, old_mode)
 
@@ -74,7 +72,7 @@ class MultipleLocaleCompilationTests(MessageCompilationTests):
     MO_FILE_FR = None
 
     def setUp(self):
-        super(MultipleLocaleCompilationTests, self).setUp()
+        super().setUp()
         localedir = os.path.join(self.test_dir, 'locale')
         self.MO_FILE_HR = os.path.join(localedir, 'hr/LC_MESSAGES/django.mo')
         self.MO_FILE_FR = os.path.join(localedir, 'fr/LC_MESSAGES/django.mo')
@@ -100,7 +98,7 @@ class ExcludedLocaleCompilationTests(MessageCompilationTests):
     MO_FILE = 'locale/%s/LC_MESSAGES/django.mo'
 
     def setUp(self):
-        super(ExcludedLocaleCompilationTests, self).setUp()
+        super().setUp()
         copytree('canned_locale', 'locale')
 
     def test_command_help(self):
@@ -140,26 +138,21 @@ class CompilationErrorHandling(MessageCompilationTests):
     def test_error_reported_by_msgfmt(self):
         # po file contains wrong po formatting.
         with self.assertRaises(CommandError):
-            call_command('compilemessages', locale=['ja'], verbosity=0)
+            call_command('compilemessages', locale=['ja'], verbosity=0, stderr=StringIO())
 
     def test_msgfmt_error_including_non_ascii(self):
         # po file contains invalid msgstr content (triggers non-ascii error content).
         # Make sure the output of msgfmt is unaffected by the current locale.
         env = os.environ.copy()
-        env.update({str('LANG'): str('C')})
+        env.update({'LANG': 'C'})
         with mock.patch('django.core.management.utils.Popen', lambda *args, **kwargs: Popen(*args, env=env, **kwargs)):
-            if six.PY2:
-                # Various assertRaises on PY2 don't support unicode error messages.
-                try:
-                    call_command('compilemessages', locale=['ko'], verbosity=0)
-                except CommandError as err:
-                    self.assertIn("' cannot start a field name", six.text_type(err))
-            else:
-                cmd = MakeMessagesCommand()
-                if cmd.gettext_version < (0, 18, 3):
-                    self.skipTest("python-brace-format is a recent gettext addition.")
-                with self.assertRaisesMessage(CommandError, "' cannot start a field name"):
-                    call_command('compilemessages', locale=['ko'], verbosity=0)
+            cmd = MakeMessagesCommand()
+            if cmd.gettext_version < (0, 18, 3):
+                self.skipTest("python-brace-format is a recent gettext addition.")
+            stderr = StringIO()
+            with self.assertRaisesMessage(CommandError, 'compilemessages generated one or more errors'):
+                call_command('compilemessages', locale=['ko'], stdout=StringIO(), stderr=stderr)
+            self.assertIn("' cannot start a field name", stderr.getvalue())
 
 
 class ProjectAndAppTests(MessageCompilationTests):
@@ -171,22 +164,22 @@ class ProjectAndAppTests(MessageCompilationTests):
 class FuzzyTranslationTest(ProjectAndAppTests):
 
     def setUp(self):
-        super(FuzzyTranslationTest, self).setUp()
+        super().setUp()
         gettext_module._translations = {}  # flush cache or test will be useless
 
     def test_nofuzzy_compiling(self):
         with override_settings(LOCALE_PATHS=[os.path.join(self.test_dir, 'locale')]):
             call_command('compilemessages', locale=[self.LOCALE], stdout=StringIO())
             with translation.override(self.LOCALE):
-                self.assertEqual(ugettext('Lenin'), force_text('Ленин'))
-                self.assertEqual(ugettext('Vodka'), force_text('Vodka'))
+                self.assertEqual(gettext('Lenin'), 'Ленин')
+                self.assertEqual(gettext('Vodka'), 'Vodka')
 
     def test_fuzzy_compiling(self):
         with override_settings(LOCALE_PATHS=[os.path.join(self.test_dir, 'locale')]):
             call_command('compilemessages', locale=[self.LOCALE], fuzzy=True, stdout=StringIO())
             with translation.override(self.LOCALE):
-                self.assertEqual(ugettext('Lenin'), force_text('Ленин'))
-                self.assertEqual(ugettext('Vodka'), force_text('Водка'))
+                self.assertEqual(gettext('Lenin'), 'Ленин')
+                self.assertEqual(gettext('Vodka'), 'Водка')
 
 
 class AppCompilationTest(ProjectAndAppTests):
