@@ -1,13 +1,10 @@
-# -*- encoding: utf-8 -*-
-from __future__ import unicode_literals
-
 import unittest
-import warnings
 
 from django.conf import settings
-from django.core.checks import Error
+from django.core.checks import Error, Warning
 from django.core.checks.model_checks import _check_lazy_references
-from django.db import connections, models
+from django.core.exceptions import ImproperlyConfigured
+from django.db import connection, connections, models
 from django.db.models.signals import post_init
 from django.test import SimpleTestCase
 from django.test.utils import isolate_apps, override_settings
@@ -17,16 +14,11 @@ def get_max_column_name_length():
     allowed_len = None
     db_alias = None
 
-    for db in settings.DATABASES.keys():
+    for db in settings.DATABASES:
         connection = connections[db]
         max_name_length = connection.ops.max_name_length()
-        if max_name_length is None or connection.features.truncates_names:
-            continue
-        else:
-            if allowed_len is None:
-                allowed_len = max_name_length
-                db_alias = db
-            elif max_name_length < allowed_len:
+        if max_name_length is not None and not connection.features.truncates_names:
+            if allowed_len is None or max_name_length < allowed_len:
                 allowed_len = max_name_length
                 db_alias = db
 
@@ -41,62 +33,52 @@ class IndexTogetherTests(SimpleTestCase):
             class Meta:
                 index_together = 42
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'index_together' must be a list or tuple.",
                 obj=Model,
                 id='models.E008',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_non_list(self):
         class Model(models.Model):
             class Meta:
                 index_together = 'not-a-list'
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'index_together' must be a list or tuple.",
                 obj=Model,
                 id='models.E008',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_list_containing_non_iterable(self):
         class Model(models.Model):
             class Meta:
                 index_together = [('a', 'b'), 42]
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "All 'index_together' elements must be lists or tuples.",
                 obj=Model,
                 id='models.E009',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_pointing_to_missing_field(self):
         class Model(models.Model):
             class Meta:
-                index_together = [
-                    ["missing_field"],
-                ]
+                index_together = [['missing_field']]
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
-                "'index_together' refers to the non-existent field 'missing_field'.",
+                "'index_together' refers to the nonexistent field 'missing_field'.",
                 obj=Model,
                 id='models.E012',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_pointing_to_non_local_field(self):
         class Foo(models.Model):
@@ -106,41 +88,33 @@ class IndexTogetherTests(SimpleTestCase):
             field2 = models.IntegerField()
 
             class Meta:
-                index_together = [
-                    ["field2", "field1"],
-                ]
+                index_together = [['field2', 'field1']]
 
-        errors = Bar.check()
-        expected = [
+        self.assertEqual(Bar.check(), [
             Error(
                 "'index_together' refers to field 'field1' which is not "
                 "local to model 'Bar'.",
-                hint=("This issue may be caused by multi-table inheritance."),
+                hint='This issue may be caused by multi-table inheritance.',
                 obj=Bar,
                 id='models.E016',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_pointing_to_m2m_field(self):
         class Model(models.Model):
             m2m = models.ManyToManyField('self')
 
             class Meta:
-                index_together = [
-                    ["m2m"],
-                ]
+                index_together = [['m2m']]
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'index_together' refers to a ManyToManyField 'm2m', but "
                 "ManyToManyFields are not permitted in 'index_together'.",
                 obj=Model,
                 id='models.E013',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
 
 # unique_together tests are very similar to index_together tests.
@@ -152,15 +126,13 @@ class UniqueTogetherTests(SimpleTestCase):
             class Meta:
                 unique_together = 42
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'unique_together' must be a list or tuple.",
                 obj=Model,
                 id='models.E010',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_list_containing_non_iterable(self):
         class Model(models.Model):
@@ -170,30 +142,26 @@ class UniqueTogetherTests(SimpleTestCase):
             class Meta:
                 unique_together = [('a', 'b'), 42]
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "All 'unique_together' elements must be lists or tuples.",
                 obj=Model,
                 id='models.E011',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_non_list(self):
         class Model(models.Model):
             class Meta:
                 unique_together = 'not-a-list'
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'unique_together' must be a list or tuple.",
                 obj=Model,
                 id='models.E010',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_valid_model(self):
         class Model(models.Model):
@@ -204,45 +172,89 @@ class UniqueTogetherTests(SimpleTestCase):
                 # unique_together can be a simple tuple
                 unique_together = ('one', 'two')
 
-        errors = Model.check()
-        self.assertEqual(errors, [])
+        self.assertEqual(Model.check(), [])
 
     def test_pointing_to_missing_field(self):
         class Model(models.Model):
             class Meta:
-                unique_together = [
-                    ["missing_field"],
-                ]
+                unique_together = [['missing_field']]
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
-                "'unique_together' refers to the non-existent field 'missing_field'.",
+                "'unique_together' refers to the nonexistent field 'missing_field'.",
                 obj=Model,
                 id='models.E012',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_pointing_to_m2m(self):
         class Model(models.Model):
             m2m = models.ManyToManyField('self')
 
             class Meta:
-                unique_together = [
-                    ["m2m"],
-                ]
+                unique_together = [['m2m']]
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'unique_together' refers to a ManyToManyField 'm2m', but "
                 "ManyToManyFields are not permitted in 'unique_together'.",
                 obj=Model,
                 id='models.E013',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
+
+
+@isolate_apps('invalid_models_tests')
+class IndexesTests(SimpleTestCase):
+
+    def test_pointing_to_missing_field(self):
+        class Model(models.Model):
+            class Meta:
+                indexes = [models.Index(fields=['missing_field'], name='name')]
+
+        self.assertEqual(Model.check(), [
+            Error(
+                "'indexes' refers to the nonexistent field 'missing_field'.",
+                obj=Model,
+                id='models.E012',
+            ),
+        ])
+
+    def test_pointing_to_m2m_field(self):
+        class Model(models.Model):
+            m2m = models.ManyToManyField('self')
+
+            class Meta:
+                indexes = [models.Index(fields=['m2m'], name='name')]
+
+        self.assertEqual(Model.check(), [
+            Error(
+                "'indexes' refers to a ManyToManyField 'm2m', but "
+                "ManyToManyFields are not permitted in 'indexes'.",
+                obj=Model,
+                id='models.E013',
+            ),
+        ])
+
+    def test_pointing_to_non_local_field(self):
+        class Foo(models.Model):
+            field1 = models.IntegerField()
+
+        class Bar(Foo):
+            field2 = models.IntegerField()
+
+            class Meta:
+                indexes = [models.Index(fields=['field2', 'field1'], name='name')]
+
+        self.assertEqual(Bar.check(), [
+            Error(
+                "'indexes' refers to field 'field1' which is not local to "
+                "model 'Bar'.",
+                hint='This issue may be caused by multi-table inheritance.',
+                obj=Bar,
+                id='models.E016',
+            ),
+        ])
 
 
 @isolate_apps('invalid_models_tests')
@@ -253,8 +265,7 @@ class FieldNamesTests(SimpleTestCase):
             field_ = models.CharField(max_length=10)
             m2m_ = models.ManyToManyField('self')
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 'Field names must not end with an underscore.',
                 obj=Model._meta.get_field('field_'),
@@ -265,8 +276,7 @@ class FieldNamesTests(SimpleTestCase):
                 obj=Model._meta.get_field('m2m_'),
                 id='fields.E001',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     max_column_name_length, column_limit_db_alias = get_max_column_name_length()
 
@@ -286,21 +296,21 @@ class FieldNamesTests(SimpleTestCase):
         class ModelWithLongField(models.Model):
             m2m_field = models.ManyToManyField(
                 VeryLongModelNamezzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz,
-                related_name="rn1"
+                related_name='rn1',
             )
             m2m_field2 = models.ManyToManyField(
                 VeryLongModelNamezzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz,
-                related_name="rn2", through='m2msimple'
+                related_name='rn2', through='m2msimple',
             )
             m2m_field3 = models.ManyToManyField(
                 VeryLongModelNamezzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz,
-                related_name="rn3",
-                through='m2mcomplex'
+                related_name='rn3',
+                through='m2mcomplex',
             )
             fk = models.ForeignKey(
                 VeryLongModelNamezzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz,
                 models.CASCADE,
-                related_name="rn4",
+                related_name='rn4',
             )
 
         # Models used for setting `through` in M2M field.
@@ -376,12 +386,7 @@ class FieldNamesTests(SimpleTestCase):
         long_field_name2 = 'b' * (self.max_column_name_length + 1)
         models.CharField(max_length=11).contribute_to_class(ModelWithLongField, long_field_name)
         models.CharField(max_length=11, db_column='vlmn').contribute_to_class(ModelWithLongField, long_field_name2)
-
-        errors = ModelWithLongField.check()
-
-        # Error because of the field with long name added to the model
-        # without specifying db_column
-        expected = [
+        self.assertEqual(ModelWithLongField.check(), [
             Error(
                 'Autogenerated column name too long for field "%s". '
                 'Maximum length is "%s" for database "%s".'
@@ -390,37 +395,46 @@ class FieldNamesTests(SimpleTestCase):
                 obj=ModelWithLongField,
                 id='models.E018',
             )
-        ]
-
-        self.assertEqual(errors, expected)
+        ])
 
     def test_including_separator(self):
         class Model(models.Model):
             some__field = models.IntegerField()
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 'Field names must not contain "__".',
                 obj=Model._meta.get_field('some__field'),
                 id='fields.E002',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_pk(self):
         class Model(models.Model):
             pk = models.IntegerField()
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'pk' is a reserved word that cannot be used as a field name.",
                 obj=Model._meta.get_field('pk'),
                 id='fields.E003',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
+
+    def test_db_column_clash(self):
+        class Model(models.Model):
+            foo = models.IntegerField()
+            bar = models.IntegerField(db_column='foo')
+
+        self.assertEqual(Model.check(), [
+            Error(
+                "Field 'bar' has column name 'foo' that is used by "
+                "another field.",
+                hint="Specify a 'db_column' for the field.",
+                obj=Model,
+                id='models.E007',
+            )
+        ])
 
 
 @isolate_apps('invalid_models_tests')
@@ -433,16 +447,14 @@ class ShadowingFieldsTests(SimpleTestCase):
         class Child(Parent):
             child = models.CharField(max_length=100)
 
-        errors = Child.check()
-        expected = [
+        self.assertEqual(Child.check(), [
             Error(
                 "The field 'child' clashes with the field "
                 "'child' from model 'invalid_models_tests.parent'.",
                 obj=Child._meta.get_field('child'),
                 id='models.E006',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_multiinheritance_clash(self):
         class Mother(models.Model):
@@ -456,8 +468,7 @@ class ShadowingFieldsTests(SimpleTestCase):
             # both parents define these fields.
             pass
 
-        errors = Child.check()
-        expected = [
+        self.assertEqual(Child.check(), [
             Error(
                 "The field 'id' from parent model "
                 "'invalid_models_tests.mother' clashes with the field 'id' "
@@ -472,8 +483,7 @@ class ShadowingFieldsTests(SimpleTestCase):
                 obj=Child,
                 id='models.E005',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_inheritance_clash(self):
         class Parent(models.Model):
@@ -487,16 +497,14 @@ class ShadowingFieldsTests(SimpleTestCase):
             # This field clashes with parent "f_id" field.
             f = models.ForeignKey(Target, models.CASCADE)
 
-        errors = Child.check()
-        expected = [
+        self.assertEqual(Child.check(), [
             Error(
                 "The field 'f' clashes with the field 'f_id' "
                 "from model 'invalid_models_tests.parent'.",
                 obj=Child._meta.get_field('f'),
                 id='models.E006',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_multigeneration_inheritance(self):
         class GrandParent(models.Model):
@@ -511,16 +519,14 @@ class ShadowingFieldsTests(SimpleTestCase):
         class GrandChild(Child):
             clash = models.IntegerField()
 
-        errors = GrandChild.check()
-        expected = [
+        self.assertEqual(GrandChild.check(), [
             Error(
                 "The field 'clash' clashes with the field 'clash' "
                 "from model 'invalid_models_tests.grandparent'.",
                 obj=GrandChild._meta.get_field('clash'),
                 id='models.E006',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_id_clash(self):
         class Target(models.Model):
@@ -530,16 +536,14 @@ class ShadowingFieldsTests(SimpleTestCase):
             fk = models.ForeignKey(Target, models.CASCADE)
             fk_id = models.IntegerField()
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "The field 'fk_id' clashes with the field 'fk' from model "
                 "'invalid_models_tests.model'.",
                 obj=Model._meta.get_field('fk_id'),
                 id='models.E006',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
 
 @isolate_apps('invalid_models_tests')
@@ -551,32 +555,28 @@ class OtherModelTests(SimpleTestCase):
         class Model(models.Model):
             id = invalid_id
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'id' can only be used as a field name if the field also sets "
                 "'primary_key=True'.",
                 obj=Model,
                 id='models.E004',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_ordering_non_iterable(self):
         class Model(models.Model):
             class Meta:
-                ordering = "missing_field"
+                ordering = 'missing_field'
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'ordering' must be a tuple or list "
                 "(even if you want to order by only one field).",
                 obj=Model,
                 id='models.E014',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_just_ordering_no_errors(self):
         class Model(models.Model):
@@ -611,15 +611,13 @@ class OtherModelTests(SimpleTestCase):
                 order_with_respect_to = 'question'
                 ordering = ['order']
 
-        errors = Answer.check()
-        expected = [
+        self.assertEqual(Answer.check(), [
             Error(
                 "'ordering' and 'order_with_respect_to' cannot be used together.",
                 obj=Answer,
                 id='models.E021',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_non_valid(self):
         class RelationModel(models.Model):
@@ -631,53 +629,43 @@ class OtherModelTests(SimpleTestCase):
             class Meta:
                 ordering = ['relation']
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
-                "'ordering' refers to the non-existent field 'relation'.",
+                "'ordering' refers to the nonexistent field 'relation'.",
                 obj=Model,
                 id='models.E015',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_ordering_pointing_to_missing_field(self):
         class Model(models.Model):
             class Meta:
-                ordering = ("missing_field",)
+                ordering = ('missing_field',)
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
-                "'ordering' refers to the non-existent field 'missing_field'.",
+                "'ordering' refers to the nonexistent field 'missing_field'.",
                 obj=Model,
                 id='models.E015',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_ordering_pointing_to_missing_foreignkey_field(self):
-        # refs #22711
-
         class Model(models.Model):
             missing_fk_field = models.IntegerField()
 
             class Meta:
-                ordering = ("missing_fk_field_id",)
+                ordering = ('missing_fk_field_id',)
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
-                "'ordering' refers to the non-existent field 'missing_fk_field_id'.",
+                "'ordering' refers to the nonexistent field 'missing_fk_field_id'.",
                 obj=Model,
                 id='models.E015',
             )
-        ]
-        self.assertEqual(errors, expected)
-
-    def test_ordering_pointing_to_existing_foreignkey_field(self):
-        # refs #22711
+        ])
 
+    def test_ordering_pointing_to_foreignkey_field(self):
         class Parent(models.Model):
             pass
 
@@ -685,7 +673,7 @@ class OtherModelTests(SimpleTestCase):
             parent = models.ForeignKey(Parent, models.CASCADE)
 
             class Meta:
-                ordering = ("parent_id",)
+                ordering = ('parent_id',)
 
         self.assertFalse(Child.check())
 
@@ -728,20 +716,47 @@ class OtherModelTests(SimpleTestCase):
             )
         ])
 
+    def test_property_and_related_field_accessor_clash(self):
+        class Model(models.Model):
+            fk = models.ForeignKey('self', models.CASCADE)
+
+            @property
+            def fk_id(self):
+                pass
+
+        self.assertEqual(Model.check(), [
+            Error(
+                "The property 'fk_id' clashes with a related field accessor.",
+                obj=Model,
+                id='models.E025',
+            )
+        ])
+
+    def test_single_primary_key(self):
+        class Model(models.Model):
+            foo = models.IntegerField(primary_key=True)
+            bar = models.IntegerField(primary_key=True)
+
+        self.assertEqual(Model.check(), [
+            Error(
+                "The model cannot have more than one field with 'primary_key=True'.",
+                obj=Model,
+                id='models.E026',
+            )
+        ])
+
     @override_settings(TEST_SWAPPED_MODEL_BAD_VALUE='not-a-model')
     def test_swappable_missing_app_name(self):
         class Model(models.Model):
             class Meta:
                 swappable = 'TEST_SWAPPED_MODEL_BAD_VALUE'
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'TEST_SWAPPED_MODEL_BAD_VALUE' is not of the form 'app_label.app_name'.",
                 id='models.E001',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     @override_settings(TEST_SWAPPED_MODEL_BAD_MODEL='not_an_app.Target')
     def test_swappable_missing_app(self):
@@ -749,60 +764,66 @@ class OtherModelTests(SimpleTestCase):
             class Meta:
                 swappable = 'TEST_SWAPPED_MODEL_BAD_MODEL'
 
-        errors = Model.check()
-        expected = [
+        self.assertEqual(Model.check(), [
             Error(
                 "'TEST_SWAPPED_MODEL_BAD_MODEL' references 'not_an_app.Target', "
                 'which has not been installed, or is abstract.',
                 id='models.E002',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_two_m2m_through_same_relationship(self):
         class Person(models.Model):
             pass
 
         class Group(models.Model):
-            primary = models.ManyToManyField(Person, through="Membership", related_name="primary")
-            secondary = models.ManyToManyField(Person, through="Membership", related_name="secondary")
+            primary = models.ManyToManyField(Person, through='Membership', related_name='primary')
+            secondary = models.ManyToManyField(Person, through='Membership', related_name='secondary')
 
         class Membership(models.Model):
             person = models.ForeignKey(Person, models.CASCADE)
             group = models.ForeignKey(Group, models.CASCADE)
 
-        errors = Group.check()
-        expected = [
+        self.assertEqual(Group.check(), [
             Error(
-                "The model has two many-to-many relations through "
+                "The model has two identical many-to-many relations through "
                 "the intermediate model 'invalid_models_tests.Membership'.",
                 obj=Group,
                 id='models.E003',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
+
+    def test_two_m2m_through_same_model_with_different_through_fields(self):
+        class Country(models.Model):
+            pass
+
+        class ShippingMethod(models.Model):
+            to_countries = models.ManyToManyField(
+                Country, through='ShippingMethodPrice',
+                through_fields=('method', 'to_country'),
+            )
+            from_countries = models.ManyToManyField(
+                Country, through='ShippingMethodPrice',
+                through_fields=('method', 'from_country'),
+                related_name='+',
+            )
+
+        class ShippingMethodPrice(models.Model):
+            method = models.ForeignKey(ShippingMethod, models.CASCADE)
+            to_country = models.ForeignKey(Country, models.CASCADE)
+            from_country = models.ForeignKey(Country, models.CASCADE)
+
+        self.assertEqual(ShippingMethod.check(), [])
 
     def test_missing_parent_link(self):
-        with warnings.catch_warnings(record=True) as warns:
-            warnings.simplefilter('always')
-
+        msg = 'Add parent_link=True to invalid_models_tests.ParkingLot.parent.'
+        with self.assertRaisesMessage(ImproperlyConfigured, msg):
             class Place(models.Model):
                 pass
 
             class ParkingLot(Place):
-                # In lieu of any other connector, an existing OneToOneField will be
-                # promoted to the primary key.
                 parent = models.OneToOneField(Place, models.CASCADE)
 
-        self.assertEqual(len(warns), 1)
-        msg = str(warns[0].message)
-        self.assertEqual(
-            msg,
-            'Add parent_link=True to invalid_models_tests.ParkingLot.parent '
-            'as an implicit link is deprecated.'
-        )
-        self.assertEqual(ParkingLot._meta.pk.name, 'parent')
-
     def test_m2m_table_name_clash(self):
         class Foo(models.Model):
             bar = models.ManyToManyField('Bar', db_table='myapp_bar')
@@ -925,7 +946,7 @@ class OtherModelTests(SimpleTestCase):
             class Meta:
                 app_label = 'invalid_models_tests'
 
-        class DummyClass(object):
+        class DummyClass:
             def __call__(self, **kwargs):
                 pass
 
@@ -942,7 +963,7 @@ class OtherModelTests(SimpleTestCase):
         post_init.connect(DummyClass(), sender='missing-app.Model', apps=apps)
         post_init.connect(DummyClass().dummy_method, sender='missing-app.Model', apps=apps)
 
-        expected = [
+        self.assertEqual(_check_lazy_references(apps), [
             Error(
                 "%r contains a lazy reference to auth.imaginarymodel, "
                 "but app 'auth' doesn't provide model 'imaginarymodel'." % dummy_function,
@@ -987,5 +1008,27 @@ class OtherModelTests(SimpleTestCase):
                 obj='invalid_models_tests.test_models',
                 id='signals.E001',
             ),
-        ]
-        self.assertEqual(_check_lazy_references(apps), expected)
+        ])
+
+
+@isolate_apps('invalid_models_tests')
+class ConstraintsTests(SimpleTestCase):
+    def test_check_constraints(self):
+        class Model(models.Model):
+            age = models.IntegerField()
+
+            class Meta:
+                constraints = [models.CheckConstraint(check=models.Q(age__gte=18), name='is_adult')]
+
+        errors = Model.check()
+        warn = Warning(
+            '%s does not support check constraints.' % connection.display_name,
+            hint=(
+                "A constraint won't be created. Silence this warning if you "
+                "don't care about it."
+            ),
+            obj=Model,
+            id='models.W027',
+        )
+        expected = [] if connection.features.supports_table_check_constraints else [warn, warn]
+        self.assertCountEqual(errors, expected)
