@@ -1,13 +1,12 @@
-# -*- encoding: utf-8 -*-
-from __future__ import unicode_literals
-
 import unittest
 
 from django.core.checks import Error, Warning as DjangoWarning
 from django.db import connection, models
-from django.test import SimpleTestCase, TestCase
+from django.test import SimpleTestCase, TestCase, skipIfDBFeature
 from django.test.utils import isolate_apps, override_settings
+from django.utils.functional import lazy
 from django.utils.timezone import now
+from django.utils.translation import gettext_lazy as _
 
 
 @isolate_apps('invalid_models_tests')
@@ -18,9 +17,7 @@ class AutoFieldTests(SimpleTestCase):
             id = models.AutoField(primary_key=True)
 
         field = Model._meta.get_field('id')
-        errors = field.check()
-        expected = []
-        self.assertEqual(errors, expected)
+        self.assertEqual(field.check(), [])
 
     def test_primary_key(self):
         # primary_key must be True. Refs #12467.
@@ -33,39 +30,17 @@ class AutoFieldTests(SimpleTestCase):
             another = models.IntegerField(primary_key=True)
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 'AutoFields must set primary_key=True.',
                 obj=field,
                 id='fields.E100',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
 
 @isolate_apps('invalid_models_tests')
-class BooleanFieldTests(SimpleTestCase):
-
-    def test_nullable_boolean_field(self):
-        class Model(models.Model):
-            field = models.BooleanField(null=True)
-
-        field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
-            Error(
-                'BooleanFields do not accept null values.',
-                hint='Use a NullBooleanField instead.',
-                obj=field,
-                id='fields.E110',
-            ),
-        ]
-        self.assertEqual(errors, expected)
-
-
-@isolate_apps('invalid_models_tests')
-class CharFieldTests(TestCase):
+class CharFieldTests(SimpleTestCase):
 
     def test_valid_field(self):
         class Model(models.Model):
@@ -75,90 +50,107 @@ class CharFieldTests(TestCase):
                     ('1', 'item1'),
                     ('2', 'item2'),
                 ],
-                db_index=True)
+                db_index=True,
+            )
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = []
-        self.assertEqual(errors, expected)
+        self.assertEqual(field.check(), [])
 
     def test_missing_max_length(self):
         class Model(models.Model):
             field = models.CharField()
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "CharFields must define a 'max_length' attribute.",
                 obj=field,
                 id='fields.E120',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_negative_max_length(self):
         class Model(models.Model):
             field = models.CharField(max_length=-1)
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "'max_length' must be a positive integer.",
                 obj=field,
                 id='fields.E121',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_bad_max_length_value(self):
         class Model(models.Model):
             field = models.CharField(max_length="bad")
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "'max_length' must be a positive integer.",
                 obj=field,
                 id='fields.E121',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_str_max_length_value(self):
         class Model(models.Model):
             field = models.CharField(max_length='20')
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "'max_length' must be a positive integer.",
                 obj=field,
                 id='fields.E121',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
+
+    def test_str_max_length_type(self):
+        class Model(models.Model):
+            field = models.CharField(max_length=True)
+
+        field = Model._meta.get_field('field')
+        self.assertEqual(field.check(), [
+            Error(
+                "'max_length' must be a positive integer.",
+                obj=field,
+                id='fields.E121'
+            ),
+        ])
 
     def test_non_iterable_choices(self):
         class Model(models.Model):
             field = models.CharField(max_length=10, choices='bad')
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "'choices' must be an iterable (e.g., a list or tuple).",
                 obj=field,
                 id='fields.E004',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
+
+    def test_non_iterable_choices_two_letters(self):
+        """Two letters isn't a valid choice pair."""
+        class Model(models.Model):
+            field = models.CharField(max_length=10, choices=['ab'])
+
+        field = Model._meta.get_field('field')
+        self.assertEqual(field.check(), [
+            Error(
+                "'choices' must be an iterable containing (actual value, "
+                "human readable name) tuples.",
+                obj=field,
+                id='fields.E005',
+            ),
+        ])
 
     def test_iterable_of_iterable_choices(self):
-        class ThingItem(object):
+        class ThingItem:
             def __init__(self, value, display):
                 self.value = value
                 self.display = display
@@ -169,7 +161,7 @@ class CharFieldTests(TestCase):
             def __len__(self):
                 return 2
 
-        class Things(object):
+        class Things:
             def __iter__(self):
                 return (x for x in [ThingItem(1, 2), ThingItem(3, 4)])
 
@@ -182,31 +174,123 @@ class CharFieldTests(TestCase):
         class Model(models.Model):
             field = models.CharField(max_length=10, choices=[(1, 2, 3), (1, 2, 3)])
 
+        class Model2(models.Model):
+            field = models.IntegerField(choices=[0])
+
+        for model in (Model, Model2):
+            with self.subTest(model.__name__):
+                field = model._meta.get_field('field')
+                self.assertEqual(field.check(), [
+                    Error(
+                        "'choices' must be an iterable containing (actual "
+                        "value, human readable name) tuples.",
+                        obj=field,
+                        id='fields.E005',
+                    ),
+                ])
+
+    def test_choices_containing_lazy(self):
+        class Model(models.Model):
+            field = models.CharField(max_length=10, choices=[['1', _('1')], ['2', _('2')]])
+
+        self.assertEqual(Model._meta.get_field('field').check(), [])
+
+    def test_lazy_choices(self):
+        class Model(models.Model):
+            field = models.CharField(max_length=10, choices=lazy(lambda: [[1, '1'], [2, '2']], tuple)())
+
+        self.assertEqual(Model._meta.get_field('field').check(), [])
+
+    def test_choices_named_group(self):
+        class Model(models.Model):
+            field = models.CharField(
+                max_length=10, choices=[
+                    ['knights', [['L', 'Lancelot'], ['G', 'Galahad']]],
+                    ['wizards', [['T', 'Tim the Enchanter']]],
+                    ['R', 'Random character'],
+                ],
+            )
+
+        self.assertEqual(Model._meta.get_field('field').check(), [])
+
+    def test_choices_named_group_non_pairs(self):
+        class Model(models.Model):
+            field = models.CharField(
+                max_length=10,
+                choices=[['knights', [['L', 'Lancelot', 'Du Lac']]]],
+            )
+
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
-                "'choices' must be an iterable containing (actual value, human readable name) tuples.",
+                "'choices' must be an iterable containing (actual value, "
+                "human readable name) tuples.",
                 obj=field,
                 id='fields.E005',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
+
+    def test_choices_named_group_bad_structure(self):
+        class Model(models.Model):
+            field = models.CharField(
+                max_length=10, choices=[
+                    ['knights', [
+                        ['Noble', [['G', 'Galahad']]],
+                        ['Combative', [['L', 'Lancelot']]],
+                    ]],
+                ],
+            )
+
+        field = Model._meta.get_field('field')
+        self.assertEqual(field.check(), [
+            Error(
+                "'choices' must be an iterable containing (actual value, "
+                "human readable name) tuples.",
+                obj=field,
+                id='fields.E005',
+            ),
+        ])
+
+    def test_choices_named_group_lazy(self):
+        class Model(models.Model):
+            field = models.CharField(
+                max_length=10, choices=[
+                    [_('knights'), [['L', _('Lancelot')], ['G', _('Galahad')]]],
+                    ['R', _('Random character')],
+                ],
+            )
+
+        self.assertEqual(Model._meta.get_field('field').check(), [])
 
     def test_bad_db_index_value(self):
         class Model(models.Model):
             field = models.CharField(max_length=10, db_index='bad')
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "'db_index' must be None, True or False.",
                 obj=field,
                 id='fields.E006',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
+
+    def test_bad_validators(self):
+        class Model(models.Model):
+            field = models.CharField(max_length=10, validators=[True])
+
+        field = Model._meta.get_field('field')
+        self.assertEqual(field.check(), [
+            Error(
+                "All 'validators' must be callable.",
+                hint=(
+                    "validators[0] (True) isn't a function or instance of a "
+                    "validator class."
+                ),
+                obj=field,
+                id='fields.E008',
+            ),
+        ])
 
     @unittest.skipUnless(connection.vendor == 'mysql',
                          "Test valid only for MySQL")
@@ -218,19 +302,18 @@ class CharFieldTests(TestCase):
 
         field = Model._meta.get_field('field')
         validator = DatabaseValidation(connection=connection)
-        errors = validator.check_field(field)
-        expected = [
+        self.assertEqual(validator.check_field(field), [
             Error(
                 'MySQL does not allow unique CharFields to have a max_length > 255.',
                 obj=field,
                 id='mysql.E001',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
 
 @isolate_apps('invalid_models_tests')
-class DateFieldTests(TestCase):
+class DateFieldTests(SimpleTestCase):
+    maxDiff = None
 
     def test_auto_now_and_auto_now_add_raise_error(self):
         class Model(models.Model):
@@ -265,7 +348,7 @@ class DateFieldTests(TestCase):
         errors = field_dt.check()
         errors.extend(field_d.check())
         errors.extend(field_now.check())  # doesn't raise a warning
-        expected = [
+        self.assertEqual(errors, [
             DjangoWarning(
                 'Fixed default value provided.',
                 hint='It seems you set a fixed date / time / datetime '
@@ -284,11 +367,7 @@ class DateFieldTests(TestCase):
                 obj=field_d,
                 id='fields.W161',
             )
-        ]
-        maxDiff = self.maxDiff
-        self.maxDiff = None
-        self.assertEqual(errors, expected)
-        self.maxDiff = maxDiff
+        ])
 
     @override_settings(USE_TZ=True)
     def test_fix_default_value_tz(self):
@@ -296,7 +375,8 @@ class DateFieldTests(TestCase):
 
 
 @isolate_apps('invalid_models_tests')
-class DateTimeFieldTests(TestCase):
+class DateTimeFieldTests(SimpleTestCase):
+    maxDiff = None
 
     def test_fix_default_value(self):
         class Model(models.Model):
@@ -310,7 +390,7 @@ class DateTimeFieldTests(TestCase):
         errors = field_dt.check()
         errors.extend(field_d.check())
         errors.extend(field_now.check())  # doesn't raise a warning
-        expected = [
+        self.assertEqual(errors, [
             DjangoWarning(
                 'Fixed default value provided.',
                 hint='It seems you set a fixed date / time / datetime '
@@ -329,11 +409,7 @@ class DateTimeFieldTests(TestCase):
                 obj=field_d,
                 id='fields.W161',
             )
-        ]
-        maxDiff = self.maxDiff
-        self.maxDiff = None
-        self.assertEqual(errors, expected)
-        self.maxDiff = maxDiff
+        ])
 
     @override_settings(USE_TZ=True)
     def test_fix_default_value_tz(self):
@@ -348,8 +424,7 @@ class DecimalFieldTests(SimpleTestCase):
             field = models.DecimalField()
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "DecimalFields must define a 'decimal_places' attribute.",
                 obj=field,
@@ -360,16 +435,14 @@ class DecimalFieldTests(SimpleTestCase):
                 obj=field,
                 id='fields.E132',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_negative_max_digits_and_decimal_places(self):
         class Model(models.Model):
             field = models.DecimalField(max_digits=-1, decimal_places=-1)
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "'decimal_places' must be a non-negative integer.",
                 obj=field,
@@ -380,16 +453,14 @@ class DecimalFieldTests(SimpleTestCase):
                 obj=field,
                 id='fields.E133',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_bad_values_of_max_digits_and_decimal_places(self):
         class Model(models.Model):
             field = models.DecimalField(max_digits="bad", decimal_places="bad")
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "'decimal_places' must be a non-negative integer.",
                 obj=field,
@@ -400,32 +471,27 @@ class DecimalFieldTests(SimpleTestCase):
                 obj=field,
                 id='fields.E133',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_decimal_places_greater_than_max_digits(self):
         class Model(models.Model):
             field = models.DecimalField(max_digits=9, decimal_places=10)
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "'max_digits' must be greater or equal to 'decimal_places'.",
                 obj=field,
                 id='fields.E134',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_valid_field(self):
         class Model(models.Model):
             field = models.DecimalField(max_digits=10, decimal_places=10)
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = []
-        self.assertEqual(errors, expected)
+        self.assertEqual(field.check(), [])
 
 
 @isolate_apps('invalid_models_tests')
@@ -442,24 +508,20 @@ class FileFieldTests(SimpleTestCase):
             field = models.FileField(upload_to='somewhere')
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = []
-        self.assertEqual(errors, expected)
+        self.assertEqual(field.check(), [])
 
     def test_primary_key(self):
         class Model(models.Model):
             field = models.FileField(primary_key=False, upload_to='somewhere')
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "'primary_key' is not a valid argument for a FileField.",
                 obj=field,
                 id='fields.E201',
             )
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
     def test_upload_to_starts_with_slash(self):
         class Model(models.Model):
@@ -495,15 +557,13 @@ class FilePathFieldTests(SimpleTestCase):
             field = models.FilePathField(allow_files=False, allow_folders=False)
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 "FilePathFields must have either 'allow_files' or 'allow_folders' set to True.",
                 obj=field,
                 id='fields.E140',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
 
 @isolate_apps('invalid_models_tests')
@@ -514,16 +574,14 @@ class GenericIPAddressFieldTests(SimpleTestCase):
             field = models.GenericIPAddressField(null=False, blank=True)
 
         field = Model._meta.get_field('field')
-        errors = field.check()
-        expected = [
+        self.assertEqual(field.check(), [
             Error(
                 ('GenericIPAddressFields cannot have blank=True if null=False, '
                  'as blank values are stored as nulls.'),
                 obj=field,
                 id='fields.E150',
             ),
-        ]
-        self.assertEqual(errors, expected)
+        ])
 
 
 @isolate_apps('invalid_models_tests')
@@ -545,7 +603,7 @@ class ImageFieldTests(SimpleTestCase):
         expected = [] if pillow_installed else [
             Error(
                 'Cannot use ImageField because Pillow is not installed.',
-                hint=('Get Pillow at https://pypi.python.org/pypi/Pillow '
+                hint=('Get Pillow at https://pypi.org/project/Pillow/ '
                       'or run command "pip install Pillow".'),
                 obj=field,
                 id='fields.E210',
@@ -559,23 +617,29 @@ class IntegerFieldTests(SimpleTestCase):
 
     def test_max_length_warning(self):
         class Model(models.Model):
-            value = models.IntegerField(max_length=2)
+            integer = models.IntegerField(max_length=2)
+            biginteger = models.BigIntegerField(max_length=2)
+            smallinteger = models.SmallIntegerField(max_length=2)
+            positiveinteger = models.PositiveIntegerField(max_length=2)
+            positivesmallinteger = models.PositiveSmallIntegerField(max_length=2)
 
-        value = Model._meta.get_field('value')
-        errors = Model.check()
-        expected = [
-            DjangoWarning(
-                "'max_length' is ignored when used with IntegerField",
-                hint="Remove 'max_length' from field",
-                obj=value,
-                id='fields.W122',
-            )
-        ]
-        self.assertEqual(errors, expected)
+        for field in Model._meta.get_fields():
+            if field.auto_created:
+                continue
+            with self.subTest(name=field.name):
+                self.assertEqual(field.check(), [
+                    DjangoWarning(
+                        "'max_length' is ignored when used with %s." % field.__class__.__name__,
+                        hint="Remove 'max_length' from field",
+                        obj=field,
+                        id='fields.W122',
+                    )
+                ])
 
 
 @isolate_apps('invalid_models_tests')
-class TimeFieldTests(TestCase):
+class TimeFieldTests(SimpleTestCase):
+    maxDiff = None
 
     def test_fix_default_value(self):
         class Model(models.Model):
@@ -589,7 +653,7 @@ class TimeFieldTests(TestCase):
         errors = field_dt.check()
         errors.extend(field_t.check())
         errors.extend(field_now.check())  # doesn't raise a warning
-        expected = [
+        self.assertEqual(errors, [
             DjangoWarning(
                 'Fixed default value provided.',
                 hint='It seems you set a fixed date / time / datetime '
@@ -608,12 +672,31 @@ class TimeFieldTests(TestCase):
                 obj=field_t,
                 id='fields.W161',
             )
-        ]
-        maxDiff = self.maxDiff
-        self.maxDiff = None
-        self.assertEqual(errors, expected)
-        self.maxDiff = maxDiff
+        ])
 
     @override_settings(USE_TZ=True)
     def test_fix_default_value_tz(self):
         self.test_fix_default_value()
+
+
+@isolate_apps('invalid_models_tests')
+class TextFieldTests(TestCase):
+
+    @skipIfDBFeature('supports_index_on_text_field')
+    def test_max_length_warning(self):
+        class Model(models.Model):
+            value = models.TextField(db_index=True)
+        field = Model._meta.get_field('value')
+        field_type = field.db_type(connection)
+        self.assertEqual(field.check(), [
+            DjangoWarning(
+                '%s does not support a database index on %s columns.'
+                % (connection.display_name, field_type),
+                hint=(
+                    "An index won't be created. Silence this warning if you "
+                    "don't care about it."
+                ),
+                obj=field,
+                id='fields.W162',
+            )
+        ])
