@@ -6,24 +6,19 @@ To define a many-to-many relationship, use ``ManyToManyField()``.
 In this example, an ``Article`` can be published in multiple ``Publication``
 objects, and a ``Publication`` has multiple ``Article`` objects.
 """
-from __future__ import unicode_literals
-
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
-@python_2_unicode_compatible
 class Publication(models.Model):
     title = models.CharField(max_length=30)
 
-    def __str__(self):
-        return self.title
-
     class Meta:
         ordering = ('title',)
 
+    def __str__(self):
+        return self.title
+
 
-@python_2_unicode_compatible
 class Tag(models.Model):
     id = models.BigAutoField(primary_key=True)
     name = models.CharField(max_length=50)
@@ -32,20 +27,26 @@ class Tag(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
+class NoDeletedArticleManager(models.Manager):
+    def get_queryset(self):
+        return super().get_queryset().exclude(headline='deleted')
+
+
 class Article(models.Model):
     headline = models.CharField(max_length=100)
-    # Assign a unicode string as name to make sure the intermediary model is
+    # Assign a string as name to make sure the intermediary model is
     # correctly created. Refs #20207
     publications = models.ManyToManyField(Publication, name='publications')
     tags = models.ManyToManyField(Tag, related_name='tags')
 
-    def __str__(self):
-        return self.headline
+    objects = NoDeletedArticleManager()
 
     class Meta:
         ordering = ('headline',)
 
+    def __str__(self):
+        return self.headline
+
 
 # Models to test correct related_name inheritance
 class AbstractArticle(models.Model):
