@@ -1,19 +1,18 @@
 import unittest
 
-from django import http
 from django.contrib.messages.middleware import MessageMiddleware
+from django.http import HttpRequest, HttpResponse
 
 
-class MiddlewareTest(unittest.TestCase):
+class MiddlewareTests(unittest.TestCase):
 
     def setUp(self):
         self.middleware = MessageMiddleware()
 
     def test_response_without_messages(self):
         """
-        Makes sure that the response middleware is tolerant of messages not
-        existing on request.
+        MessageMiddleware is tolerant of messages not existing on request.
         """
-        request = http.HttpRequest()
-        response = http.HttpResponse()
+        request = HttpRequest()
+        response = HttpResponse()
         self.middleware.process_response(request, response)
