@@ -1,11 +1,9 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 import gzip
 import random
 import re
+import struct
 from io import BytesIO
-from unittest import skipIf
+from urllib.parse import quote
 
 from django.conf import settings
 from django.core import mail
@@ -21,10 +19,8 @@ from django.middleware.common import (
 from django.middleware.gzip import GZipMiddleware
 from django.middleware.http import ConditionalGetMiddleware
 from django.test import RequestFactory, SimpleTestCase, override_settings
-from django.utils import six
-from django.utils.encoding import force_str
-from django.utils.six.moves import range
-from django.utils.six.moves.urllib.parse import quote
+
+int2byte = struct.Struct(">B").pack
 
 
 @override_settings(ROOT_URLCONF='middleware.urls')
@@ -80,10 +76,22 @@ class CommonMiddlewareTest(SimpleTestCase):
         r = CommonMiddleware().process_response(request, response)
         self.assertEqual(r.url, '/slash/?test=1')
 
+    @override_settings(APPEND_SLASH=True)
+    def test_append_slash_redirect_querystring_have_slash(self):
+        """
+        APPEND_SLASH should append slash to path when redirecting a request
+        with a querystring ending with slash.
+        """
+        request = self.rf.get('/slash?test=slash/')
+        response = HttpResponseNotFound()
+        r = CommonMiddleware().process_response(request, response)
+        self.assertIsInstance(r, HttpResponsePermanentRedirect)
+        self.assertEqual(r.url, '/slash/?test=slash/')
+
     @override_settings(APPEND_SLASH=True, DEBUG=True)
     def test_append_slash_no_redirect_on_POST_in_DEBUG(self):
         """
-        Tests that while in debug mode, an exception is raised with a warning
+        While in debug mode, an exception is raised with a warning
         when a failed attempt is made to POST, PUT, or PATCH to an URL which
         would normally be redirected to a slashed version.
         """
@@ -122,6 +130,25 @@ class CommonMiddlewareTest(SimpleTestCase):
         self.assertEqual(r.status_code, 301)
         self.assertEqual(r.url, '/needsquoting%23/')
 
+    @override_settings(APPEND_SLASH=True)
+    def test_append_slash_leading_slashes(self):
+        """
+        Paths starting with two slashes are escaped to prevent open redirects.
+        If there's a URL pattern that allows paths to start with two slashes, a
+        request with path //evil.com must not redirect to //evil.com/ (appended
+        slash) which is a schemaless absolute URL. The browser would navigate
+        to evil.com/.
+        """
+        # Use 4 slashes because of RequestFactory behavior.
+        request = self.rf.get('////evil.com/security')
+        response = HttpResponseNotFound()
+        r = CommonMiddleware().process_request(request)
+        self.assertEqual(r.status_code, 301)
+        self.assertEqual(r.url, '/%2Fevil.com/security/')
+        r = CommonMiddleware().process_response(request, response)
+        self.assertEqual(r.status_code, 301)
+        self.assertEqual(r.url, '/%2Fevil.com/security/')
+
     @override_settings(APPEND_SLASH=False, PREPEND_WWW=True)
     def test_prepend_www(self):
         request = self.rf.get('/path/')
@@ -195,7 +222,7 @@ class CommonMiddlewareTest(SimpleTestCase):
     @override_settings(APPEND_SLASH=True, DEBUG=True)
     def test_append_slash_no_redirect_on_POST_in_DEBUG_custom_urlconf(self):
         """
-        Tests that while in debug mode, an exception is raised with a warning
+        While in debug mode, an exception is raised with a warning
         when a failed attempt is made to POST to an URL which would normally be
         redirected to a slashed version.
         """
@@ -254,36 +281,26 @@ class CommonMiddlewareTest(SimpleTestCase):
         self.assertEqual(r.status_code, 301)
         self.assertEqual(r.url, 'http://www.testserver/customurlconf/slash/')
 
-    # ETag + If-Not-Modified support tests
+    # Tests for the Content-Length header
 
-    @override_settings(USE_ETAGS=True)
-    def test_etag(self):
-        req = HttpRequest()
-        res = HttpResponse('content')
-        self.assertTrue(CommonMiddleware().process_response(req, res).has_header('ETag'))
+    def test_content_length_header_added(self):
+        response = HttpResponse('content')
+        self.assertNotIn('Content-Length', response)
+        response = CommonMiddleware().process_response(HttpRequest(), response)
+        self.assertEqual(int(response['Content-Length']), len(response.content))
 
-    @override_settings(USE_ETAGS=True)
-    def test_etag_streaming_response(self):
-        req = HttpRequest()
-        res = StreamingHttpResponse(['content'])
-        res['ETag'] = 'tomatoes'
-        self.assertEqual(CommonMiddleware().process_response(req, res).get('ETag'), 'tomatoes')
+    def test_content_length_header_not_added_for_streaming_response(self):
+        response = StreamingHttpResponse('content')
+        self.assertNotIn('Content-Length', response)
+        response = CommonMiddleware().process_response(HttpRequest(), response)
+        self.assertNotIn('Content-Length', response)
 
-    @override_settings(USE_ETAGS=True)
-    def test_no_etag_streaming_response(self):
-        req = HttpRequest()
-        res = StreamingHttpResponse(['content'])
-        self.assertFalse(CommonMiddleware().process_response(req, res).has_header('ETag'))
-
-    @override_settings(USE_ETAGS=True)
-    def test_if_none_match(self):
-        first_req = HttpRequest()
-        first_res = CommonMiddleware().process_response(first_req, HttpResponse('content'))
-        second_req = HttpRequest()
-        second_req.method = 'GET'
-        second_req.META['HTTP_IF_NONE_MATCH'] = first_res['ETag']
-        second_res = CommonMiddleware().process_response(second_req, HttpResponse('content'))
-        self.assertEqual(second_res.status_code, 304)
+    def test_content_length_header_not_changed(self):
+        response = HttpResponse()
+        bad_content_length = len(response.content) + 10
+        response['Content-Length'] = bad_content_length
+        response = CommonMiddleware().process_response(HttpRequest(), response)
+        self.assertEqual(int(response['Content-Length']), bad_content_length)
 
     # Other tests
 
@@ -297,7 +314,7 @@ class CommonMiddlewareTest(SimpleTestCase):
     def test_non_ascii_query_string_does_not_crash(self):
         """Regression test for #15152"""
         request = self.rf.get('/slash')
-        request.META['QUERY_STRING'] = force_str('drink=café')
+        request.META['QUERY_STRING'] = 'drink=café'
         r = CommonMiddleware().process_request(request)
         self.assertEqual(r.status_code, 301)
 
@@ -348,31 +365,13 @@ class BrokenLinkEmailsMiddlewareTest(SimpleTestCase):
         BrokenLinkEmailsMiddleware().process_response(self.req, self.resp)
         self.assertEqual(len(mail.outbox), 0)
 
-    @skipIf(six.PY3, "HTTP_REFERER is str type on Python 3")
-    def test_404_error_nonascii_referrer(self):
-        # Such referer strings should not happen, but anyway, if it happens,
-        # let's not crash
-        self.req.META['HTTP_REFERER'] = b'http://testserver/c/\xd0\xbb\xd0\xb8/'
-        BrokenLinkEmailsMiddleware().process_response(self.req, self.resp)
-        self.assertEqual(len(mail.outbox), 1)
-
-    @skipIf(six.PY3, "HTTP_USER_AGENT is str type on Python 3")
-    def test_404_error_nonascii_user_agent(self):
-        # Such user agent strings should not happen, but anyway, if it happens,
-        # let's not crash
-        self.req.META['HTTP_REFERER'] = '/another/url/'
-        self.req.META['HTTP_USER_AGENT'] = b'\xd0\xbb\xd0\xb8\xff\xff'
-        BrokenLinkEmailsMiddleware().process_response(self.req, self.resp)
-        self.assertEqual(len(mail.outbox), 1)
-        self.assertIn('User agent: \u043b\u0438\ufffd\ufffd\n', mail.outbox[0].body)
-
     def test_custom_request_checker(self):
         class SubclassedMiddleware(BrokenLinkEmailsMiddleware):
             ignored_user_agent_patterns = (re.compile(r'Spider.*'), re.compile(r'Robot.*'))
 
             def is_ignorable_request(self, request, uri, domain, referer):
                 '''Check user-agent in addition to normal checks.'''
-                if super(SubclassedMiddleware, self).is_ignorable_request(request, uri, domain, referer):
+                if super().is_ignorable_request(request, uri, domain, referer):
                     return True
                 user_agent = request.META['HTTP_USER_AGENT']
                 return any(pattern.search(user_agent) for pattern in self.ignored_user_agent_patterns)
@@ -429,41 +428,38 @@ class BrokenLinkEmailsMiddlewareTest(SimpleTestCase):
 
 @override_settings(ROOT_URLCONF='middleware.cond_get_urls')
 class ConditionalGetMiddlewareTest(SimpleTestCase):
+    request_factory = RequestFactory()
 
     def setUp(self):
-        self.req = RequestFactory().get('/')
+        self.req = self.request_factory.get('/')
         self.resp = self.client.get(self.req.path_info)
 
-    # Tests for the Date header
-
-    def test_date_header_added(self):
-        self.assertNotIn('Date', self.resp)
-        self.resp = ConditionalGetMiddleware().process_response(self.req, self.resp)
-        self.assertIn('Date', self.resp)
-
-    # Tests for the Content-Length header
-
-    def test_content_length_header_added(self):
-        content_length = len(self.resp.content)
-        self.assertNotIn('Content-Length', self.resp)
-        self.resp = ConditionalGetMiddleware().process_response(self.req, self.resp)
-        self.assertIn('Content-Length', self.resp)
-        self.assertEqual(int(self.resp['Content-Length']), content_length)
-
-    def test_content_length_header_not_added(self):
-        resp = StreamingHttpResponse('content')
-        self.assertNotIn('Content-Length', resp)
-        resp = ConditionalGetMiddleware().process_response(self.req, resp)
-        self.assertNotIn('Content-Length', resp)
-
-    def test_content_length_header_not_changed(self):
-        bad_content_length = len(self.resp.content) + 10
-        self.resp['Content-Length'] = bad_content_length
-        self.resp = ConditionalGetMiddleware().process_response(self.req, self.resp)
-        self.assertEqual(int(self.resp['Content-Length']), bad_content_length)
-
     # Tests for the ETag header
 
+    def test_middleware_calculates_etag(self):
+        self.assertNotIn('ETag', self.resp)
+        self.resp = ConditionalGetMiddleware().process_response(self.req, self.resp)
+        self.assertEqual(self.resp.status_code, 200)
+        self.assertNotEqual('', self.resp['ETag'])
+
+    def test_middleware_wont_overwrite_etag(self):
+        self.resp['ETag'] = 'eggs'
+        self.resp = ConditionalGetMiddleware().process_response(self.req, self.resp)
+        self.assertEqual(self.resp.status_code, 200)
+        self.assertEqual('eggs', self.resp['ETag'])
+
+    def test_no_etag_streaming_response(self):
+        res = StreamingHttpResponse(['content'])
+        self.assertFalse(ConditionalGetMiddleware().process_response(self.req, res).has_header('ETag'))
+
+    def test_no_etag_no_store_cache(self):
+        self.resp['Cache-Control'] = 'No-Cache, No-Store, Max-age=0'
+        self.assertFalse(ConditionalGetMiddleware().process_response(self.req, self.resp).has_header('ETag'))
+
+    def test_etag_extended_cache_control(self):
+        self.resp['Cache-Control'] = 'my-directive="my-no-store"'
+        self.assertTrue(ConditionalGetMiddleware().process_response(self.req, self.resp).has_header('ETag'))
+
     def test_if_none_match_and_no_etag(self):
         self.req.META['HTTP_IF_NONE_MATCH'] = 'spam'
         self.resp = ConditionalGetMiddleware().process_response(self.req, self.resp)
@@ -475,11 +471,6 @@ class ConditionalGetMiddlewareTest(SimpleTestCase):
         self.assertEqual(self.resp.status_code, 200)
 
     def test_if_none_match_and_same_etag(self):
-        self.req.META['HTTP_IF_NONE_MATCH'] = self.resp['ETag'] = 'spam'
-        self.resp = ConditionalGetMiddleware().process_response(self.req, self.resp)
-        self.assertEqual(self.resp.status_code, 304)
-
-    def test_if_none_match_and_same_etag_with_quotes(self):
         self.req.META['HTTP_IF_NONE_MATCH'] = self.resp['ETag'] = '"spam"'
         self.resp = ConditionalGetMiddleware().process_response(self.req, self.resp)
         self.assertEqual(self.resp.status_code, 304)
@@ -548,6 +539,53 @@ class ConditionalGetMiddlewareTest(SimpleTestCase):
         self.resp = ConditionalGetMiddleware().process_response(self.req, self.resp)
         self.assertEqual(self.resp.status_code, 400)
 
+    def test_not_modified_headers(self):
+        """
+        The 304 Not Modified response should include only the headers required
+        by section 4.1 of RFC 7232, Last-Modified, and the cookies.
+        """
+        self.req.META['HTTP_IF_NONE_MATCH'] = self.resp['ETag'] = '"spam"'
+        self.resp['Date'] = 'Sat, 12 Feb 2011 17:35:44 GMT'
+        self.resp['Last-Modified'] = 'Sat, 12 Feb 2011 17:35:44 GMT'
+        self.resp['Expires'] = 'Sun, 13 Feb 2011 17:35:44 GMT'
+        self.resp['Vary'] = 'Cookie'
+        self.resp['Cache-Control'] = 'public'
+        self.resp['Content-Location'] = '/alt'
+        self.resp['Content-Language'] = 'en'  # shouldn't be preserved
+        self.resp.set_cookie('key', 'value')
+
+        new_response = ConditionalGetMiddleware().process_response(self.req, self.resp)
+        self.assertEqual(new_response.status_code, 304)
+        for header in ('Cache-Control', 'Content-Location', 'Date', 'ETag', 'Expires', 'Last-Modified', 'Vary'):
+            self.assertEqual(new_response[header], self.resp[header])
+        self.assertEqual(new_response.cookies, self.resp.cookies)
+        self.assertNotIn('Content-Language', new_response)
+
+    def test_no_unsafe(self):
+        """
+        ConditionalGetMiddleware shouldn't return a conditional response on an
+        unsafe request. A response has already been generated by the time
+        ConditionalGetMiddleware is called, so it's too late to return a 412
+        Precondition Failed.
+        """
+        get_response = ConditionalGetMiddleware().process_response(self.req, self.resp)
+        etag = get_response['ETag']
+        put_request = self.request_factory.put('/', HTTP_IF_MATCH=etag)
+        put_response = HttpResponse(status=200)
+        conditional_get_response = ConditionalGetMiddleware().process_response(put_request, put_response)
+        self.assertEqual(conditional_get_response.status_code, 200)  # should never be a 412
+
+    def test_no_head(self):
+        """
+        ConditionalGetMiddleware shouldn't compute and return an ETag on a
+        HEAD request since it can't do so accurately without access to the
+        response body of the corresponding GET.
+        """
+        request = self.request_factory.head('/')
+        response = HttpResponse(status=200)
+        conditional_get_response = ConditionalGetMiddleware().process_response(request, response)
+        self.assertNotIn('ETag', conditional_get_response)
+
 
 class XFrameOptionsMiddlewareTest(SimpleTestCase):
     """
@@ -660,12 +698,13 @@ class GZipMiddlewareTest(SimpleTestCase):
     """
     short_string = b"This string is too short to be worth compressing."
     compressible_string = b'a' * 500
-    incompressible_string = b''.join(six.int2byte(random.randint(0, 255)) for _ in range(500))
+    incompressible_string = b''.join(int2byte(random.randint(0, 255)) for _ in range(500))
     sequence = [b'a' * 500, b'b' * 200, b'a' * 300]
     sequence_unicode = ['a' * 500, 'é' * 200, 'a' * 300]
+    request_factory = RequestFactory()
 
     def setUp(self):
-        self.req = RequestFactory().get('/')
+        self.req = self.request_factory.get('/')
         self.req.META['HTTP_ACCEPT_ENCODING'] = 'gzip, deflate'
         self.req.META['HTTP_USER_AGENT'] = 'Mozilla/5.0 (Windows NT 5.1; rv:9.0.1) Gecko/20100101 Firefox/9.0.1'
         self.resp = HttpResponse()
@@ -682,6 +721,12 @@ class GZipMiddlewareTest(SimpleTestCase):
         with gzip.GzipFile(mode='rb', fileobj=BytesIO(gzipped_string)) as f:
             return f.read()
 
+    @staticmethod
+    def get_mtime(gzipped_string):
+        with gzip.GzipFile(mode='rb', fileobj=BytesIO(gzipped_string)) as f:
+            f.read()  # must read the data before accessing the header
+            return f.mtime
+
     def test_compress_response(self):
         """
         Compression is performed on responses with compressible content.
@@ -707,7 +752,7 @@ class GZipMiddlewareTest(SimpleTestCase):
         r = GZipMiddleware().process_response(self.req, self.stream_resp_unicode)
         self.assertEqual(
             self.decompress(b''.join(r)),
-            b''.join(x.encode('utf-8') for x in self.sequence_unicode)
+            b''.join(x.encode() for x in self.sequence_unicode)
         )
         self.assertEqual(r.get('Content-Encoding'), 'gzip')
         self.assertFalse(r.has_header('Content-Length'))
@@ -762,31 +807,61 @@ class GZipMiddlewareTest(SimpleTestCase):
         self.assertEqual(r.content, self.incompressible_string)
         self.assertIsNone(r.get('Content-Encoding'))
 
+    def test_compress_deterministic(self):
+        """
+        Compression results are the same for the same content and don't
+        include a modification time (since that would make the results
+        of compression non-deterministic and prevent
+        ConditionalGetMiddleware from recognizing conditional matches
+        on gzipped content).
+        """
+        r1 = GZipMiddleware().process_response(self.req, self.resp)
+        r2 = GZipMiddleware().process_response(self.req, self.resp)
+        self.assertEqual(r1.content, r2.content)
+        self.assertEqual(self.get_mtime(r1.content), 0)
+        self.assertEqual(self.get_mtime(r2.content), 0)
+
 
-@override_settings(USE_ETAGS=True)
 class ETagGZipMiddlewareTest(SimpleTestCase):
     """
-    Tests if the ETagMiddleware behaves correctly with GZipMiddleware.
+    ETags are handled properly by GZipMiddleware.
     """
     rf = RequestFactory()
     compressible_string = b'a' * 500
 
-    def test_compress_response(self):
+    def test_strong_etag_modified(self):
         """
-        ETag is changed after gzip compression is performed.
+        GZipMiddleware makes a strong ETag weak.
+        """
+        request = self.rf.get('/', HTTP_ACCEPT_ENCODING='gzip, deflate')
+        response = HttpResponse(self.compressible_string)
+        response['ETag'] = '"eggs"'
+        gzip_response = GZipMiddleware().process_response(request, response)
+        self.assertEqual(gzip_response['ETag'], 'W/"eggs"')
+
+    def test_weak_etag_not_modified(self):
+        """
+        GZipMiddleware doesn't modify a weak ETag.
+        """
+        request = self.rf.get('/', HTTP_ACCEPT_ENCODING='gzip, deflate')
+        response = HttpResponse(self.compressible_string)
+        response['ETag'] = 'W/"eggs"'
+        gzip_response = GZipMiddleware().process_response(request, response)
+        self.assertEqual(gzip_response['ETag'], 'W/"eggs"')
+
+    def test_etag_match(self):
+        """
+        GZipMiddleware allows 304 Not Modified responses.
         """
         request = self.rf.get('/', HTTP_ACCEPT_ENCODING='gzip, deflate')
         response = GZipMiddleware().process_response(
             request,
-            CommonMiddleware().process_response(request, HttpResponse(self.compressible_string))
+            ConditionalGetMiddleware().process_response(request, HttpResponse(self.compressible_string))
         )
-        gzip_etag = response.get('ETag')
-
-        request = self.rf.get('/', HTTP_ACCEPT_ENCODING='')
-        response = GZipMiddleware().process_response(
-            request,
-            CommonMiddleware().process_response(request, HttpResponse(self.compressible_string))
+        gzip_etag = response['ETag']
+        next_request = self.rf.get('/', HTTP_ACCEPT_ENCODING='gzip, deflate', HTTP_IF_NONE_MATCH=gzip_etag)
+        next_response = ConditionalGetMiddleware().process_response(
+            next_request,
+            HttpResponse(self.compressible_string)
         )
-        nogzip_etag = response.get('ETag')
-
-        self.assertNotEqual(gzip_etag, nogzip_etag)
+        self.assertEqual(next_response.status_code, 304)
