@@ -1,6 +1,5 @@
-from __future__ import unicode_literals
-
-from unittest import skipIf
+import compileall
+import os
 
 from django.db import connection, connections
 from django.db.migrations.exceptions import (
@@ -9,13 +8,15 @@ from django.db.migrations.exceptions import (
 from django.db.migrations.loader import MigrationLoader
 from django.db.migrations.recorder import MigrationRecorder
 from django.test import TestCase, modify_settings, override_settings
-from django.utils import six
+
+from .test_base import MigrationTestBase
 
 
 class RecorderTests(TestCase):
     """
     Tests recording migrations as applied or not.
     """
+    databases = {'default', 'other'}
 
     def test_apply(self):
         """
@@ -23,23 +24,23 @@ class RecorderTests(TestCase):
         """
         recorder = MigrationRecorder(connection)
         self.assertEqual(
-            set((x, y) for (x, y) in recorder.applied_migrations() if x == "myapp"),
+            {(x, y) for (x, y) in recorder.applied_migrations() if x == "myapp"},
             set(),
         )
         recorder.record_applied("myapp", "0432_ponies")
         self.assertEqual(
-            set((x, y) for (x, y) in recorder.applied_migrations() if x == "myapp"),
+            {(x, y) for (x, y) in recorder.applied_migrations() if x == "myapp"},
             {("myapp", "0432_ponies")},
         )
         # That should not affect records of another database
         recorder_other = MigrationRecorder(connections['other'])
         self.assertEqual(
-            set((x, y) for (x, y) in recorder_other.applied_migrations() if x == "myapp"),
+            {(x, y) for (x, y) in recorder_other.applied_migrations() if x == "myapp"},
             set(),
         )
         recorder.record_unapplied("myapp", "0432_ponies")
         self.assertEqual(
-            set((x, y) for (x, y) in recorder.applied_migrations() if x == "myapp"),
+            {(x, y) for (x, y) in recorder.applied_migrations() if x == "myapp"},
             set(),
         )
 
@@ -85,6 +86,22 @@ class LoaderTests(TestCase):
         # Ensure we've included unmigrated apps in there too
         self.assertIn("basic", project_state.real_apps)
 
+    @override_settings(MIGRATION_MODULES={
+        'migrations': 'migrations.test_migrations',
+        'migrations2': 'migrations2.test_migrations_2',
+    })
+    @modify_settings(INSTALLED_APPS={'append': 'migrations2'})
+    def test_plan_handles_repeated_migrations(self):
+        """
+        _generate_plan() doesn't readd migrations already in the plan (#29180).
+        """
+        migration_loader = MigrationLoader(connection)
+        nodes = [('migrations', '0002_second'), ('migrations2', '0001_initial')]
+        self.assertEqual(
+            migration_loader.graph._generate_plan(nodes, at_end=True),
+            [('migrations', '0001_initial'), ('migrations', '0002_second'), ('migrations2', '0001_initial')]
+        )
+
     @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations_unmigdep"})
     def test_load_unmigrated_dependency(self):
         """
@@ -172,7 +189,6 @@ class LoaderTests(TestCase):
                 "App with migrations module file not in unmigrated apps."
             )
 
-    @skipIf(six.PY2, "PY2 doesn't load empty dirs.")
     def test_load_empty_dir(self):
         with override_settings(MIGRATION_MODULES={"migrations": "migrations.faulty_migrations.namespace"}):
             loader = MigrationLoader(connection)
@@ -204,6 +220,22 @@ class LoaderTests(TestCase):
         self.assertEqual(migration_loader.migrated_apps, set())
         self.assertEqual(migration_loader.unmigrated_apps, {'migrated_app'})
 
+    @override_settings(
+        INSTALLED_APPS=['migrations.migrations_test_apps.migrated_app'],
+        MIGRATION_MODULES={'migrated_app': 'missing-module'},
+    )
+    def test_explicit_missing_module(self):
+        """
+        If a MIGRATION_MODULES override points to a missing module, the error
+        raised during the importation attempt should be propagated unless
+        `ignore_no_migrations=True`.
+        """
+        with self.assertRaisesMessage(ImportError, 'missing-module'):
+            migration_loader = MigrationLoader(connection)
+        migration_loader = MigrationLoader(connection, ignore_no_migrations=True)
+        self.assertEqual(migration_loader.migrated_apps, set())
+        self.assertEqual(migration_loader.unmigrated_apps, {'migrated_app'})
+
     @override_settings(MIGRATION_MODULES={"migrations": "migrations.test_migrations_squashed"})
     def test_loading_squashed(self):
         "Tests loading a squashed migration"
@@ -341,7 +373,7 @@ class LoaderTests(TestCase):
         loader.build_graph()
         self.assertEqual(num_nodes(), 3)
 
-        # However, starting at 3 or 4 we'd need to use non-existing migrations
+        # However, starting at 3 or 4, nonexistent migrations would be needed.
         msg = ("Migration migrations.6_auto depends on nonexistent node ('migrations', '5_auto'). "
                "Django tried to replace migration migrations.5_auto with any of "
                "[migrations.3_squashed_5] but wasn't able to because some of the replaced "
@@ -411,7 +443,7 @@ class LoaderTests(TestCase):
     ]})
     def test_loading_squashed_ref_squashed(self):
         "Tests loading a squashed migration with a new migration referencing it"
-        """
+        r"""
         The sample migrations are structured like this:
 
         app_1       1 --> 2 ---------------------*--> 3        *--> 4
@@ -467,3 +499,45 @@ class LoaderTests(TestCase):
             ('app1', '4_auto'),
         }
         self.assertEqual(plan, expected_plan)
+
+    @override_settings(MIGRATION_MODULES={'migrations': 'migrations.test_migrations_private'})
+    def test_ignore_files(self):
+        """Files prefixed with underscore, tilde, or dot aren't loaded."""
+        loader = MigrationLoader(connection)
+        loader.load_disk()
+        migrations = [name for app, name in loader.disk_migrations if app == 'migrations']
+        self.assertEqual(migrations, ['0001_initial'])
+
+
+class PycLoaderTests(MigrationTestBase):
+
+    def test_valid(self):
+        """
+        To support frozen environments, MigrationLoader loads .pyc migrations.
+        """
+        with self.temporary_migration_module(module='migrations.test_migrations') as migration_dir:
+            # Compile .py files to .pyc files and delete .py files.
+            compileall.compile_dir(migration_dir, force=True, quiet=1, legacy=True)
+            for name in os.listdir(migration_dir):
+                if name.endswith('.py'):
+                    os.remove(os.path.join(migration_dir, name))
+            loader = MigrationLoader(connection)
+            self.assertIn(('migrations', '0001_initial'), loader.disk_migrations)
+
+    def test_invalid(self):
+        """
+        MigrationLoader reraises ImportErrors caused by "bad magic number" pyc
+        files with a more helpful message.
+        """
+        with self.temporary_migration_module(module='migrations.test_migrations_bad_pyc') as migration_dir:
+            # The -tpl suffix is to avoid the pyc exclusion in MANIFEST.in.
+            os.rename(
+                os.path.join(migration_dir, '0001_initial.pyc-tpl'),
+                os.path.join(migration_dir, '0001_initial.pyc'),
+            )
+            msg = (
+                r"Couldn't import '\w+.migrations.0001_initial' as it appears "
+                "to be a stale .pyc file."
+            )
+            with self.assertRaisesRegex(ImportError, msg):
+                MigrationLoader(connection)
