@@ -1,6 +1,3 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
 from django.db import migrations, models
 
 
@@ -21,7 +18,7 @@ class Migration(migrations.Migration):
             "Book",
             [
                 ("id", models.AutoField(primary_key=True)),
-                ("author", models.ForeignKey("migrations.Author", null=True)),
+                ("author", models.ForeignKey("migrations.Author", models.SET_NULL, null=True)),
             ],
         )
 
