@@ -12,7 +12,7 @@ from .models import DurationModel, NullDurationModel
 class TestSaveLoad(TestCase):
 
     def test_simple_roundtrip(self):
-        duration = datetime.timedelta(days=123, seconds=123, microseconds=123)
+        duration = datetime.timedelta(microseconds=8999999999999999)
         DurationModel.objects.create(field=duration)
         loaded = DurationModel.objects.get()
         self.assertEqual(loaded.field, duration)
