@@ -21,14 +21,13 @@ class TestFloatField(TestCase):
         instance.size = instance
         msg = (
             'Tried to update field model_fields.FloatModel.size with a model '
-            'instance, <FloatModel: FloatModel object>. Use a value '
-            'compatible with FloatField.'
-        )
+            'instance, %r. Use a value compatible with FloatField.'
+        ) % instance
         with transaction.atomic():
             with self.assertRaisesMessage(TypeError, msg):
                 instance.save()
         # Try setting field to object on retrieved object
         obj = FloatModel.objects.get(pk=instance.id)
         obj.size = obj
-        with self.assertRaises(TypeError):
+        with self.assertRaisesMessage(TypeError, msg):
             obj.save()
