@@ -5,7 +5,6 @@ from django.core import checks
 from django.db import models
 from django.test import TestCase, skipIfDBFeature
 from django.test.utils import isolate_apps
-from django.utils import six
 
 from .models import Bar, FkToChar, Foo, PrimaryKeyCharModel
 
@@ -50,7 +49,7 @@ class ForeignKeyTests(TestCase):
 
     def test_related_name_converted_to_text(self):
         rel_name = Bar._meta.get_field('a').remote_field.related_name
-        self.assertIsInstance(rel_name, six.text_type)
+        self.assertIsInstance(rel_name, str)
 
     def test_abstract_model_pending_operations(self):
         """
@@ -94,3 +93,38 @@ class ForeignKeyTests(TestCase):
 
         assert_app_model_resolved('model_fields')
         assert_app_model_resolved('tests')
+
+    @isolate_apps('model_fields')
+    def test_to_python(self):
+        class Foo(models.Model):
+            pass
+
+        class Bar(models.Model):
+            fk = models.ForeignKey(Foo, models.CASCADE)
+
+        self.assertEqual(Bar._meta.get_field('fk').to_python('1'), 1)
+
+    @isolate_apps('model_fields')
+    def test_fk_to_fk_get_col_output_field(self):
+        class Foo(models.Model):
+            pass
+
+        class Bar(models.Model):
+            foo = models.ForeignKey(Foo, models.CASCADE, primary_key=True)
+
+        class Baz(models.Model):
+            bar = models.ForeignKey(Bar, models.CASCADE, primary_key=True)
+
+        col = Baz._meta.get_field('bar').get_col('alias')
+        self.assertIs(col.output_field, Foo._meta.pk)
+
+    @isolate_apps('model_fields')
+    def test_recursive_fks_get_col(self):
+        class Foo(models.Model):
+            bar = models.ForeignKey('Bar', models.CASCADE, primary_key=True)
+
+        class Bar(models.Model):
+            foo = models.ForeignKey(Foo, models.CASCADE, primary_key=True)
+
+        with self.assertRaisesMessage(ValueError, 'Cannot resolve output_field'):
+            Foo._meta.get_field('bar').get_col('alias')
