@@ -1,14 +1,11 @@
 """
 Regression tests for proper working of ForeignKey(null=True). Tests these bugs:
 
-    * #7512: including a nullable foreign key reference in Meta ordering has un
-xpected results
+    * #7512: including a nullable foreign key reference in Meta ordering has
+unexpected results
 
 """
-from __future__ import unicode_literals
-
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 
 # The first two models represent a very simple null FK ordering case.
@@ -16,16 +13,15 @@ class Author(models.Model):
     name = models.CharField(max_length=150)
 
 
-@python_2_unicode_compatible
 class Article(models.Model):
     title = models.CharField(max_length=150)
-    author = models.ForeignKey(Author, null=True)
-
-    def __str__(self):
-        return 'Article titled: %s' % (self.title, )
+    author = models.ForeignKey(Author, models.SET_NULL, null=True)
 
     class Meta:
-        ordering = ['author__name', ]
+        ordering = ['author__name']
+
+    def __str__(self):
+        return 'Article titled: %s' % self.title
 
 
 # These following 4 models represent a far more complex ordering case.
@@ -34,22 +30,20 @@ class SystemInfo(models.Model):
 
 
 class Forum(models.Model):
-    system_info = models.ForeignKey(SystemInfo)
+    system_info = models.ForeignKey(SystemInfo, models.CASCADE)
     forum_name = models.CharField(max_length=32)
 
 
-@python_2_unicode_compatible
 class Post(models.Model):
-    forum = models.ForeignKey(Forum, null=True)
+    forum = models.ForeignKey(Forum, models.SET_NULL, null=True)
     title = models.CharField(max_length=32)
 
     def __str__(self):
         return self.title
 
 
-@python_2_unicode_compatible
 class Comment(models.Model):
-    post = models.ForeignKey(Post, null=True)
+    post = models.ForeignKey(Post, models.SET_NULL, null=True)
     comment_text = models.CharField(max_length=250)
 
     class Meta:
