@@ -4,7 +4,7 @@ import unittest
 import uuid
 
 from django import forms
-from django.core import exceptions, serializers, validators
+from django.core import checks, exceptions, serializers, validators
 from django.core.exceptions import FieldError
 from django.core.management import call_command
 from django.db import IntegrityError, connection, models
@@ -12,7 +12,9 @@ from django.test import TransactionTestCase, modify_settings, override_settings
 from django.test.utils import isolate_apps
 from django.utils import timezone
 
-from . import PostgreSQLTestCase, PostgreSQLWidgetTestCase
+from . import (
+    PostgreSQLSimpleTestCase, PostgreSQLTestCase, PostgreSQLWidgetTestCase,
+)
 from .models import (
     ArrayFieldSubclass, CharArrayModel, DateTimeArrayModel, IntegerArrayModel,
     NestedIntegerArrayModel, NullableIntegerArrayModel, OtherTypesArrayModel,
@@ -24,6 +26,7 @@ try:
     from django.contrib.postgres.forms import (
         SimpleArrayField, SplitArrayField, SplitArrayWidget,
     )
+    from psycopg2.extras import NumericRange
 except ImportError:
     pass
 
@@ -96,6 +99,12 @@ class TestSaveLoad(PostgreSQLTestCase):
             uuids=[uuid.uuid4()],
             decimals=[decimal.Decimal(1.25), 1.75],
             tags=[Tag(1), Tag(2), Tag(3)],
+            json=[{'a': 1}, {'b': 2}],
+            int_ranges=[NumericRange(10, 20), NumericRange(30, 40)],
+            bigint_ranges=[
+                NumericRange(7000000000, 10000000000),
+                NumericRange(50000000000, 70000000000),
+            ]
         )
         instance.save()
         loaded = OtherTypesArrayModel.objects.get()
@@ -103,6 +112,9 @@ class TestSaveLoad(PostgreSQLTestCase):
         self.assertEqual(instance.uuids, loaded.uuids)
         self.assertEqual(instance.decimals, loaded.decimals)
         self.assertEqual(instance.tags, loaded.tags)
+        self.assertEqual(instance.json, loaded.json)
+        self.assertEqual(instance.int_ranges, loaded.int_ranges)
+        self.assertEqual(instance.bigint_ranges, loaded.bigint_ranges)
 
     def test_null_from_db_value_handling(self):
         instance = OtherTypesArrayModel.objects.create(
@@ -113,6 +125,9 @@ class TestSaveLoad(PostgreSQLTestCase):
         )
         instance.refresh_from_db()
         self.assertIsNone(instance.tags)
+        self.assertEqual(instance.json, [])
+        self.assertIsNone(instance.int_ranges)
+        self.assertIsNone(instance.bigint_ranges)
 
     def test_model_set_on_base_field(self):
         instance = IntegerArrayModel()
@@ -123,14 +138,15 @@ class TestSaveLoad(PostgreSQLTestCase):
 
 class TestQuerying(PostgreSQLTestCase):
 
-    def setUp(self):
-        self.objs = [
-            NullableIntegerArrayModel.objects.create(field=[1]),
-            NullableIntegerArrayModel.objects.create(field=[2]),
-            NullableIntegerArrayModel.objects.create(field=[2, 3]),
-            NullableIntegerArrayModel.objects.create(field=[20, 30, 40]),
-            NullableIntegerArrayModel.objects.create(field=None),
-        ]
+    @classmethod
+    def setUpTestData(cls):
+        cls.objs = NullableIntegerArrayModel.objects.bulk_create([
+            NullableIntegerArrayModel(field=[1]),
+            NullableIntegerArrayModel(field=[2]),
+            NullableIntegerArrayModel(field=[2, 3]),
+            NullableIntegerArrayModel(field=[20, 30, 40]),
+            NullableIntegerArrayModel(field=None),
+        ])
 
     def test_exact(self):
         self.assertSequenceEqual(
@@ -176,6 +192,15 @@ class TestQuerying(PostgreSQLTestCase):
             self.objs[:2]
         )
 
+    def test_in_subquery(self):
+        IntegerArrayModel.objects.create(field=[2, 3])
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.filter(
+                field__in=IntegerArrayModel.objects.all().values_list('field', flat=True)
+            ),
+            self.objs[2:3]
+        )
+
     @unittest.expectedFailure
     def test_in_including_F_object(self):
         # This test asserts that Array objects passed to filters can be
@@ -300,6 +325,22 @@ class TestQuerying(PostgreSQLTestCase):
             self.objs[2:3]
         )
 
+    def test_order_by_slice(self):
+        more_objs = (
+            NullableIntegerArrayModel.objects.create(field=[1, 637]),
+            NullableIntegerArrayModel.objects.create(field=[2, 1]),
+            NullableIntegerArrayModel.objects.create(field=[3, -98123]),
+            NullableIntegerArrayModel.objects.create(field=[4, 2]),
+        )
+        self.assertSequenceEqual(
+            NullableIntegerArrayModel.objects.order_by('field__1'),
+            [
+                more_objs[2], more_objs[1], more_objs[3], self.objs[2],
+                self.objs[3], more_objs[0], self.objs[4], self.objs[1],
+                self.objs[0],
+            ]
+        )
+
     @unittest.expectedFailure
     def test_slice_nested(self):
         instance = NestedIntegerArrayModel.objects.create(field=[[1, 2], [3, 4]])
@@ -328,17 +369,14 @@ class TestQuerying(PostgreSQLTestCase):
 
 class TestDateTimeExactQuerying(PostgreSQLTestCase):
 
-    def setUp(self):
+    @classmethod
+    def setUpTestData(cls):
         now = timezone.now()
-        self.datetimes = [now]
-        self.dates = [now.date()]
-        self.times = [now.time()]
-        self.objs = [
-            DateTimeArrayModel.objects.create(
-                datetimes=self.datetimes,
-                dates=self.dates,
-                times=self.times,
-            )
+        cls.datetimes = [now]
+        cls.dates = [now.date()]
+        cls.times = [now.time()]
+        cls.objs = [
+            DateTimeArrayModel.objects.create(datetimes=cls.datetimes, dates=cls.dates, times=cls.times),
         ]
 
     def test_exact_datetimes(self):
@@ -362,17 +400,18 @@ class TestDateTimeExactQuerying(PostgreSQLTestCase):
 
 class TestOtherTypesExactQuerying(PostgreSQLTestCase):
 
-    def setUp(self):
-        self.ips = ['192.168.0.1', '::1']
-        self.uuids = [uuid.uuid4()]
-        self.decimals = [decimal.Decimal(1.25), 1.75]
-        self.tags = [Tag(1), Tag(2), Tag(3)]
-        self.objs = [
+    @classmethod
+    def setUpTestData(cls):
+        cls.ips = ['192.168.0.1', '::1']
+        cls.uuids = [uuid.uuid4()]
+        cls.decimals = [decimal.Decimal(1.25), 1.75]
+        cls.tags = [Tag(1), Tag(2), Tag(3)]
+        cls.objs = [
             OtherTypesArrayModel.objects.create(
-                ips=self.ips,
-                uuids=self.uuids,
-                decimals=self.decimals,
-                tags=self.tags,
+                ips=cls.ips,
+                uuids=cls.uuids,
+                decimals=cls.decimals,
+                tags=cls.tags,
             )
         ]
 
@@ -402,7 +441,7 @@ class TestOtherTypesExactQuerying(PostgreSQLTestCase):
 
 
 @isolate_apps('postgres_tests')
-class TestChecks(PostgreSQLTestCase):
+class TestChecks(PostgreSQLSimpleTestCase):
 
     def test_field_checks(self):
         class MyModel(PostgreSQLModel):
@@ -424,6 +463,38 @@ class TestChecks(PostgreSQLTestCase):
         self.assertEqual(len(errors), 1)
         self.assertEqual(errors[0].id, 'postgres.E002')
 
+    def test_invalid_default(self):
+        class MyModel(PostgreSQLModel):
+            field = ArrayField(models.IntegerField(), default=[])
+
+        model = MyModel()
+        self.assertEqual(model.check(), [
+            checks.Warning(
+                msg=(
+                    "ArrayField default should be a callable instead of an "
+                    "instance so that it's not shared between all field "
+                    "instances."
+                ),
+                hint='Use a callable instead, e.g., use `list` instead of `[]`.',
+                obj=MyModel._meta.get_field('field'),
+                id='postgres.E003',
+            )
+        ])
+
+    def test_valid_default(self):
+        class MyModel(PostgreSQLModel):
+            field = ArrayField(models.IntegerField(), default=list)
+
+        model = MyModel()
+        self.assertEqual(model.check(), [])
+
+    def test_valid_default_none(self):
+        class MyModel(PostgreSQLModel):
+            field = ArrayField(models.IntegerField(), default=None)
+
+        model = MyModel()
+        self.assertEqual(model.check(), [])
+
     def test_nested_field_checks(self):
         """
         Nested ArrayFields are permitted.
@@ -519,7 +590,7 @@ class TestMigrations(TransactionTestCase):
             self.assertNotIn(table_name, connection.introspection.table_names(cursor))
 
 
-class TestSerialization(PostgreSQLTestCase):
+class TestSerialization(PostgreSQLSimpleTestCase):
     test_data = (
         '[{"fields": {"field": "[\\"1\\", \\"2\\", null]"}, "model": "postgres_tests.integerarraymodel", "pk": null}]'
     )
@@ -534,7 +605,7 @@ class TestSerialization(PostgreSQLTestCase):
         self.assertEqual(instance.field, [1, 2, None])
 
 
-class TestValidation(PostgreSQLTestCase):
+class TestValidation(PostgreSQLSimpleTestCase):
 
     def test_unbounded(self):
         field = ArrayField(models.IntegerField())
@@ -543,7 +614,7 @@ class TestValidation(PostgreSQLTestCase):
         self.assertEqual(cm.exception.code, 'item_invalid')
         self.assertEqual(
             cm.exception.message % cm.exception.params,
-            'Item 1 in the array did not validate: This field cannot be null.'
+            'Item 2 in the array did not validate: This field cannot be null.'
         )
 
     def test_blank_true(self):
@@ -574,10 +645,10 @@ class TestValidation(PostgreSQLTestCase):
         exception = cm.exception.error_list[0]
         self.assertEqual(
             exception.message,
-            'Item 0 in the array did not validate: Ensure this value has at most 2 characters (it has 3).'
+            'Item 1 in the array did not validate: Ensure this value has at most 2 characters (it has 3).'
         )
         self.assertEqual(exception.code, 'item_invalid')
-        self.assertEqual(exception.params, {'nth': 0, 'value': 'abc', 'limit_value': 2, 'show_value': 3})
+        self.assertEqual(exception.params, {'nth': 1, 'value': 'abc', 'limit_value': 2, 'show_value': 3})
 
     def test_with_validators(self):
         field = ArrayField(models.IntegerField(validators=[validators.MinValueValidator(1)]))
@@ -588,13 +659,13 @@ class TestValidation(PostgreSQLTestCase):
         exception = cm.exception.error_list[0]
         self.assertEqual(
             exception.message,
-            'Item 0 in the array did not validate: Ensure this value is greater than or equal to 1.'
+            'Item 1 in the array did not validate: Ensure this value is greater than or equal to 1.'
         )
         self.assertEqual(exception.code, 'item_invalid')
-        self.assertEqual(exception.params, {'nth': 0, 'value': 0, 'limit_value': 1, 'show_value': 0})
+        self.assertEqual(exception.params, {'nth': 1, 'value': 0, 'limit_value': 1, 'show_value': 0})
 
 
-class TestSimpleFormField(PostgreSQLTestCase):
+class TestSimpleFormField(PostgreSQLSimpleTestCase):
 
     def test_valid(self):
         field = SimpleArrayField(forms.CharField())
@@ -605,13 +676,13 @@ class TestSimpleFormField(PostgreSQLTestCase):
         field = SimpleArrayField(forms.IntegerField())
         with self.assertRaises(exceptions.ValidationError) as cm:
             field.clean('a,b,9')
-        self.assertEqual(cm.exception.messages[0], 'Item 0 in the array did not validate: Enter a whole number.')
+        self.assertEqual(cm.exception.messages[0], 'Item 1 in the array did not validate: Enter a whole number.')
 
     def test_validate_fail(self):
         field = SimpleArrayField(forms.CharField(required=True))
         with self.assertRaises(exceptions.ValidationError) as cm:
             field.clean('a,b,')
-        self.assertEqual(cm.exception.messages[0], 'Item 2 in the array did not validate: This field is required.')
+        self.assertEqual(cm.exception.messages[0], 'Item 3 in the array did not validate: This field is required.')
 
     def test_validate_fail_base_field_error_params(self):
         field = SimpleArrayField(forms.CharField(max_length=2))
@@ -622,23 +693,23 @@ class TestSimpleFormField(PostgreSQLTestCase):
         first_error = errors[0]
         self.assertEqual(
             first_error.message,
-            'Item 0 in the array did not validate: Ensure this value has at most 2 characters (it has 3).'
+            'Item 1 in the array did not validate: Ensure this value has at most 2 characters (it has 3).'
         )
         self.assertEqual(first_error.code, 'item_invalid')
-        self.assertEqual(first_error.params, {'nth': 0, 'value': 'abc', 'limit_value': 2, 'show_value': 3})
+        self.assertEqual(first_error.params, {'nth': 1, 'value': 'abc', 'limit_value': 2, 'show_value': 3})
         second_error = errors[1]
         self.assertEqual(
             second_error.message,
-            'Item 2 in the array did not validate: Ensure this value has at most 2 characters (it has 4).'
+            'Item 3 in the array did not validate: Ensure this value has at most 2 characters (it has 4).'
         )
         self.assertEqual(second_error.code, 'item_invalid')
-        self.assertEqual(second_error.params, {'nth': 2, 'value': 'defg', 'limit_value': 2, 'show_value': 4})
+        self.assertEqual(second_error.params, {'nth': 3, 'value': 'defg', 'limit_value': 2, 'show_value': 4})
 
     def test_validators_fail(self):
         field = SimpleArrayField(forms.RegexField('[a-e]{2}'))
         with self.assertRaises(exceptions.ValidationError) as cm:
             field.clean('a,bc,de')
-        self.assertEqual(cm.exception.messages[0], 'Item 0 in the array did not validate: Enter a valid value.')
+        self.assertEqual(cm.exception.messages[0], 'Item 1 in the array did not validate: Enter a valid value.')
 
     def test_delimiter(self):
         field = SimpleArrayField(forms.CharField(), delimiter='|')
@@ -686,13 +757,33 @@ class TestSimpleFormField(PostgreSQLTestCase):
         self.assertIsInstance(form_field, SimpleArrayField)
         self.assertEqual(form_field.max_length, 4)
 
+    def test_model_field_choices(self):
+        model_field = ArrayField(models.IntegerField(choices=((1, 'A'), (2, 'B'))))
+        form_field = model_field.formfield()
+        self.assertEqual(form_field.clean('1,2'), [1, 2])
+
     def test_already_converted_value(self):
         field = SimpleArrayField(forms.CharField())
         vals = ['a', 'b', 'c']
         self.assertEqual(field.clean(vals), vals)
 
+    def test_has_changed(self):
+        field = SimpleArrayField(forms.IntegerField())
+        self.assertIs(field.has_changed([1, 2], [1, 2]), False)
+        self.assertIs(field.has_changed([1, 2], '1,2'), False)
+        self.assertIs(field.has_changed([1, 2], '1,2,3'), True)
+        self.assertIs(field.has_changed([1, 2], 'a,b'), True)
 
-class TestSplitFormField(PostgreSQLTestCase):
+    def test_has_changed_empty(self):
+        field = SimpleArrayField(forms.CharField())
+        self.assertIs(field.has_changed(None, None), False)
+        self.assertIs(field.has_changed(None, ''), False)
+        self.assertIs(field.has_changed(None, []), False)
+        self.assertIs(field.has_changed([], None), False)
+        self.assertIs(field.has_changed([], ''), False)
+
+
+class TestSplitFormField(PostgreSQLSimpleTestCase):
 
     def test_valid(self):
         class SplitForm(forms.Form):
@@ -742,10 +833,10 @@ class TestSplitFormField(PostgreSQLTestCase):
         data = {'array_0': 'a', 'array_1': 'b', 'array_2': ''}
         form = SplitForm(data)
         self.assertFalse(form.is_valid())
-        self.assertEqual(form.errors, {'array': ['Item 2 in the array did not validate: This field is required.']})
+        self.assertEqual(form.errors, {'array': ['Item 3 in the array did not validate: This field is required.']})
 
     def test_invalid_integer(self):
-        msg = 'Item 1 in the array did not validate: Ensure this value is less than or equal to 100.'
+        msg = 'Item 2 in the array did not validate: Ensure this value is less than or equal to 100.'
         with self.assertRaisesMessage(exceptions.ValidationError, msg):
             SplitArrayField(forms.IntegerField(max_value=100), size=2).clean([0, 101])
 
@@ -759,9 +850,9 @@ class TestSplitFormField(PostgreSQLTestCase):
             <tr>
                 <th><label for="id_array_0">Array:</label></th>
                 <td>
-                    <input id="id_array_0" name="array_0" type="text" required />
-                    <input id="id_array_1" name="array_1" type="text" required />
-                    <input id="id_array_2" name="array_2" type="text" required />
+                    <input id="id_array_0" name="array_0" type="text" required>
+                    <input id="id_array_1" name="array_1" type="text" required>
+                    <input id="id_array_2" name="array_2" type="text" required>
                 </td>
             </tr>
         ''')
@@ -771,8 +862,8 @@ class TestSplitFormField(PostgreSQLTestCase):
         with self.assertRaises(exceptions.ValidationError) as cm:
             field.clean(['abc', 'c', 'defg'])
         self.assertEqual(cm.exception.messages, [
-            'Item 0 in the array did not validate: Ensure this value has at most 2 characters (it has 3).',
-            'Item 2 in the array did not validate: Ensure this value has at most 2 characters (it has 4).',
+            'Item 1 in the array did not validate: Ensure this value has at most 2 characters (it has 3).',
+            'Item 3 in the array did not validate: Ensure this value has at most 2 characters (it has 4).',
         ])
 
     def test_splitarraywidget_value_omitted_from_data(self):
@@ -841,8 +932,8 @@ class TestSplitFormWidget(PostgreSQLWidgetTestCase):
         self.check_html(
             SplitArrayWidget(forms.TextInput(), size=2), 'array', None,
             """
-            <input name="array_0" type="text" />
-            <input name="array_1" type="text" />
+            <input name="array_0" type="text">
+            <input name="array_1" type="text">
             """
         )
 
@@ -852,8 +943,8 @@ class TestSplitFormWidget(PostgreSQLWidgetTestCase):
             'array', ['val1', 'val2'], attrs={'id': 'foo'},
             html=(
                 """
-                <input id="foo_0" name="array_0" type="text" value="val1" />
-                <input id="foo_1" name="array_1" type="text" value="val2" />
+                <input id="foo_0" name="array_0" type="text" value="val1">
+                <input id="foo_1" name="array_1" type="text" value="val2">
                 """
             )
         )
