@@ -1,13 +1,9 @@
 """
 Various complex queries that have been problematic in the past.
 """
-from __future__ import unicode_literals
-
 import threading
 
 from django.db import models
-from django.utils import six
-from django.utils.encoding import python_2_unicode_compatible
 
 
 class DumbCategory(models.Model):
@@ -19,7 +15,6 @@ class ProxyCategory(DumbCategory):
         proxy = True
 
 
-@python_2_unicode_compatible
 class NamedCategory(DumbCategory):
     name = models.CharField(max_length=10)
 
@@ -27,7 +22,6 @@ class NamedCategory(DumbCategory):
         return self.name
 
 
-@python_2_unicode_compatible
 class Tag(models.Model):
     name = models.CharField(max_length=10)
     parent = models.ForeignKey(
@@ -45,7 +39,6 @@ class Tag(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Note(models.Model):
     note = models.CharField(max_length=100)
     misc = models.CharField(max_length=10)
@@ -58,14 +51,13 @@ class Note(models.Model):
         return self.note
 
     def __init__(self, *args, **kwargs):
-        super(Note, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         # Regression for #13227 -- having an attribute that
         # is unpicklable doesn't stop you from cloning queries
         # that use objects of that type as an argument.
         self.lock = threading.Lock()
 
 
-@python_2_unicode_compatible
 class Annotation(models.Model):
     name = models.CharField(max_length=10)
     tag = models.ForeignKey(Tag, models.CASCADE)
@@ -75,7 +67,6 @@ class Annotation(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class ExtraInfo(models.Model):
     info = models.CharField(max_length=100)
     note = models.ForeignKey(Note, models.CASCADE, null=True)
@@ -88,7 +79,6 @@ class ExtraInfo(models.Model):
         return self.info
 
 
-@python_2_unicode_compatible
 class Author(models.Model):
     name = models.CharField(max_length=10)
     num = models.IntegerField(unique=True)
@@ -101,7 +91,6 @@ class Author(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Item(models.Model):
     name = models.CharField(max_length=10)
     created = models.DateTimeField()
@@ -117,7 +106,6 @@ class Item(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Report(models.Model):
     name = models.CharField(max_length=10)
     creator = models.ForeignKey(Author, models.SET_NULL, to_field='num', null=True)
@@ -130,7 +118,6 @@ class ReportComment(models.Model):
     report = models.ForeignKey(Report, models.CASCADE)
 
 
-@python_2_unicode_compatible
 class Ranking(models.Model):
     rank = models.IntegerField()
     author = models.ForeignKey(Author, models.CASCADE)
@@ -143,7 +130,6 @@ class Ranking(models.Model):
         return '%d: %s' % (self.rank, self.author.name)
 
 
-@python_2_unicode_compatible
 class Cover(models.Model):
     title = models.CharField(max_length=50)
     item = models.ForeignKey(Item, models.CASCADE)
@@ -155,12 +141,12 @@ class Cover(models.Model):
         return self.title
 
 
-@python_2_unicode_compatible
 class Number(models.Model):
     num = models.IntegerField()
+    other_num = models.IntegerField(null=True)
 
     def __str__(self):
-        return six.text_type(self.num)
+        return str(self.num)
 
 # Symmetrical m2m field with a normal field using the reverse accessor name
 # ("valid").
@@ -214,11 +200,10 @@ class LoopZ(models.Model):
 
 class CustomManager(models.Manager):
     def get_queryset(self):
-        qs = super(CustomManager, self).get_queryset()
+        qs = super().get_queryset()
         return qs.filter(public=True, tag__name='t1')
 
 
-@python_2_unicode_compatible
 class ManagedModel(models.Model):
     data = models.CharField(max_length=10)
     tag = models.ForeignKey(Tag, models.CASCADE)
@@ -239,7 +224,7 @@ class Detail(models.Model):
 
 class MemberManager(models.Manager):
     def get_queryset(self):
-        return super(MemberManager, self).get_queryset().select_related("details")
+        return super().get_queryset().select_related("details")
 
 
 class Member(models.Model):
@@ -277,7 +262,6 @@ class CustomPkTag(models.Model):
 # path to another model, and a return path from that model.
 
 
-@python_2_unicode_compatible
 class Celebrity(models.Model):
     name = models.CharField("Name", max_length=20)
     greatest_fan = models.ForeignKey("Fan", models.SET_NULL, null=True, unique=True)
@@ -296,7 +280,6 @@ class Fan(models.Model):
 # Multiple foreign keys
 
 
-@python_2_unicode_compatible
 class LeafA(models.Model):
     data = models.CharField(max_length=10)
 
@@ -313,7 +296,6 @@ class Join(models.Model):
     b = models.ForeignKey(LeafB, models.CASCADE)
 
 
-@python_2_unicode_compatible
 class ReservedName(models.Model):
     name = models.CharField(max_length=20)
     order = models.IntegerField()
@@ -324,7 +306,6 @@ class ReservedName(models.Model):
 # A simpler shared-foreign-key setup that can expose some problems.
 
 
-@python_2_unicode_compatible
 class SharedConnection(models.Model):
     data = models.CharField(max_length=10)
 
@@ -342,7 +323,6 @@ class PointerB(models.Model):
 # Multi-layer ordering
 
 
-@python_2_unicode_compatible
 class SingleObject(models.Model):
     name = models.CharField(max_length=10)
 
@@ -361,7 +341,6 @@ class RelatedObject(models.Model):
         ordering = ['single']
 
 
-@python_2_unicode_compatible
 class Plaything(models.Model):
     name = models.CharField(max_length=10)
     others = models.ForeignKey(RelatedObject, models.SET_NULL, null=True)
@@ -373,7 +352,6 @@ class Plaything(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Article(models.Model):
     name = models.CharField(max_length=20)
     created = models.DateTimeField()
@@ -382,7 +360,6 @@ class Article(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Food(models.Model):
     name = models.CharField(max_length=20, unique=True)
 
@@ -390,7 +367,6 @@ class Food(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Eaten(models.Model):
     food = models.ForeignKey(Food, models.SET_NULL, to_field="name", null=True)
     meal = models.CharField(max_length=20)
@@ -399,7 +375,6 @@ class Eaten(models.Model):
         return "%s at %s" % (self.food, self.meal)
 
 
-@python_2_unicode_compatible
 class Node(models.Model):
     num = models.IntegerField(unique=True)
     parent = models.ForeignKey("self", models.SET_NULL, to_field="num", null=True)
@@ -410,7 +385,6 @@ class Node(models.Model):
 # Bug #12252
 
 
-@python_2_unicode_compatible
 class ObjectA(models.Model):
     name = models.CharField(max_length=50)
 
@@ -431,7 +405,6 @@ class ChildObjectA(ObjectA):
     pass
 
 
-@python_2_unicode_compatible
 class ObjectB(models.Model):
     name = models.CharField(max_length=50)
     objecta = models.ForeignKey(ObjectA, models.CASCADE)
@@ -446,7 +419,6 @@ class ProxyObjectB(ObjectB):
         proxy = True
 
 
-@python_2_unicode_compatible
 class ObjectC(models.Model):
     name = models.CharField(max_length=50)
     objecta = models.ForeignKey(ObjectA, models.SET_NULL, null=True)
@@ -457,7 +429,6 @@ class ObjectC(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class SimpleCategory(models.Model):
     name = models.CharField(max_length=15)
 
@@ -465,7 +436,6 @@ class SimpleCategory(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class SpecialCategory(SimpleCategory):
     special_name = models.CharField(max_length=15)
 
@@ -473,7 +443,6 @@ class SpecialCategory(SimpleCategory):
         return self.name + " " + self.special_name
 
 
-@python_2_unicode_compatible
 class CategoryItem(models.Model):
     category = models.ForeignKey(SimpleCategory, models.CASCADE)
 
@@ -481,7 +450,14 @@ class CategoryItem(models.Model):
         return "category item: " + str(self.category)
 
 
-@python_2_unicode_compatible
+class MixedCaseFieldCategoryItem(models.Model):
+    CaTeGoRy = models.ForeignKey(SimpleCategory, models.CASCADE)
+
+
+class MixedCaseDbColumnCategoryItem(models.Model):
+    category = models.ForeignKey(SimpleCategory, models.CASCADE, db_column='CaTeGoRy_Id')
+
+
 class OneToOneCategory(models.Model):
     new_name = models.CharField(max_length=15)
     category = models.OneToOneField(SimpleCategory, models.CASCADE)
@@ -495,6 +471,12 @@ class CategoryRelationship(models.Model):
     second = models.ForeignKey(SimpleCategory, models.CASCADE, related_name='second_rel')
 
 
+class CommonMixedCaseForeignKeys(models.Model):
+    category = models.ForeignKey(CategoryItem, models.CASCADE)
+    mixed_case_field_category = models.ForeignKey(MixedCaseFieldCategoryItem, models.CASCADE)
+    mixed_case_db_column_category = models.ForeignKey(MixedCaseDbColumnCategoryItem, models.CASCADE)
+
+
 class NullableName(models.Model):
     name = models.CharField(max_length=20, null=True)
 
@@ -521,7 +503,6 @@ class ModelA(models.Model):
     d = models.ForeignKey(ModelD, models.CASCADE)
 
 
-@python_2_unicode_compatible
 class Job(models.Model):
     name = models.CharField(max_length=20, unique=True)
 
@@ -534,7 +515,6 @@ class JobResponsibilities(models.Model):
     responsibility = models.ForeignKey('Responsibility', models.CASCADE, to_field='description')
 
 
-@python_2_unicode_compatible
 class Responsibility(models.Model):
     description = models.CharField(max_length=20, unique=True)
     jobs = models.ManyToManyField(Job, through=JobResponsibilities,
@@ -567,7 +547,6 @@ class BaseA(models.Model):
     c = models.ForeignKey(FK3, models.SET_NULL, null=True)
 
 
-@python_2_unicode_compatible
 class Identifier(models.Model):
     name = models.CharField(max_length=100)
 
@@ -611,24 +590,23 @@ class MyObject(models.Model):
 # Models for #17600 regressions
 
 
-@python_2_unicode_compatible
 class Order(models.Model):
     id = models.IntegerField(primary_key=True)
+    name = models.CharField(max_length=12, null=True, default='')
 
     class Meta:
-        ordering = ('pk', )
+        ordering = ('pk',)
 
     def __str__(self):
         return '%s' % self.pk
 
 
-@python_2_unicode_compatible
 class OrderItem(models.Model):
     order = models.ForeignKey(Order, models.CASCADE, related_name='items')
     status = models.IntegerField()
 
     class Meta:
-        ordering = ('pk', )
+        ordering = ('pk',)
 
     def __str__(self):
         return '%s' % self.pk
@@ -638,7 +616,6 @@ class BaseUser(models.Model):
     pass
 
 
-@python_2_unicode_compatible
 class Task(models.Model):
     title = models.CharField(max_length=10)
     owner = models.ForeignKey(BaseUser, models.CASCADE, related_name='owner')
@@ -648,7 +625,6 @@ class Task(models.Model):
         return self.title
 
 
-@python_2_unicode_compatible
 class Staff(models.Model):
     name = models.CharField(max_length=10)
 
@@ -656,7 +632,6 @@ class Staff(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class StaffUser(BaseUser):
     staff = models.OneToOneField(Staff, models.CASCADE, related_name='user')
 
@@ -679,7 +654,6 @@ class Person(models.Model):
     name = models.CharField(max_length=128)
 
 
-@python_2_unicode_compatible
 class Company(models.Model):
     name = models.CharField(max_length=128)
     employees = models.ManyToManyField(Person, related_name='employers', through='Employment')
@@ -694,8 +668,6 @@ class Employment(models.Model):
     title = models.CharField(max_length=128)
 
 
-# Bug #22429
-
 class School(models.Model):
     pass
 
@@ -705,6 +677,8 @@ class Student(models.Model):
 
 
 class Classroom(models.Model):
+    name = models.CharField(max_length=20)
+    has_blackboard = models.BooleanField(null=True)
     school = models.ForeignKey(School, models.CASCADE)
     students = models.ManyToManyField(Student, related_name='classroom')
 
@@ -746,3 +720,8 @@ class RelatedIndividual(models.Model):
 
     class Meta:
         db_table = 'RelatedIndividual'
+
+
+class CustomDbColumn(models.Model):
+    custom_column = models.IntegerField(db_column='custom_name', null=True)
+    ip_address = models.GenericIPAddressField(null=True)
