@@ -1,9 +1,6 @@
-from __future__ import unicode_literals
-
-from django.db.models import F, IntegerField, Value
-from django.db.utils import DatabaseError
+from django.db.models import Exists, F, IntegerField, OuterRef, Value
+from django.db.utils import DatabaseError, NotSupportedError
 from django.test import TestCase, skipIfDBFeature, skipUnlessDBFeature
-from django.utils.six.moves import range
 
 from .models import Number, ReservedName
 
@@ -12,7 +9,7 @@ from .models import Number, ReservedName
 class QuerySetSetOperationTests(TestCase):
     @classmethod
     def setUpTestData(cls):
-        Number.objects.bulk_create(Number(num=i) for i in range(10))
+        Number.objects.bulk_create(Number(num=i, other_num=10 - i) for i in range(10))
 
     def number_transform(self, value):
         return value.num
@@ -103,12 +100,6 @@ class QuerySetSetOperationTests(TestCase):
         self.assertEqual(len(qs2.union(qs2)), 0)
         self.assertEqual(len(qs3.union(qs3)), 0)
 
-    def test_union_bad_kwarg(self):
-        qs1 = Number.objects.all()
-        msg = "union() received an unexpected keyword argument 'bad'"
-        with self.assertRaisesMessage(TypeError, msg):
-            self.assertEqual(len(list(qs1.union(qs1, bad=True))), 20)
-
     def test_limits(self):
         qs1 = Number.objects.all()
         qs2 = Number.objects.all()
@@ -119,6 +110,11 @@ class QuerySetSetOperationTests(TestCase):
         qs2 = Number.objects.filter(num__gte=2, num__lte=3)
         self.assertNumbersEqual(qs1.union(qs2).order_by('-num'), [3, 2, 1, 0])
 
+    def test_ordering_by_f_expression(self):
+        qs1 = Number.objects.filter(num__lte=1)
+        qs2 = Number.objects.filter(num__gte=2, num__lte=3)
+        self.assertNumbersEqual(qs1.union(qs2).order_by(F('num').desc()), [3, 2, 1, 0])
+
     def test_union_with_values(self):
         ReservedName.objects.create(name='a', order=2)
         qs1 = ReservedName.objects.all()
@@ -128,6 +124,32 @@ class QuerySetSetOperationTests(TestCase):
         reserved_name = qs1.union(qs1).values_list('name', 'order', 'id').get()
         self.assertEqual(reserved_name[:2], ('a', 2))
 
+    def test_union_with_two_annotated_values_list(self):
+        qs1 = Number.objects.filter(num=1).annotate(
+            count=Value(0, IntegerField()),
+        ).values_list('num', 'count')
+        qs2 = Number.objects.filter(num=2).values('pk').annotate(
+            count=F('num'),
+        ).annotate(
+            num=Value(1, IntegerField()),
+        ).values_list('num', 'count')
+        self.assertCountEqual(qs1.union(qs2), [(1, 0), (2, 1)])
+
+    def test_union_with_extra_and_values_list(self):
+        qs1 = Number.objects.filter(num=1).extra(
+            select={'count': 0},
+        ).values_list('num', 'count')
+        qs2 = Number.objects.filter(num=2).extra(select={'count': 1})
+        self.assertCountEqual(qs1.union(qs2), [(1, 0), (2, 1)])
+
+    def test_union_with_values_list_on_annotated_and_unannotated(self):
+        ReservedName.objects.create(name='rn1', order=1)
+        qs1 = Number.objects.annotate(
+            has_reserved_name=Exists(ReservedName.objects.filter(order=OuterRef('num')))
+        ).filter(has_reserved_name=True)
+        qs2 = Number.objects.filter(num=9)
+        self.assertCountEqual(qs1.union(qs2).values_list('num', flat=True), [1, 9])
+
     def test_count_union(self):
         qs1 = Number.objects.filter(num__lte=1).values('num')
         qs2 = Number.objects.filter(num__gte=2, num__lte=3).values('num')
@@ -170,8 +192,8 @@ class QuerySetSetOperationTests(TestCase):
     def test_unsupported_intersection_raises_db_error(self):
         qs1 = Number.objects.all()
         qs2 = Number.objects.all()
-        msg = 'intersection not supported on this database backend'
-        with self.assertRaisesMessage(DatabaseError, msg):
+        msg = 'intersection is not supported on this database backend'
+        with self.assertRaisesMessage(NotSupportedError, msg):
             list(qs1.intersection(qs2))
 
     def test_combining_multiple_models(self):
@@ -197,3 +219,16 @@ class QuerySetSetOperationTests(TestCase):
             list(qs1.union(qs2).order_by('num'))
         # switched order, now 'exists' again:
         list(qs2.union(qs1).order_by('num'))
+
+    @skipUnlessDBFeature('supports_select_difference', 'supports_select_intersection')
+    def test_qs_with_subcompound_qs(self):
+        qs1 = Number.objects.all()
+        qs2 = Number.objects.intersection(Number.objects.filter(num__gt=1))
+        self.assertEqual(qs1.difference(qs2).count(), 2)
+
+    def test_order_by_same_type(self):
+        qs = Number.objects.all()
+        union = qs.union(qs)
+        numbers = list(range(10))
+        self.assertNumbersEqual(union.order_by('num'), numbers)
+        self.assertNumbersEqual(union.order_by('other_num'), reversed(numbers))
