@@ -7,7 +7,7 @@ class Author(models.Model):
     dob = models.DateField()
 
     def __init__(self, *args, **kwargs):
-        super(Author, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         # Protect against annotations being passed to __init__ --
         # this'll make the test suite get angry if annotations aren't
         # treated differently than fields.
@@ -18,17 +18,22 @@ class Author(models.Model):
 
 class Book(models.Model):
     title = models.CharField(max_length=255)
-    author = models.ForeignKey(Author)
+    author = models.ForeignKey(Author, models.CASCADE)
     paperback = models.BooleanField(default=False)
     opening_line = models.TextField()
 
 
 class BookFkAsPk(models.Model):
-    book = models.ForeignKey(Book, primary_key=True, db_column="not_the_default")
+    book = models.ForeignKey(Book, models.CASCADE, primary_key=True, db_column="not_the_default")
 
 
 class Coffee(models.Model):
     brand = models.CharField(max_length=255, db_column="name")
+    price = models.DecimalField(max_digits=10, decimal_places=2, default=0)
+
+
+class MixedCaseIDColumn(models.Model):
+    id = models.AutoField(primary_key=True, db_column='MiXeD_CaSe_Id')
 
 
 class Reviewer(models.Model):
