@@ -1,9 +1,10 @@
+from functools import partial
+
 from django.db import models
 from django.db.models.fields.related import (
     RECURSIVE_RELATIONSHIP_CONSTANT, ManyToManyDescriptor, ManyToManyField,
     ManyToManyRel, RelatedField, create_many_to_many_intermediary_model,
 )
-from django.utils.functional import curry
 
 
 class CustomManyToManyField(RelatedField):
@@ -13,36 +14,37 @@ class CustomManyToManyField(RelatedField):
     """
     many_to_many = True
 
-    def __init__(self, to, db_constraint=True, swappable=True, **kwargs):
+    def __init__(self, to, db_constraint=True, swappable=True, related_name=None, related_query_name=None,
+                 limit_choices_to=None, symmetrical=None, through=None, through_fields=None, db_table=None, **kwargs):
         try:
             to._meta
         except AttributeError:
             to = str(to)
         kwargs['rel'] = ManyToManyRel(
             self, to,
-            related_name=kwargs.pop('related_name', None),
-            related_query_name=kwargs.pop('related_query_name', None),
-            limit_choices_to=kwargs.pop('limit_choices_to', None),
-            symmetrical=kwargs.pop('symmetrical', to == RECURSIVE_RELATIONSHIP_CONSTANT),
-            through=kwargs.pop('through', None),
-            through_fields=kwargs.pop('through_fields', None),
+            related_name=related_name,
+            related_query_name=related_query_name,
+            limit_choices_to=limit_choices_to,
+            symmetrical=symmetrical if symmetrical is not None else (to == RECURSIVE_RELATIONSHIP_CONSTANT),
+            through=through,
+            through_fields=through_fields,
             db_constraint=db_constraint,
         )
         self.swappable = swappable
-        self.db_table = kwargs.pop('db_table', None)
+        self.db_table = db_table
         if kwargs['rel'].through is not None:
             assert self.db_table is None, "Cannot specify a db_table if an intermediary model is used."
-        super(CustomManyToManyField, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     def contribute_to_class(self, cls, name, **kwargs):
         if self.remote_field.symmetrical and (
                 self.remote_field.model == "self" or self.remote_field.model == cls._meta.object_name):
             self.remote_field.related_name = "%s_rel_+" % name
-        super(CustomManyToManyField, self).contribute_to_class(cls, name, **kwargs)
+        super().contribute_to_class(cls, name, **kwargs)
         if not self.remote_field.through and not cls._meta.abstract and not cls._meta.swapped:
             self.remote_field.through = create_many_to_many_intermediary_model(self, cls)
         setattr(cls, self.name, ManyToManyDescriptor(self.remote_field))
-        self.m2m_db_table = curry(self._get_m2m_db_table, cls._meta)
+        self.m2m_db_table = partial(self._get_m2m_db_table, cls._meta)
 
     def get_internal_type(self):
         return 'ManyToManyField'
