@@ -1,6 +1,5 @@
 from django.apps.registry import Apps
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 # Because we want to test creation and deletion of these as separate things,
 # these models are all inserted into a separate Apps so the main test
@@ -12,6 +11,22 @@ new_apps = Apps()
 class Author(models.Model):
     name = models.CharField(max_length=255)
     height = models.PositiveIntegerField(null=True, blank=True)
+    weight = models.IntegerField(null=True, blank=True)
+    uuid = models.UUIDField(null=True)
+
+    class Meta:
+        apps = new_apps
+
+
+class AuthorCharFieldWithIndex(models.Model):
+    char_field = models.CharField(max_length=31, db_index=True)
+
+    class Meta:
+        apps = new_apps
+
+
+class AuthorTextFieldWithIndex(models.Model):
+    text_field = models.TextField(db_index=True)
 
     class Meta:
         apps = new_apps
@@ -33,6 +48,38 @@ class AuthorWithEvenLongerName(models.Model):
         apps = new_apps
 
 
+class AuthorWithIndexedName(models.Model):
+    name = models.CharField(max_length=255, db_index=True)
+
+    class Meta:
+        apps = new_apps
+
+
+class AuthorWithUniqueName(models.Model):
+    name = models.CharField(max_length=255, unique=True)
+
+    class Meta:
+        apps = new_apps
+
+
+class AuthorWithIndexedNameAndBirthday(models.Model):
+    name = models.CharField(max_length=255)
+    birthday = models.DateField()
+
+    class Meta:
+        apps = new_apps
+        index_together = [['name', 'birthday']]
+
+
+class AuthorWithUniqueNameAndBirthday(models.Model):
+    name = models.CharField(max_length=255)
+    birthday = models.DateField()
+
+    class Meta:
+        apps = new_apps
+        unique_together = [['name', 'birthday']]
+
+
 class Book(models.Model):
     author = models.ForeignKey(Author, models.CASCADE)
     title = models.CharField(max_length=100, db_index=True)
@@ -159,7 +206,6 @@ class TagUniqueRename(models.Model):
 
 
 # Based on tests/reserved_names/models.py
-@python_2_unicode_compatible
 class Thing(models.Model):
     when = models.CharField(max_length=1, primary_key=True)
 
@@ -182,3 +228,6 @@ class UniqueTest(models.Model):
 class Node(models.Model):
     node_id = models.AutoField(primary_key=True)
     parent = models.ForeignKey('self', models.CASCADE, null=True, blank=True)
+
+    class Meta:
+        apps = new_apps
