@@ -23,6 +23,20 @@ class EUCity(models.Model):
     country = models.ForeignKey(EUCountry, models.CASCADE)
 
 
+class CountryProxy(Country):
+    class Meta:
+        proxy = True
+
+
+class CountryProxyProxy(CountryProxy):
+    class Meta:
+        proxy = True
+
+
+class CityCountryProxy(models.Model):
+    country = models.ForeignKey(CountryProxyProxy, models.CASCADE)
+
+
 class Person(models.Model):
     name = models.CharField(max_length=30)
     born = models.ForeignKey(City, models.CASCADE, related_name='+')
