@@ -12,12 +12,10 @@ from django.contrib.contenttypes.fields import (
 )
 from django.contrib.contenttypes.models import ContentType
 from django.db import models
-from django.utils.encoding import python_2_unicode_compatible
 
 # Who remembers high school biology?
 
 
-@python_2_unicode_compatible
 class Domain(models.Model):
     name = models.CharField(max_length=50)
 
@@ -25,7 +23,6 @@ class Domain(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Kingdom(models.Model):
     name = models.CharField(max_length=50)
     domain = models.ForeignKey(Domain, models.CASCADE)
@@ -34,7 +31,6 @@ class Kingdom(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Phylum(models.Model):
     name = models.CharField(max_length=50)
     kingdom = models.ForeignKey(Kingdom, models.CASCADE)
@@ -43,7 +39,6 @@ class Phylum(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Klass(models.Model):
     name = models.CharField(max_length=50)
     phylum = models.ForeignKey(Phylum, models.CASCADE)
@@ -52,7 +47,6 @@ class Klass(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Order(models.Model):
     name = models.CharField(max_length=50)
     klass = models.ForeignKey(Klass, models.CASCADE)
@@ -61,7 +55,6 @@ class Order(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Family(models.Model):
     name = models.CharField(max_length=50)
     order = models.ForeignKey(Order, models.CASCADE)
@@ -70,7 +63,6 @@ class Family(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Genus(models.Model):
     name = models.CharField(max_length=50)
     family = models.ForeignKey(Family, models.CASCADE)
@@ -79,7 +71,6 @@ class Genus(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Species(models.Model):
     name = models.CharField(max_length=50)
     genus = models.ForeignKey(Genus, models.CASCADE)
@@ -90,7 +81,6 @@ class Species(models.Model):
 # and we'll invent a new thing so we have a model with two foreign keys
 
 
-@python_2_unicode_compatible
 class HybridSpecies(models.Model):
     name = models.CharField(max_length=50)
     parent_1 = models.ForeignKey(Species, models.CASCADE, related_name='child_1')
@@ -100,7 +90,6 @@ class HybridSpecies(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Topping(models.Model):
     name = models.CharField(max_length=30)
 
@@ -108,7 +97,6 @@ class Topping(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class Pizza(models.Model):
     name = models.CharField(max_length=100)
     toppings = models.ManyToManyField(Topping)
@@ -117,7 +105,6 @@ class Pizza(models.Model):
         return self.name
 
 
-@python_2_unicode_compatible
 class TaggedItem(models.Model):
     tag = models.CharField(max_length=30)
 
@@ -129,7 +116,6 @@ class TaggedItem(models.Model):
         return self.tag
 
 
-@python_2_unicode_compatible
 class Bookmark(models.Model):
     url = models.URLField()
     tags = GenericRelation(TaggedItem)
