@@ -1,7 +1,4 @@
-from __future__ import unicode_literals
-
 from django.test import TestCase
-from django.utils import six
 
 from .models import (
     A, B, Building, C, Chick, Child, Class, Client, ClientStatus, Connection,
@@ -37,7 +34,7 @@ class SelectRelatedRegressTests(TestCase):
 
         connections = Connection.objects.filter(start__device__building=b, end__device__building=b).order_by('id')
         self.assertEqual(
-            [(c.id, six.text_type(c.start), six.text_type(c.end)) for c in connections],
+            [(c.id, str(c.start), str(c.end)) for c in connections],
             [(c1.id, 'router/4', 'switch/7'), (c2.id, 'switch/7', 'server/1')]
         )
 
@@ -48,7 +45,7 @@ class SelectRelatedRegressTests(TestCase):
             .order_by('id')
         )
         self.assertEqual(
-            [(c.id, six.text_type(c.start), six.text_type(c.end)) for c in connections],
+            [(c.id, str(c.start), str(c.end)) for c in connections],
             [(c1.id, 'router/4', 'switch/7'), (c2.id, 'switch/7', 'server/1')]
         )
 
@@ -187,7 +184,7 @@ class SelectRelatedRegressTests(TestCase):
                              c_a=a, c_b=b)
         results = C.objects.all().only('name', 'lots_of_text', 'c_a', 'c_b', 'c_b__lots_of_text',
                                        'c_a__name', 'c_b__name').select_related()
-        self.assertQuerysetEqual(results, [c], lambda x: x)
+        self.assertSequenceEqual(results, [c])
         with self.assertNumQueries(0):
             qs_c = results[0]
             self.assertEqual(qs_c.name, 'c')
