@@ -1,67 +1,56 @@
-import logging
 from io import BytesIO
 
 from django.core.handlers.wsgi import WSGIRequest
 from django.core.servers.basehttp import WSGIRequestHandler
 from django.test import SimpleTestCase
 from django.test.client import RequestFactory
-from django.test.utils import patch_logger
 
 
-class Stub(object):
+class Stub:
     def __init__(self, **kwargs):
         self.__dict__.update(kwargs)
 
+    def sendall(self, data):
+        self.makefile('wb').write(data)
+
 
 class WSGIRequestHandlerTestCase(SimpleTestCase):
+    request_factory = RequestFactory()
 
     def test_log_message(self):
-        # Silence the django.server logger by replacing its StreamHandler with
-        # NullHandler.
-        logger = logging.getLogger('django.server')
-        original_handlers = logger.handlers
-        logger.handlers = [logging.NullHandler()]
-        try:
-            request = WSGIRequest(RequestFactory().get('/').environ)
-            request.makefile = lambda *args, **kwargs: BytesIO()
-            handler = WSGIRequestHandler(request, '192.168.0.2', None)
-            level_status_codes = {
-                'info': [200, 301, 304],
-                'warning': [400, 403, 404],
-                'error': [500, 503],
-            }
-
-            def _log_level_code(level, status_code):
-                with patch_logger('django.server', level) as messages:
+        request = WSGIRequest(self.request_factory.get('/').environ)
+        request.makefile = lambda *args, **kwargs: BytesIO()
+        handler = WSGIRequestHandler(request, '192.168.0.2', None)
+        level_status_codes = {
+            'info': [200, 301, 304],
+            'warning': [400, 403, 404],
+            'error': [500, 503],
+        }
+        for level, status_codes in level_status_codes.items():
+            for status_code in status_codes:
+                # The correct level gets the message.
+                with self.assertLogs('django.server', level.upper()) as cm:
                     handler.log_message('GET %s %s', 'A', str(status_code))
-                return messages
-
-            for level, status_codes in level_status_codes.items():
-                for status_code in status_codes:
-                    # The correct level gets the message.
-                    messages = _log_level_code(level, status_code)
-                    self.assertIn('GET A %d' % status_code, messages[0])
-
-                    # Incorrect levels shouldn't have any messages.
-                    for wrong_level in level_status_codes.keys():
-                        if wrong_level != level:
-                            messages = _log_level_code(wrong_level, status_code)
-                            self.assertEqual(len(messages), 0)
-        finally:
-            logger.handlers = original_handlers
+                self.assertIn('GET A %d' % status_code, cm.output[0])
+                # Incorrect levels don't have any messages.
+                for wrong_level in level_status_codes:
+                    if wrong_level != level:
+                        with self.assertLogs('django.server', 'INFO') as cm:
+                            handler.log_message('GET %s %s', 'A', str(status_code))
+                        self.assertNotEqual(cm.records[0].levelname, wrong_level.upper())
 
     def test_https(self):
-        request = WSGIRequest(RequestFactory().get('/').environ)
+        request = WSGIRequest(self.request_factory.get('/').environ)
         request.makefile = lambda *args, **kwargs: BytesIO()
 
         handler = WSGIRequestHandler(request, '192.168.0.2', None)
 
-        with patch_logger('django.server', 'error') as messages:
-            handler.log_message("GET %s %s", str('\x16\x03'), "4")
+        with self.assertLogs('django.server', 'ERROR') as cm:
+            handler.log_message("GET %s %s", '\x16\x03', "4")
         self.assertIn(
             "You're accessing the development server over HTTPS, "
             "but it only supports HTTP.",
-            messages[0]
+            cm.records[0].getMessage()
         )
 
     def test_strips_underscore_headers(self):
@@ -78,7 +67,7 @@ class WSGIRequestHandlerTestCase(SimpleTestCase):
                 '%s:%s' % (k, v) for k, v in environ.items()
                 if k.startswith('HTTP_')
             )
-            yield (','.join(http_environ_items)).encode('utf-8')
+            yield (','.join(http_environ_items)).encode()
 
         rfile = BytesIO()
         rfile.write(b"GET / HTTP/1.0\r\n")
@@ -104,8 +93,8 @@ class WSGIRequestHandlerTestCase(SimpleTestCase):
         request = Stub(makefile=makefile)
         server = Stub(base_environ={}, get_app=lambda: test_app)
 
-        # We don't need to check stderr, but we don't want it in test output
-        with patch_logger('django.server', 'info'):
+        # Prevent logging from appearing in test output.
+        with self.assertLogs('django.server', 'INFO'):
             # instantiating a handler runs the request as side effect
             WSGIRequestHandler(request, '192.168.0.2', server)
 
