@@ -1,17 +1,16 @@
-from __future__ import unicode_literals
+from unittest import mock
 
 from django.apps.registry import Apps
 from django.db import models
 from django.db.models import signals
 from django.dispatch import receiver
-from django.test import TestCase, mock
+from django.test import SimpleTestCase, TestCase
 from django.test.utils import isolate_apps
-from django.utils import six
 
 from .models import Author, Book, Car, Person
 
 
-class BaseSignalTest(TestCase):
+class BaseSignalSetup:
     def setUp(self):
         # Save up the number of connected signals so that we can check at the
         # end that all the signals we register get properly unregistered (#9989)
@@ -33,7 +32,7 @@ class BaseSignalTest(TestCase):
         self.assertEqual(self.pre_signals, post_signals)
 
 
-class SignalTests(BaseSignalTest):
+class SignalTests(BaseSignalSetup, TestCase):
     def test_model_pre_init_and_post_init(self):
         data = []
 
@@ -125,7 +124,7 @@ class SignalTests(BaseSignalTest):
             )
 
         # #8285: signals can be any callable
-        class PostDeleteHandler(object):
+        class PostDeleteHandler:
             def __init__(self, data):
                 self.data = data
 
@@ -162,7 +161,7 @@ class SignalTests(BaseSignalTest):
                 Person.objects.all(), [
                     "James Jones",
                 ],
-                six.text_type
+                str
             )
         finally:
             signals.pre_delete.disconnect(pre_delete_handler)
@@ -250,7 +249,7 @@ class SignalTests(BaseSignalTest):
         dispatching.
         """
 
-        class Handler(object):
+        class Handler:
             def __init__(self, param):
                 self.param = param
                 self._run = False
@@ -282,9 +281,9 @@ class SignalTests(BaseSignalTest):
         ref.assert_not_called()
 
 
-class LazyModelRefTest(BaseSignalTest):
+class LazyModelRefTests(BaseSignalSetup, SimpleTestCase):
     def setUp(self):
-        super(LazyModelRefTest, self).setUp()
+        super().setUp()
         self.received = []
 
     def receiver(self, **kwargs):
