@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 from django.apps import apps
 from django.apps.registry import Apps
 from django.conf import settings
@@ -12,21 +10,20 @@ from django.contrib.sites.shortcuts import get_current_site
 from django.core.exceptions import ObjectDoesNotExist, ValidationError
 from django.db.models.signals import post_migrate
 from django.http import HttpRequest, HttpResponse
-from django.test import TestCase, modify_settings, override_settings
+from django.test import (
+    SimpleTestCase, TestCase, modify_settings, override_settings,
+)
 from django.test.utils import captured_stdout
 
 
 @modify_settings(INSTALLED_APPS={'append': 'django.contrib.sites'})
 class SitesFrameworkTests(TestCase):
-    multi_db = True
+    databases = {'default', 'other'}
 
-    def setUp(self):
-        self.site = Site(
-            id=settings.SITE_ID,
-            domain="example.com",
-            name="example.com",
-        )
-        self.site.save()
+    @classmethod
+    def setUpTestData(cls):
+        cls.site = Site(id=settings.SITE_ID, domain='example.com', name='example.com')
+        cls.site.save()
 
     def tearDown(self):
         Site.objects.clear_cache()
@@ -205,42 +202,49 @@ class SitesFrameworkTests(TestCase):
         self.assertEqual(Site.objects.get_by_natural_key(self.site.domain), self.site)
         self.assertEqual(self.site.natural_key(), (self.site.domain,))
 
-    @override_settings(ALLOWED_HOSTS=['example.com'])
-    def test_requestsite_save_notimplemented_msg(self):
-        # Test response msg for RequestSite.save NotImplementedError
+
+@override_settings(ALLOWED_HOSTS=['example.com'])
+class RequestSiteTests(SimpleTestCase):
+
+    def setUp(self):
         request = HttpRequest()
-        request.META = {
-            "HTTP_HOST": "example.com",
-        }
+        request.META = {'HTTP_HOST': 'example.com'}
+        self.site = RequestSite(request)
+
+    def test_init_attributes(self):
+        self.assertEqual(self.site.domain, 'example.com')
+        self.assertEqual(self.site.name, 'example.com')
+
+    def test_str(self):
+        self.assertEqual(str(self.site), 'example.com')
+
+    def test_save(self):
         msg = 'RequestSite cannot be saved.'
         with self.assertRaisesMessage(NotImplementedError, msg):
-            RequestSite(request).save()
+            self.site.save()
 
-    @override_settings(ALLOWED_HOSTS=['example.com'])
-    def test_requestsite_delete_notimplemented_msg(self):
-        # Test response msg for RequestSite.delete NotImplementedError
-        request = HttpRequest()
-        request.META = {
-            "HTTP_HOST": "example.com",
-        }
+    def test_delete(self):
         msg = 'RequestSite cannot be deleted.'
         with self.assertRaisesMessage(NotImplementedError, msg):
-            RequestSite(request).delete()
+            self.site.delete()
 
 
-class JustOtherRouter(object):
+class JustOtherRouter:
     def allow_migrate(self, db, app_label, **hints):
         return db == 'other'
 
 
 @modify_settings(INSTALLED_APPS={'append': 'django.contrib.sites'})
 class CreateDefaultSiteTests(TestCase):
-    multi_db = True
+    databases = {'default', 'other'}
+
+    @classmethod
+    def setUpTestData(cls):
+        # Delete the site created as part of the default migration process.
+        Site.objects.all().delete()
 
     def setUp(self):
         self.app_config = apps.get_app_config('sites')
-        # Delete the site created as part of the default migration process.
-        Site.objects.all().delete()
 
     def test_basic(self):
         """
