@@ -1,12 +1,11 @@
-from __future__ import unicode_literals
-
-from swappable_models.models import Article
+from io import StringIO
 
 from django.contrib.auth.models import Permission
 from django.contrib.contenttypes.models import ContentType
 from django.core import management
 from django.test import TestCase, override_settings
-from django.utils.six import StringIO
+
+from .models import Article
 
 
 class SwappableModelTests(TestCase):
