@@ -161,7 +161,7 @@ class IfChangedTests(SimpleTestCase):
     @classmethod
     def setUpClass(cls):
         cls.engine = Engine()
-        super(IfChangedTests, cls).setUpClass()
+        super().setUpClass()
 
     def test_ifchanged_concurrency(self):
         """
@@ -210,7 +210,7 @@ class IfChangedTests(SimpleTestCase):
                 'include': '{% ifchanged %}{{ x }}{% endifchanged %}',
             }),
         ])
-        output = engine.render_to_string('template', dict(vars=[1, 1, 2, 2, 3, 3]))
+        output = engine.render_to_string('template', {'vars': [1, 1, 2, 2, 3, 3]})
         self.assertEqual(output, "123")
 
     def test_include_state(self):
@@ -221,5 +221,5 @@ class IfChangedTests(SimpleTestCase):
                 'include': '{% ifchanged %}{{ x }}{% endifchanged %}',
             }),
         ])
-        output = engine.render_to_string('template', dict(vars=[1, 1, 2, 2, 3, 3]))
+        output = engine.render_to_string('template', {'vars': [1, 1, 2, 2, 3, 3]})
         self.assertEqual(output, '112233')
