@@ -1,10 +1,9 @@
 import operator
-import warnings
 
 from django import template
 from django.template.defaultfilters import stringfilter
-from django.utils import six
 from django.utils.html import escape, format_html
+from django.utils.safestring import mark_safe
 
 register = template.Library()
 
@@ -15,6 +14,13 @@ def trim(value, num):
     return value[:num]
 
 
+@register.filter
+@mark_safe
+def make_data_div(value):
+    """A filter that uses a decorator (@mark_safe)."""
+    return '<div data-name="%s"></div>' % value
+
+
 @register.filter
 def noop(value, param=None):
     """A noop filter that always return its first argument and does nothing with
@@ -32,6 +38,8 @@ def context_stack_length(context):
 def no_params():
     """Expected no_params __doc__"""
     return "no_params - Expected result"
+
+
 no_params.anything = "Expected no_params __dict__"
 
 
@@ -39,6 +47,8 @@ no_params.anything = "Expected no_params __dict__"
 def one_param(arg):
     """Expected one_param __doc__"""
     return "one_param - Expected result: %s" % arg
+
+
 one_param.anything = "Expected one_param __dict__"
 
 
@@ -46,6 +56,8 @@ one_param.anything = "Expected one_param __dict__"
 def explicit_no_context(arg):
     """Expected explicit_no_context __doc__"""
     return "explicit_no_context - Expected result: %s" % arg
+
+
 explicit_no_context.anything = "Expected explicit_no_context __dict__"
 
 
@@ -53,6 +65,8 @@ explicit_no_context.anything = "Expected explicit_no_context __dict__"
 def no_params_with_context(context):
     """Expected no_params_with_context __doc__"""
     return "no_params_with_context - Expected result (context value: %s)" % context['value']
+
+
 no_params_with_context.anything = "Expected no_params_with_context __dict__"
 
 
@@ -60,6 +74,8 @@ no_params_with_context.anything = "Expected no_params_with_context __dict__"
 def params_and_context(context, arg):
     """Expected params_and_context __doc__"""
     return "params_and_context - Expected result (context value: %s): %s" % (context['value'], arg)
+
+
 params_and_context.anything = "Expected params_and_context __dict__"
 
 
@@ -67,13 +83,27 @@ params_and_context.anything = "Expected params_and_context __dict__"
 def simple_two_params(one, two):
     """Expected simple_two_params __doc__"""
     return "simple_two_params - Expected result: %s, %s" % (one, two)
+
+
 simple_two_params.anything = "Expected simple_two_params __dict__"
 
 
+@register.simple_tag
+def simple_keyword_only_param(*, kwarg):
+    return "simple_keyword_only_param - Expected result: %s" % kwarg
+
+
+@register.simple_tag
+def simple_keyword_only_default(*, kwarg=42):
+    return "simple_keyword_only_default - Expected result: %s" % kwarg
+
+
 @register.simple_tag
 def simple_one_default(one, two='hi'):
     """Expected simple_one_default __doc__"""
     return "simple_one_default - Expected result: %s, %s" % (one, two)
+
+
 simple_one_default.anything = "Expected simple_one_default __dict__"
 
 
@@ -81,15 +111,19 @@ simple_one_default.anything = "Expected simple_one_default __dict__"
 def simple_unlimited_args(one, two='hi', *args):
     """Expected simple_unlimited_args __doc__"""
     return "simple_unlimited_args - Expected result: %s" % (
-        ', '.join(six.text_type(arg) for arg in [one, two] + list(args))
+        ', '.join(str(arg) for arg in [one, two, *args])
     )
+
+
 simple_unlimited_args.anything = "Expected simple_unlimited_args __dict__"
 
 
 @register.simple_tag
 def simple_only_unlimited_args(*args):
     """Expected simple_only_unlimited_args __doc__"""
-    return "simple_only_unlimited_args - Expected result: %s" % ', '.join(six.text_type(arg) for arg in args)
+    return "simple_only_unlimited_args - Expected result: %s" % ', '.join(str(arg) for arg in args)
+
+
 simple_only_unlimited_args.anything = "Expected simple_only_unlimited_args __dict__"
 
 
@@ -97,11 +131,13 @@ simple_only_unlimited_args.anything = "Expected simple_only_unlimited_args __dic
 def simple_unlimited_args_kwargs(one, two='hi', *args, **kwargs):
     """Expected simple_unlimited_args_kwargs __doc__"""
     # Sort the dictionary by key to guarantee the order for testing.
-    sorted_kwarg = sorted(six.iteritems(kwargs), key=operator.itemgetter(0))
+    sorted_kwarg = sorted(kwargs.items(), key=operator.itemgetter(0))
     return "simple_unlimited_args_kwargs - Expected result: %s / %s" % (
-        ', '.join(six.text_type(arg) for arg in [one, two] + list(args)),
+        ', '.join(str(arg) for arg in [one, two, *args]),
         ', '.join('%s=%s' % (k, v) for (k, v) in sorted_kwarg)
     )
+
+
 simple_unlimited_args_kwargs.anything = "Expected simple_unlimited_args_kwargs __dict__"
 
 
@@ -109,6 +145,8 @@ simple_unlimited_args_kwargs.anything = "Expected simple_unlimited_args_kwargs _
 def simple_tag_without_context_parameter(arg):
     """Expected simple_tag_without_context_parameter __doc__"""
     return "Expected result"
+
+
 simple_tag_without_context_parameter.anything = "Expected simple_tag_without_context_parameter __dict__"
 
 
@@ -144,20 +182,20 @@ def use_l10n(context):
 def minustwo_overridden_name(value):
     return value - 2
 
+
 register.simple_tag(lambda x: x - 1, name='minusone')
 
 
-with warnings.catch_warnings():
-    warnings.simplefilter('ignore')
+@register.tag('counter')
+def counter(parser, token):
+    return CounterNode()
 
-    @register.assignment_tag
-    def assignment_no_params():
-        """Expected assignment_no_params __doc__"""
-        return "assignment_no_params - Expected result"
-    assignment_no_params.anything = "Expected assignment_no_params __dict__"
 
-    @register.assignment_tag(takes_context=True)
-    def assignment_tag_without_context_parameter(arg):
-        """Expected assignment_tag_without_context_parameter __doc__"""
-        return "Expected result"
-    assignment_tag_without_context_parameter.anything = "Expected assignment_tag_without_context_parameter __dict__"
+class CounterNode(template.Node):
+    def __init__(self):
+        self.count = 0
+
+    def render(self, context):
+        count = self.count
+        self.count = count + 1
+        return count
