@@ -1,14 +1,10 @@
-from __future__ import unicode_literals
-
 import os
-from unittest import skipUnless
 
 from django.template import Context, Engine, TemplateSyntaxError
 from django.template.base import Node
 from django.template.library import InvalidTemplateLibrary
 from django.test import SimpleTestCase
 from django.test.utils import extend_sys_path
-from django.utils import six
 
 from .templatetags import custom, inclusion
 from .utils import ROOT
@@ -29,13 +25,18 @@ class CustomFilterTests(SimpleTestCase):
             "abcde"
         )
 
+    def test_decorated_filter(self):
+        engine = Engine(libraries=LIBRARIES)
+        t = engine.from_string('{% load custom %}{{ name|make_data_div }}')
+        self.assertEqual(t.render(Context({'name': 'foo'})), '<div data-name="foo"></div>')
+
 
 class TagTestCase(SimpleTestCase):
 
     @classmethod
     def setUpClass(cls):
         cls.engine = Engine(app_dirs=True, libraries=LIBRARIES)
-        super(TagTestCase, cls).setUpClass()
+        super().setUpClass()
 
     def verify_tag(self, tag, name):
         self.assertEqual(tag.__name__, name)
@@ -57,6 +58,10 @@ class SimpleTagTests(TagTestCase):
             ('{% load custom %}{% params_and_context 37 %}',
                 'params_and_context - Expected result (context value: 42): 37'),
             ('{% load custom %}{% simple_two_params 37 42 %}', 'simple_two_params - Expected result: 37, 42'),
+            ('{% load custom %}{% simple_keyword_only_param kwarg=37 %}',
+                'simple_keyword_only_param - Expected result: 37'),
+            ('{% load custom %}{% simple_keyword_only_default %}',
+                'simple_keyword_only_default - Expected result: 42'),
             ('{% load custom %}{% simple_one_default 37 %}', 'simple_one_default - Expected result: 37, hi'),
             ('{% load custom %}{% simple_one_default 37 two="hello" %}',
                 'simple_one_default - Expected result: 37, hello'),
@@ -90,6 +95,8 @@ class SimpleTagTests(TagTestCase):
                 '{% load custom %}{% simple_two_params 37 42 56 %}'),
             ("'simple_one_default' received too many positional arguments",
                 '{% load custom %}{% simple_one_default 37 42 56 %}'),
+            ("'simple_keyword_only_param' did not receive value(s) for the argument(s): 'kwarg'",
+                '{% load custom %}{% simple_keyword_only_param %}'),
             ("'simple_unlimited_args_kwargs' received some positional argument(s) after some keyword argument(s)",
                 '{% load custom %}{% simple_unlimited_args_kwargs 37 40|add:2 eggs="scrambled" 56 four=1|add:3 %}'),
             ("'simple_unlimited_args_kwargs' received multiple values for keyword argument 'eggs'",
@@ -127,7 +134,8 @@ class SimpleTagTests(TagTestCase):
         self.assertEqual(t.render(c), "Hello Jack &amp; Jill!")
 
     def test_simple_tag_registration(self):
-        # Test that the decorators preserve the decorated function's docstring, name and attributes.
+        # The decorators preserve the decorated function's docstring, name,
+        # and attributes.
         self.verify_tag(custom.no_params, 'no_params')
         self.verify_tag(custom.one_param, 'one_param')
         self.verify_tag(custom.explicit_no_context, 'explicit_no_context')
@@ -257,7 +265,8 @@ class InclusionTagTests(TagTestCase):
             self.assertEqual(t.render(c), entry[1])
 
     def test_inclusion_tag_registration(self):
-        # Test that the decorators preserve the decorated function's docstring, name and attributes.
+        # The decorators preserve the decorated function's docstring, name,
+        # and attributes.
         self.verify_tag(inclusion.inclusion_no_params, 'inclusion_no_params')
         self.verify_tag(inclusion.inclusion_one_param, 'inclusion_one_param')
         self.verify_tag(inclusion.inclusion_explicit_no_context, 'inclusion_explicit_no_context')
@@ -273,7 +282,7 @@ class InclusionTagTests(TagTestCase):
 
     def test_15070_use_l10n(self):
         """
-        Test that inclusion tag passes down `use_l10n` of context to the
+        Inclusion tag passes down `use_l10n` of context to the
         Context of the included/rendered template as well.
         """
         c = Context({})
@@ -304,58 +313,32 @@ class InclusionTagTests(TagTestCase):
         self.assertEqual(template.render(Context({})).strip(), 'one\ntwo')
 
 
-class AssignmentTagTests(TagTestCase):
-
-    def test_assignment_tags(self):
-        c = Context({'value': 42})
-
-        t = self.engine.from_string('{% load custom %}{% assignment_no_params as var %}The result is: {{ var }}')
-        self.assertEqual(t.render(c), 'The result is: assignment_no_params - Expected result')
-
-    def test_assignment_tag_registration(self):
-        # Test that the decorators preserve the decorated function's docstring, name and attributes.
-        self.verify_tag(custom.assignment_no_params, 'assignment_no_params')
-
-    def test_assignment_tag_missing_context(self):
-        # The 'context' parameter must be present when takes_context is True
-        msg = (
-            "'assignment_tag_without_context_parameter' is decorated with "
-            "takes_context=True so it must have a first argument of 'context'"
-        )
-        with self.assertRaisesMessage(TemplateSyntaxError, msg):
-            self.engine.from_string('{% load custom %}{% assignment_tag_without_context_parameter 123 as var %}')
-
-
 class TemplateTagLoadingTests(SimpleTestCase):
 
     @classmethod
     def setUpClass(cls):
         cls.egg_dir = os.path.join(ROOT, 'eggs')
-        super(TemplateTagLoadingTests, cls).setUpClass()
+        super().setUpClass()
 
     def test_load_error(self):
         msg = (
             "Invalid template library specified. ImportError raised when "
             "trying to load 'template_tests.broken_tag': cannot import name "
-            "'?Xtemplate'?"
+            "'Xtemplate'"
         )
-        with six.assertRaisesRegex(self, InvalidTemplateLibrary, msg):
-            Engine(libraries={
-                'broken_tag': 'template_tests.broken_tag',
-            })
+        with self.assertRaisesMessage(InvalidTemplateLibrary, msg):
+            Engine(libraries={'broken_tag': 'template_tests.broken_tag'})
 
     def test_load_error_egg(self):
         egg_name = '%s/tagsegg.egg' % self.egg_dir
         msg = (
             "Invalid template library specified. ImportError raised when "
             "trying to load 'tagsegg.templatetags.broken_egg': cannot "
-            "import name '?Xtemplate'?"
+            "import name 'Xtemplate'"
         )
         with extend_sys_path(egg_name):
-            with six.assertRaisesRegex(self, InvalidTemplateLibrary, msg):
-                Engine(libraries={
-                    'broken_egg': 'tagsegg.templatetags.broken_egg',
-                })
+            with self.assertRaisesMessage(InvalidTemplateLibrary, msg):
+                Engine(libraries={'broken_egg': 'tagsegg.templatetags.broken_egg'})
 
     def test_load_working_egg(self):
         ttext = "{% load working_egg %}"
@@ -366,7 +349,6 @@ class TemplateTagLoadingTests(SimpleTestCase):
             })
             engine.from_string(ttext)
 
-    @skipUnless(six.PY3, "Python 3 only -- Python 2 doesn't have annotations.")
     def test_load_annotated_function(self):
         Engine(libraries={
             'annotated_tag_function': 'template_tests.annotated_tag_function',
