@@ -14,7 +14,7 @@
 
 DATABASES = {
     'default': {
-        'ENGINE': 'django.db.backends.sqlite3'
+        'ENGINE': 'django.db.backends.sqlite3',
     },
     'other': {
         'ENGINE': 'django.db.backends.sqlite3',
@@ -22,3 +22,8 @@ DATABASES = {
 }
 
 SECRET_KEY = "django_tests_secret_key"
+
+# Use a fast hasher to speed up tests.
+PASSWORD_HASHERS = [
+    'django.contrib.auth.hashers.MD5PasswordHasher',
+]
