@@ -1,7 +1,7 @@
-from django.conf.urls import url
+from django.urls import path
 
 from . import admin as tz_admin  # NOQA: register tz_admin
 
 urlpatterns = [
-    url(r'^admin/', tz_admin.site.urls),
+    path('admin/', tz_admin.site.urls),
 ]
