@@ -5,12 +5,11 @@ each name to resolve and Django must distinguish the possibilities based on the
 argument list.
 """
 
-from django.conf.urls import patterns, url
+from django.urls import re_path
 
 from .views import empty_view
 
-
-urlpatterns = patterns('',
-    url(r'^part/(?P<value>\w+)/$', empty_view, name="part"),
-    url(r'^part2/(?:(?P<value>\w+)/)?$', empty_view, name="part2"),
-)
+urlpatterns = [
+    re_path(r'^part/(?P<value>\w+)/$', empty_view, name='part'),
+    re_path(r'^part2/(?:(?P<value>\w+)/)?$', empty_view, name='part2'),
+]
