@@ -18,6 +18,6 @@ class Command(BaseCommand):
             raise CommandError()
         if options['verbosity'] > 0:
             self.stdout.write("I don't feel like dancing %s." % options["style"])
-            self.stdout.write(','.join(options.keys()))
+            self.stdout.write(','.join(options))
         if options['integer'] > 0:
             self.stdout.write("You passed %d as a positional argument." % options['integer'])
