@@ -1,8 +1,4 @@
-from __future__ import unicode_literals
-
-import sys
 from datetime import date, datetime
-from unittest import skipIf
 
 from django.test import SimpleTestCase, override_settings
 from django.test.utils import TZ_SUPPORT, requires_tz_support
@@ -12,11 +8,6 @@ from django.utils.timezone import (
     get_default_timezone, get_fixed_timezone, make_aware, utc,
 )
 
-try:
-    import pytz
-except ImportError:
-    pytz = None
-
 
 @override_settings(TIME_ZONE='Europe/Copenhagen')
 class DateFormatTests(SimpleTestCase):
@@ -36,18 +27,16 @@ class DateFormatTests(SimpleTestCase):
         dt = datetime(2009, 5, 16, 5, 30, 30)
         self.assertEqual(datetime.fromtimestamp(int(format(dt, 'U'))), dt)
 
-    @skipIf(sys.platform.startswith('win') and not pytz, "Test requires pytz on Windows")
     def test_naive_ambiguous_datetime(self):
-        # dt is ambiguous in Europe/Copenhagen. LocalTimezone guesses the
-        # offset (and gets it wrong 50% of the time) while pytz refuses the
-        # temptation to guess. In any case, this shouldn't crash.
+        # dt is ambiguous in Europe/Copenhagen. pytz raises an exception for
+        # the ambiguity, which results in an empty string.
         dt = datetime(2015, 10, 25, 2, 30, 0)
 
         # Try all formatters that involve self.timezone.
-        self.assertEqual(format(dt, 'I'), '0' if pytz is None else '')
-        self.assertEqual(format(dt, 'O'), '+0100' if pytz is None else '')
-        self.assertEqual(format(dt, 'T'), 'CET' if pytz is None else '')
-        self.assertEqual(format(dt, 'Z'), '3600' if pytz is None else '')
+        self.assertEqual(format(dt, 'I'), '')
+        self.assertEqual(format(dt, 'O'), '')
+        self.assertEqual(format(dt, 'T'), '')
+        self.assertEqual(format(dt, 'Z'), '')
 
     @requires_tz_support
     def test_datetime_with_local_tzinfo(self):
