@@ -1,10 +1,8 @@
 from django import template
 
-from ..views import BrokenException
-
 register = template.Library()
 
 
 @register.simple_tag
-def go_boom(arg):
-    raise BrokenException(arg)
+def go_boom():
+    raise Exception('boom')
