@@ -1,52 +1,51 @@
-# -*- coding: utf-8 -*-
-# This coding header is significant for tests, as the debug view is parsing
-# files to search for such a header to decode the source file content
-from __future__ import unicode_literals
-
 import importlib
 import inspect
 import os
 import re
 import sys
 import tempfile
-from unittest import skipIf
+from io import StringIO
+from pathlib import Path
+from unittest import mock
 
 from django.core import mail
 from django.core.files.uploadedfile import SimpleUploadedFile
 from django.db import DatabaseError, connection
+from django.shortcuts import render
 from django.template import TemplateDoesNotExist
 from django.test import RequestFactory, SimpleTestCase, override_settings
 from django.test.utils import LoggingCaptureMixin
-from django.urls import reverse
-from django.utils import six
-from django.utils.encoding import force_bytes, force_text
+from django.urls import path, reverse
 from django.utils.functional import SimpleLazyObject
+from django.utils.safestring import mark_safe
+from django.utils.version import PY36
 from django.views.debug import (
     CLEANSED_SUBSTITUTE, CallableSettingWrapper, ExceptionReporter,
-    cleanse_setting, technical_500_response,
+    Path as DebugPath, cleanse_setting, technical_500_response,
 )
 
-from .. import BrokenException, except_args
 from ..views import (
-    custom_exception_reporter_filter_view, multivalue_dict_key_error,
-    non_sensitive_view, paranoid_view, sensitive_args_function_caller,
-    sensitive_kwargs_function_caller, sensitive_method_view, sensitive_view,
+    custom_exception_reporter_filter_view, index_page,
+    multivalue_dict_key_error, non_sensitive_view, paranoid_view,
+    sensitive_args_function_caller, sensitive_kwargs_function_caller,
+    sensitive_method_view, sensitive_view,
 )
 
-if six.PY3:
-    from .py3_test_debug import Py3ExceptionReporterTests  # NOQA
 
-
-class User(object):
+class User:
     def __str__(self):
         return 'jacob'
 
 
+class WithoutEmptyPathUrls:
+    urlpatterns = [path('url/', index_page, name='url')]
+
+
 class CallableSettingWrapperTests(SimpleTestCase):
     """ Unittests for CallableSettingWrapper
     """
     def test_repr(self):
-        class WrappedCallable(object):
+        class WrappedCallable:
             def __repr__(self):
                 return "repr from the wrapped callable"
 
@@ -58,22 +57,25 @@ class CallableSettingWrapperTests(SimpleTestCase):
 
 
 @override_settings(DEBUG=True, ROOT_URLCONF='view_tests.urls')
-class DebugViewTests(LoggingCaptureMixin, SimpleTestCase):
+class DebugViewTests(SimpleTestCase):
 
     def test_files(self):
-        response = self.client.get('/raises/')
+        with self.assertLogs('django.request', 'ERROR'):
+            response = self.client.get('/raises/')
         self.assertEqual(response.status_code, 500)
 
         data = {
             'file_data.txt': SimpleUploadedFile('file_data.txt', b'haha'),
         }
-        response = self.client.post('/raises/', data)
+        with self.assertLogs('django.request', 'ERROR'):
+            response = self.client.post('/raises/', data)
         self.assertContains(response, 'file_data.txt', status_code=500)
         self.assertNotContains(response, 'haha', status_code=500)
 
     def test_400(self):
-        # Ensure that when DEBUG=True, technical_500_template() is called.
-        response = self.client.get('/raises400/')
+        # When DEBUG=True, technical_500_template() is called.
+        with self.assertLogs('django.security', 'WARNING'):
+            response = self.client.get('/raises400/')
         self.assertContains(response, '<div class="context" id="', status_code=400)
 
     # Ensure no 403.html template exists to test the default case.
@@ -103,37 +105,42 @@ class DebugViewTests(LoggingCaptureMixin, SimpleTestCase):
     def test_404(self):
         response = self.client.get('/raises404/')
         self.assertEqual(response.status_code, 404)
-
-    def test_raised_404(self):
-        response = self.client.get('/views/raises404/')
         self.assertContains(response, "<code>not-in-urls</code>, didn't match", status_code=404)
 
     def test_404_not_in_urls(self):
         response = self.client.get('/not-in-urls')
         self.assertNotContains(response, "Raised by:", status_code=404)
+        self.assertContains(response, "Django tried these URL patterns", status_code=404)
         self.assertContains(response, "<code>not-in-urls</code>, didn't match", status_code=404)
+        # Pattern and view name of a RegexURLPattern appear.
+        self.assertContains(response, r"^regex-post/(?P&lt;pk&gt;[0-9]+)/$", status_code=404)
+        self.assertContains(response, "[name='regex-post']", status_code=404)
+        # Pattern and view name of a RoutePattern appear.
+        self.assertContains(response, r"path-post/&lt;int:pk&gt;/", status_code=404)
+        self.assertContains(response, "[name='path-post']", status_code=404)
+
+    @override_settings(ROOT_URLCONF=WithoutEmptyPathUrls)
+    def test_404_empty_path_not_in_urls(self):
+        response = self.client.get('/')
+        self.assertContains(response, "The empty path didn't match any of these.", status_code=404)
 
     def test_technical_404(self):
-        response = self.client.get('/views/technical404/')
+        response = self.client.get('/technical404/')
         self.assertContains(response, "Raised by:", status_code=404)
         self.assertContains(response, "view_tests.views.technical404", status_code=404)
 
     def test_classbased_technical_404(self):
-        response = self.client.get('/views/classbased404/')
+        response = self.client.get('/classbased404/')
         self.assertContains(response, "Raised by:", status_code=404)
         self.assertContains(response, "view_tests.views.Http404View", status_code=404)
 
-    def test_view_exceptions(self):
-        for n in range(len(except_args)):
-            with self.assertRaises(BrokenException):
-                self.client.get(reverse('view_exception', args=(n,)))
-
     def test_non_l10ned_numeric_ids(self):
         """
         Numeric IDs and fancy traceback context blocks line numbers shouldn't be localized.
         """
         with self.settings(DEBUG=True, USE_L10N=True):
-            response = self.client.get('/raises500/')
+            with self.assertLogs('django.request', 'ERROR'):
+                response = self.client.get('/raises500/')
             # We look for a HTML fragment of the form
             # '<div class="context" id="c38123208">', not '<div class="context" id="c38,123,208"'
             self.assertContains(response, '<div class="context" id="', status_code=500)
@@ -142,18 +149,18 @@ class DebugViewTests(LoggingCaptureMixin, SimpleTestCase):
             id_repr = match.group('id')
             self.assertFalse(
                 re.search(b'[^c0-9]', id_repr),
-                "Numeric IDs in debug response HTML page shouldn't be localized (value: %s)." % id_repr
+                "Numeric IDs in debug response HTML page shouldn't be localized (value: %s)." % id_repr.decode()
             )
 
     def test_template_exceptions(self):
-        for n in range(len(except_args)):
+        with self.assertLogs('django.request', 'ERROR'):
             try:
-                self.client.get(reverse('template_exception', args=(n,)))
+                self.client.get(reverse('template_exception'))
             except Exception:
                 raising_loc = inspect.trace()[-1][-2][0].strip()
                 self.assertNotEqual(
-                    raising_loc.find('raise BrokenException'), -1,
-                    "Failed to find 'raise BrokenException' in last frame of "
+                    raising_loc.find("raise Exception('boom')"), -1,
+                    "Failed to find 'raise Exception' in last frame of "
                     "traceback, instead found: %s" % raising_loc
                 )
 
@@ -166,7 +173,7 @@ class DebugViewTests(LoggingCaptureMixin, SimpleTestCase):
             with override_settings(TEMPLATES=[{
                 'BACKEND': 'django.template.backends.django.DjangoTemplates',
                 'DIRS': [tempdir],
-            }]):
+            }]), self.assertLogs('django.request', 'ERROR'):
                 response = self.client.get(reverse('raises_template_does_not_exist', kwargs={"path": template_name}))
             self.assertContains(response, "%s (Source does not exist)" % template_path, status_code=500, count=2)
             # Assert as HTML.
@@ -182,8 +189,9 @@ class DebugViewTests(LoggingCaptureMixin, SimpleTestCase):
         """
         Make sure if you don't specify a template, the debug view doesn't blow up.
         """
-        with self.assertRaises(TemplateDoesNotExist):
-            self.client.get('/render_no_template/')
+        with self.assertLogs('django.request', 'ERROR'):
+            with self.assertRaises(TemplateDoesNotExist):
+                self.client.get('/render_no_template/')
 
     @override_settings(ROOT_URLCONF='view_tests.default_urls')
     def test_default_urlconf_template(self):
@@ -195,7 +203,7 @@ class DebugViewTests(LoggingCaptureMixin, SimpleTestCase):
         response = self.client.get('/')
         self.assertContains(
             response,
-            "<h2>Congratulations on your first Django-powered page.</h2>"
+            "<h2>The install worked successfully! Congratulations!</h2>"
         )
 
     @override_settings(ROOT_URLCONF='view_tests.regression_21530_urls')
@@ -218,7 +226,7 @@ class DebugViewTests(LoggingCaptureMixin, SimpleTestCase):
 
 class DebugViewQueriesAllowedTests(SimpleTestCase):
     # May need a query to initialize MySQL connection
-    allow_database_queries = True
+    databases = {'default'}
 
     def test_handle_db_exception(self):
         """
@@ -247,8 +255,9 @@ class DebugViewQueriesAllowedTests(SimpleTestCase):
 class NonDjangoTemplatesDebugViewTests(SimpleTestCase):
 
     def test_400(self):
-        # Ensure that when DEBUG=True, technical_500_template() is called.
-        response = self.client.get('/raises400/')
+        # When DEBUG=True, technical_500_template() is called.
+        with self.assertLogs('django.security', 'WARNING'):
+            response = self.client.get('/raises400/')
         self.assertContains(response, '<div class="context" id="', status_code=400)
 
     def test_403(self):
@@ -262,7 +271,8 @@ class NonDjangoTemplatesDebugViewTests(SimpleTestCase):
     def test_template_not_found_error(self):
         # Raises a TemplateDoesNotExist exception and shows the debug view.
         url = reverse('raises_template_does_not_exist', kwargs={"path": "notfound.html"})
-        response = self.client.get(url)
+        with self.assertLogs('django.request', 'ERROR'):
+            response = self.client.get(url)
         self.assertContains(response, '<div class="context" id="', status_code=500)
 
 
@@ -279,7 +289,7 @@ class ExceptionReporterTests(SimpleTestCase):
             exc_type, exc_value, tb = sys.exc_info()
         reporter = ExceptionReporter(request, exc_type, exc_value, tb)
         html = reporter.get_traceback_html()
-        self.assertIn('<h1>ValueError at /test_view/</h1>', html)
+        self.assertInHTML('<h1>ValueError at /test_view/</h1>', html)
         self.assertIn('<pre class="exception_value">Can&#39;t find my keys</pre>', html)
         self.assertIn('<th>Request Method:</th>', html)
         self.assertIn('<th>Request URL:</th>', html)
@@ -290,6 +300,7 @@ class ExceptionReporterTests(SimpleTestCase):
         self.assertIn('<h2>Traceback ', html)
         self.assertIn('<h2>Request information</h2>', html)
         self.assertNotIn('<p>Request data not supplied</p>', html)
+        self.assertIn('<p>No POST data</p>', html)
 
     def test_no_request(self):
         "An exception report can be generated without request"
@@ -299,7 +310,7 @@ class ExceptionReporterTests(SimpleTestCase):
             exc_type, exc_value, tb = sys.exc_info()
         reporter = ExceptionReporter(None, exc_type, exc_value, tb)
         html = reporter.get_traceback_html()
-        self.assertIn('<h1>ValueError</h1>', html)
+        self.assertInHTML('<h1>ValueError</h1>', html)
         self.assertIn('<pre class="exception_value">Can&#39;t find my keys</pre>', html)
         self.assertNotIn('<th>Request Method:</th>', html)
         self.assertNotIn('<th>Request URL:</th>', html)
@@ -311,13 +322,13 @@ class ExceptionReporterTests(SimpleTestCase):
         self.assertIn('<p>Request data not supplied</p>', html)
 
     def test_eol_support(self):
-        """Test that the ExceptionReporter supports Unix, Windows and Macintosh EOL markers"""
-        LINES = list('print %d' % i for i in range(1, 6))
+        """The ExceptionReporter supports Unix, Windows and Macintosh EOL markers"""
+        LINES = ['print %d' % i for i in range(1, 6)]
         reporter = ExceptionReporter(None, None, None, None)
 
         for newline in ['\n', '\r\n', '\r']:
             fd, filename = tempfile.mkstemp(text=False)
-            os.write(fd, force_bytes(newline.join(LINES) + newline))
+            os.write(fd, (newline.join(LINES) + newline).encode())
             os.close(fd)
 
             try:
@@ -333,7 +344,7 @@ class ExceptionReporterTests(SimpleTestCase):
         request = self.rf.get('/test_view/')
         reporter = ExceptionReporter(request, None, None, None)
         html = reporter.get_traceback_html()
-        self.assertIn('<h1>Report at /test_view/</h1>', html)
+        self.assertInHTML('<h1>Report at /test_view/</h1>', html)
         self.assertIn('<pre class="exception_value">No exception message supplied</pre>', html)
         self.assertIn('<th>Request Method:</th>', html)
         self.assertIn('<th>Request URL:</th>', html)
@@ -343,12 +354,63 @@ class ExceptionReporterTests(SimpleTestCase):
         self.assertIn('<h2>Request information</h2>', html)
         self.assertNotIn('<p>Request data not supplied</p>', html)
 
+    def test_reporting_of_nested_exceptions(self):
+        request = self.rf.get('/test_view/')
+        try:
+            try:
+                raise AttributeError(mark_safe('<p>Top level</p>'))
+            except AttributeError as explicit:
+                try:
+                    raise ValueError(mark_safe('<p>Second exception</p>')) from explicit
+                except ValueError:
+                    raise IndexError(mark_safe('<p>Final exception</p>'))
+        except Exception:
+            # Custom exception handler, just pass it into ExceptionReporter
+            exc_type, exc_value, tb = sys.exc_info()
+
+        explicit_exc = 'The above exception ({0}) was the direct cause of the following exception:'
+        implicit_exc = 'During handling of the above exception ({0}), another exception occurred:'
+
+        reporter = ExceptionReporter(request, exc_type, exc_value, tb)
+        html = reporter.get_traceback_html()
+        # Both messages are twice on page -- one rendered as html,
+        # one as plain text (for pastebin)
+        self.assertEqual(2, html.count(explicit_exc.format('&lt;p&gt;Top level&lt;/p&gt;')))
+        self.assertEqual(2, html.count(implicit_exc.format('&lt;p&gt;Second exception&lt;/p&gt;')))
+        self.assertEqual(10, html.count('&lt;p&gt;Final exception&lt;/p&gt;'))
+
+        text = reporter.get_traceback_text()
+        self.assertIn(explicit_exc.format('<p>Top level</p>'), text)
+        self.assertIn(implicit_exc.format('<p>Second exception</p>'), text)
+        self.assertEqual(3, text.count('<p>Final exception</p>'))
+
+    def test_reporting_frames_without_source(self):
+        try:
+            source = "def funcName():\n    raise Error('Whoops')\nfuncName()"
+            namespace = {}
+            code = compile(source, 'generated', 'exec')
+            exec(code, namespace)
+        except Exception:
+            exc_type, exc_value, tb = sys.exc_info()
+        request = self.rf.get('/test_view/')
+        reporter = ExceptionReporter(request, exc_type, exc_value, tb)
+        frames = reporter.get_traceback_frames()
+        last_frame = frames[-1]
+        self.assertEqual(last_frame['context_line'], '<source code not available>')
+        self.assertEqual(last_frame['filename'], 'generated')
+        self.assertEqual(last_frame['function'], 'funcName')
+        self.assertEqual(last_frame['lineno'], 2)
+        html = reporter.get_traceback_html()
+        self.assertIn('generated in funcName', html)
+        text = reporter.get_traceback_text()
+        self.assertIn('"generated" in funcName', text)
+
     def test_request_and_message(self):
         "A message can be provided in addition to a request"
         request = self.rf.get('/test_view/')
         reporter = ExceptionReporter(request, None, "I'm a little teapot", None)
         html = reporter.get_traceback_html()
-        self.assertIn('<h1>Report at /test_view/</h1>', html)
+        self.assertInHTML('<h1>Report at /test_view/</h1>', html)
         self.assertIn('<pre class="exception_value">I&#39;m a little teapot</pre>', html)
         self.assertIn('<th>Request Method:</th>', html)
         self.assertIn('<th>Request URL:</th>', html)
@@ -361,7 +423,7 @@ class ExceptionReporterTests(SimpleTestCase):
     def test_message_only(self):
         reporter = ExceptionReporter(None, None, "I'm a little teapot", None)
         html = reporter.get_traceback_html()
-        self.assertIn('<h1>Report</h1>', html)
+        self.assertInHTML('<h1>Report</h1>', html)
         self.assertIn('<pre class="exception_value">I&#39;m a little teapot</pre>', html)
         self.assertNotIn('<th>Request Method:</th>', html)
         self.assertNotIn('<th>Request URL:</th>', html)
@@ -386,10 +448,20 @@ class ExceptionReporterTests(SimpleTestCase):
         self.assertIn('VAL\\xe9VAL', html)
         self.assertIn('EXC\\xe9EXC', html)
 
+    def test_local_variable_escaping(self):
+        """Safe strings in local variables are escaped."""
+        try:
+            local = mark_safe('<p>Local variable</p>')
+            raise ValueError(local)
+        except Exception:
+            exc_type, exc_value, tb = sys.exc_info()
+        html = ExceptionReporter(None, exc_type, exc_value, tb).get_traceback_html()
+        self.assertIn('<td class="code"><pre>&#39;&lt;p&gt;Local variable&lt;/p&gt;&#39;</pre></td>', html)
+
     def test_unprintable_values_handling(self):
         "Unprintable values should not make the output generation choke."
         try:
-            class OomOutput(object):
+            class OomOutput:
                 def __repr__(self):
                     raise MemoryError('OOM')
             oomvalue = OomOutput()  # NOQA
@@ -405,7 +477,7 @@ class ExceptionReporterTests(SimpleTestCase):
         large = 256 * 1024
         repr_of_str_adds = len(repr(''))
         try:
-            class LargeOutput(object):
+            class LargeOutput:
                 def __repr__(self):
                     return repr('A' * large)
             largevalue = LargeOutput()  # NOQA
@@ -417,11 +489,25 @@ class ExceptionReporterTests(SimpleTestCase):
         self.assertEqual(len(html) // 1024 // 128, 0)  # still fit in 128Kb
         self.assertIn('&lt;trimmed %d bytes string&gt;' % (large + repr_of_str_adds,), html)
 
-    @skipIf(six.PY2, 'Bug manifests on PY3 only')
+    def test_encoding_error(self):
+        """
+        A UnicodeError displays a portion of the problematic string. HTML in
+        safe strings is escaped.
+        """
+        try:
+            mark_safe('abcdefghijkl<p>mnὀp</p>qrstuwxyz').encode('ascii')
+        except Exception:
+            exc_type, exc_value, tb = sys.exc_info()
+        reporter = ExceptionReporter(None, exc_type, exc_value, tb)
+        html = reporter.get_traceback_html()
+        self.assertIn('<h2>Unicode error hint</h2>', html)
+        self.assertIn('The string that could not be encoded/decoded was: ', html)
+        self.assertIn('<strong>&lt;p&gt;mnὀp&lt;/p&gt;</strong>', html)
+
     def test_unfrozen_importlib(self):
         """
         importlib is not a frozen app, but its loader thinks it's frozen which
-        results in an ImportError on Python 3. Refs #21443.
+        results in an ImportError. Refs #21443.
         """
         try:
             request = self.rf.get('/test_view/')
@@ -430,7 +516,7 @@ class ExceptionReporterTests(SimpleTestCase):
             exc_type, exc_value, tb = sys.exc_info()
         reporter = ExceptionReporter(request, exc_type, exc_value, tb)
         html = reporter.get_traceback_html()
-        self.assertIn('<h1>ImportError at /test_view/</h1>', html)
+        self.assertInHTML('<h1>%sError at /test_view/</h1>' % ('ModuleNotFound' if PY36 else 'Import'), html)
 
     def test_ignore_traceback_evaluation_exceptions(self):
         """
@@ -464,12 +550,46 @@ class ExceptionReporterTests(SimpleTestCase):
         html = reporter.get_traceback_html()
         self.assertIn("http://evil.com/", html)
 
+    def test_request_with_items_key(self):
+        """
+        An exception report can be generated for requests with 'items' in
+        request GET, POST, FILES, or COOKIES QueryDicts.
+        """
+        value = '<td>items</td><td class="code"><pre>&#39;Oops&#39;</pre></td>'
+        # GET
+        request = self.rf.get('/test_view/?items=Oops')
+        reporter = ExceptionReporter(request, None, None, None)
+        html = reporter.get_traceback_html()
+        self.assertInHTML(value, html)
+        # POST
+        request = self.rf.post('/test_view/', data={'items': 'Oops'})
+        reporter = ExceptionReporter(request, None, None, None)
+        html = reporter.get_traceback_html()
+        self.assertInHTML(value, html)
+        # FILES
+        fp = StringIO('filecontent')
+        request = self.rf.post('/test_view/', data={'name': 'filename', 'items': fp})
+        reporter = ExceptionReporter(request, None, None, None)
+        html = reporter.get_traceback_html()
+        self.assertInHTML(
+            '<td>items</td><td class="code"><pre>&lt;InMemoryUploadedFile: '
+            'items (application/octet-stream)&gt;</pre></td>',
+            html
+        )
+        # COOKES
+        rf = RequestFactory()
+        rf.cookies['items'] = 'Oops'
+        request = rf.get('/test_view/')
+        reporter = ExceptionReporter(request, None, None, None)
+        html = reporter.get_traceback_html()
+        self.assertInHTML('<td>items</td><td class="code"><pre>&#39;Oops&#39;</pre></td>', html)
+
     def test_exception_fetching_user(self):
         """
         The error page can be rendered if the current user can't be retrieved
         (such as when the database is unavailable).
         """
-        class ExceptionUser(object):
+        class ExceptionUser:
             def __str__(self):
                 raise Exception()
 
@@ -483,7 +603,7 @@ class ExceptionReporterTests(SimpleTestCase):
 
         reporter = ExceptionReporter(request, exc_type, exc_value, tb)
         html = reporter.get_traceback_html()
-        self.assertIn('<h1>ValueError at /test_view/</h1>', html)
+        self.assertInHTML('<h1>ValueError at /test_view/</h1>', html)
         self.assertIn('<pre class="exception_value">Oops</pre>', html)
         self.assertIn('<h3 id="user-info">USER</h3>', html)
         self.assertIn('<p>[unable to retrieve the current user]</p>', html)
@@ -491,6 +611,20 @@ class ExceptionReporterTests(SimpleTestCase):
         text = reporter.get_traceback_text()
         self.assertIn('USER: [unable to retrieve the current user]', text)
 
+    def test_template_encoding(self):
+        """
+        The templates are loaded directly, not via a template loader, and
+        should be opened as utf-8 charset as is the default specified on
+        template engines.
+        """
+        reporter = ExceptionReporter(None, None, None, None)
+        with mock.patch.object(DebugPath, 'open') as m:
+            reporter.get_traceback_html()
+            m.assert_called_once_with(encoding='utf-8')
+            m.reset_mock()
+            reporter.get_traceback_text()
+            m.assert_called_once_with(encoding='utf-8')
+
 
 class PlainTextReportTests(SimpleTestCase):
     rf = RequestFactory()
@@ -546,6 +680,55 @@ class PlainTextReportTests(SimpleTestCase):
         reporter = ExceptionReporter(request, None, "I'm a little teapot", None)
         reporter.get_traceback_text()
 
+    @override_settings(DEBUG=True)
+    def test_template_exception(self):
+        request = self.rf.get('/test_view/')
+        try:
+            render(request, 'debug/template_error.html')
+        except Exception:
+            exc_type, exc_value, tb = sys.exc_info()
+        reporter = ExceptionReporter(request, exc_type, exc_value, tb)
+        text = reporter.get_traceback_text()
+        templ_path = Path(Path(__file__).parent.parent, 'templates', 'debug', 'template_error.html')
+        self.assertIn(
+            'Template error:\n'
+            'In template %(path)s, error at line 2\n'
+            '   \'cycle\' tag requires at least two arguments\n'
+            '   1 : Template with error:\n'
+            '   2 :  {%% cycle %%} \n'
+            '   3 : ' % {'path': templ_path},
+            text
+        )
+
+    def test_request_with_items_key(self):
+        """
+        An exception report can be generated for requests with 'items' in
+        request GET, POST, FILES, or COOKIES QueryDicts.
+        """
+        # GET
+        request = self.rf.get('/test_view/?items=Oops')
+        reporter = ExceptionReporter(request, None, None, None)
+        text = reporter.get_traceback_text()
+        self.assertIn("items = 'Oops'", text)
+        # POST
+        request = self.rf.post('/test_view/', data={'items': 'Oops'})
+        reporter = ExceptionReporter(request, None, None, None)
+        text = reporter.get_traceback_text()
+        self.assertIn("items = 'Oops'", text)
+        # FILES
+        fp = StringIO('filecontent')
+        request = self.rf.post('/test_view/', data={'name': 'filename', 'items': fp})
+        reporter = ExceptionReporter(request, None, None, None)
+        text = reporter.get_traceback_text()
+        self.assertIn('items = <InMemoryUploadedFile:', text)
+        # COOKES
+        rf = RequestFactory()
+        rf.cookies['items'] = 'Oops'
+        request = rf.get('/test_view/')
+        reporter = ExceptionReporter(request, None, None, None)
+        text = reporter.get_traceback_text()
+        self.assertIn("items = 'Oops'", text)
+
     def test_message_only(self):
         reporter = ExceptionReporter(None, None, "I'm a little teapot", None)
         reporter.get_traceback_text()
@@ -559,15 +742,15 @@ class PlainTextReportTests(SimpleTestCase):
         self.assertIn("http://evil.com/", text)
 
 
-class ExceptionReportTestMixin(object):
-
+class ExceptionReportTestMixin:
     # Mixin used in the ExceptionReporterFilterTests and
     # AjaxResponseExceptionReporterFilter tests below
-
-    breakfast_data = {'sausage-key': 'sausage-value',
-                      'baked-beans-key': 'baked-beans-value',
-                      'hash-brown-key': 'hash-brown-value',
-                      'bacon-key': 'bacon-value'}
+    breakfast_data = {
+        'sausage-key': 'sausage-value',
+        'baked-beans-key': 'baked-beans-value',
+        'hash-brown-key': 'hash-brown-value',
+        'bacon-key': 'bacon-value',
+    }
 
     def verify_unsafe_response(self, view, check_for_vars=True,
                                check_for_POST_params=True):
@@ -603,7 +786,7 @@ class ExceptionReportTestMixin(object):
             self.assertContains(response, 'sauce', status_code=500)
             self.assertNotContains(response, 'worcestershire', status_code=500)
         if check_for_POST_params:
-            for k, v in self.breakfast_data.items():
+            for k in self.breakfast_data:
                 # All POST parameters' names are shown.
                 self.assertContains(response, k, status_code=500)
             # Non-sensitive POST parameters' values are shown.
@@ -645,14 +828,14 @@ class ExceptionReportTestMixin(object):
             email = mail.outbox[0]
 
             # Frames vars are never shown in plain text email reports.
-            body_plain = force_text(email.body)
+            body_plain = str(email.body)
             self.assertNotIn('cooked_eggs', body_plain)
             self.assertNotIn('scrambled', body_plain)
             self.assertNotIn('sauce', body_plain)
             self.assertNotIn('worcestershire', body_plain)
 
             # Frames vars are shown in html email reports.
-            body_html = force_text(email.alternatives[0][0])
+            body_html = str(email.alternatives[0][0])
             self.assertIn('cooked_eggs', body_html)
             self.assertIn('scrambled', body_html)
             self.assertIn('sauce', body_html)
@@ -678,21 +861,21 @@ class ExceptionReportTestMixin(object):
             email = mail.outbox[0]
 
             # Frames vars are never shown in plain text email reports.
-            body_plain = force_text(email.body)
+            body_plain = str(email.body)
             self.assertNotIn('cooked_eggs', body_plain)
             self.assertNotIn('scrambled', body_plain)
             self.assertNotIn('sauce', body_plain)
             self.assertNotIn('worcestershire', body_plain)
 
             # Frames vars are shown in html email reports.
-            body_html = force_text(email.alternatives[0][0])
+            body_html = str(email.alternatives[0][0])
             self.assertIn('cooked_eggs', body_html)
             self.assertIn('scrambled', body_html)
             self.assertIn('sauce', body_html)
             self.assertNotIn('worcestershire', body_html)
 
             if check_for_POST_params:
-                for k, v in self.breakfast_data.items():
+                for k in self.breakfast_data:
                     # All POST parameters' names are shown.
                     self.assertIn(k, body_plain)
                 # Non-sensitive POST parameters' values are shown.
@@ -717,7 +900,7 @@ class ExceptionReportTestMixin(object):
             self.assertEqual(len(mail.outbox), 1)
             email = mail.outbox[0]
             # Frames vars are never shown in plain text email reports.
-            body = force_text(email.body)
+            body = str(email.body)
             self.assertNotIn('cooked_eggs', body)
             self.assertNotIn('scrambled', body)
             self.assertNotIn('sauce', body)
@@ -732,14 +915,13 @@ class ExceptionReportTestMixin(object):
 @override_settings(ROOT_URLCONF='view_tests.urls')
 class ExceptionReporterFilterTests(ExceptionReportTestMixin, LoggingCaptureMixin, SimpleTestCase):
     """
-    Ensure that sensitive information can be filtered out of error reports.
-    Refs #14614.
+    Sensitive information can be filtered out of error reports (#14614).
     """
     rf = RequestFactory()
 
     def test_non_sensitive_request(self):
         """
-        Ensure that everything (request info and frame variables) can bee seen
+        Everything (request info and frame variables) can bee seen
         in the default error reports for non-sensitive requests.
         """
         with self.settings(DEBUG=True):
@@ -752,7 +934,7 @@ class ExceptionReporterFilterTests(ExceptionReportTestMixin, LoggingCaptureMixin
 
     def test_sensitive_request(self):
         """
-        Ensure that sensitive POST parameters and frame variables cannot be
+        Sensitive POST parameters and frame variables cannot be
         seen in the default error reports for sensitive requests.
         """
         with self.settings(DEBUG=True):
@@ -765,7 +947,7 @@ class ExceptionReporterFilterTests(ExceptionReportTestMixin, LoggingCaptureMixin
 
     def test_paranoid_request(self):
         """
-        Ensure that no POST parameters and frame variables can be seen in the
+        No POST parameters and frame variables can be seen in the
         default error reports for "paranoid" requests.
         """
         with self.settings(DEBUG=True):
@@ -778,7 +960,7 @@ class ExceptionReporterFilterTests(ExceptionReportTestMixin, LoggingCaptureMixin
 
     def test_multivalue_dict_key_error(self):
         """
-        #21098 -- Ensure that sensitive POST parameters cannot be seen in the
+        #21098 -- Sensitive POST parameters cannot be seen in the
         error reports for if request.POST['nonexistent_key'] throws an error.
         """
         with self.settings(DEBUG=True):
@@ -791,7 +973,7 @@ class ExceptionReporterFilterTests(ExceptionReportTestMixin, LoggingCaptureMixin
 
     def test_custom_exception_reporter_filter(self):
         """
-        Ensure that it's possible to assign an exception reporter filter to
+        It's possible to assign an exception reporter filter to
         the request to bypass the one set in DEFAULT_EXCEPTION_REPORTER_FILTER.
         """
         with self.settings(DEBUG=True):
@@ -804,28 +986,21 @@ class ExceptionReporterFilterTests(ExceptionReportTestMixin, LoggingCaptureMixin
 
     def test_sensitive_method(self):
         """
-        Ensure that the sensitive_variables decorator works with object
-        methods.
-        Refs #18379.
+        The sensitive_variables decorator works with object methods.
         """
         with self.settings(DEBUG=True):
-            self.verify_unsafe_response(sensitive_method_view,
-                                        check_for_POST_params=False)
-            self.verify_unsafe_email(sensitive_method_view,
-                                     check_for_POST_params=False)
+            self.verify_unsafe_response(sensitive_method_view, check_for_POST_params=False)
+            self.verify_unsafe_email(sensitive_method_view, check_for_POST_params=False)
 
         with self.settings(DEBUG=False):
-            self.verify_safe_response(sensitive_method_view,
-                                      check_for_POST_params=False)
-            self.verify_safe_email(sensitive_method_view,
-                                   check_for_POST_params=False)
+            self.verify_safe_response(sensitive_method_view, check_for_POST_params=False)
+            self.verify_safe_email(sensitive_method_view, check_for_POST_params=False)
 
     def test_sensitive_function_arguments(self):
         """
-        Ensure that sensitive variables don't leak in the sensitive_variables
-        decorator's frame, when those variables are passed as arguments to the
-        decorated function.
-        Refs #19453.
+        Sensitive variables don't leak in the sensitive_variables decorator's
+        frame, when those variables are passed as arguments to the decorated
+        function.
         """
         with self.settings(DEBUG=True):
             self.verify_unsafe_response(sensitive_args_function_caller)
@@ -837,10 +1012,9 @@ class ExceptionReporterFilterTests(ExceptionReportTestMixin, LoggingCaptureMixin
 
     def test_sensitive_function_keyword_arguments(self):
         """
-        Ensure that sensitive variables don't leak in the sensitive_variables
-        decorator's frame, when those variables are passed as keyword arguments
-        to the decorated function.
-        Refs #19453.
+        Sensitive variables don't leak in the sensitive_variables decorator's
+        frame, when those variables are passed as keyword arguments to the
+        decorated function.
         """
         with self.settings(DEBUG=True):
             self.verify_unsafe_response(sensitive_kwargs_function_caller)
@@ -865,7 +1039,7 @@ class ExceptionReporterFilterTests(ExceptionReportTestMixin, LoggingCaptureMixin
         Callable settings which forbid to set attributes should not break
         the debug page (#23070).
         """
-        class CallableSettingWithSlots(object):
+        class CallableSettingWithSlots:
             __slots__ = []
 
             def __call__(self):
@@ -923,7 +1097,7 @@ class ExceptionReporterFilterTests(ExceptionReportTestMixin, LoggingCaptureMixin
 
 class AjaxResponseExceptionReporterFilter(ExceptionReportTestMixin, LoggingCaptureMixin, SimpleTestCase):
     """
-    Ensure that sensitive information can be filtered out of error reports.
+    Sensitive information can be filtered out of error reports.
 
     Here we specifically test the plain text 500 debug-only error page served
     when it has been detected the request was sent by JS code. We don't check
@@ -935,7 +1109,7 @@ class AjaxResponseExceptionReporterFilter(ExceptionReportTestMixin, LoggingCaptu
 
     def test_non_sensitive_request(self):
         """
-        Ensure that request info can bee seen in the default error reports for
+        Request info can bee seen in the default error reports for
         non-sensitive requests.
         """
         with self.settings(DEBUG=True):
@@ -946,7 +1120,7 @@ class AjaxResponseExceptionReporterFilter(ExceptionReportTestMixin, LoggingCaptu
 
     def test_sensitive_request(self):
         """
-        Ensure that sensitive POST parameters cannot be seen in the default
+        Sensitive POST parameters cannot be seen in the default
         error reports for sensitive requests.
         """
         with self.settings(DEBUG=True):
@@ -957,7 +1131,7 @@ class AjaxResponseExceptionReporterFilter(ExceptionReportTestMixin, LoggingCaptu
 
     def test_paranoid_request(self):
         """
-        Ensure that no POST parameters can be seen in the default error reports
+        No POST parameters can be seen in the default error reports
         for "paranoid" requests.
         """
         with self.settings(DEBUG=True):
@@ -968,7 +1142,7 @@ class AjaxResponseExceptionReporterFilter(ExceptionReportTestMixin, LoggingCaptu
 
     def test_custom_exception_reporter_filter(self):
         """
-        Ensure that it's possible to assign an exception reporter filter to
+        It's possible to assign an exception reporter filter to
         the request to bypass the one set in DEFAULT_EXCEPTION_REPORTER_FILTER.
         """
         with self.settings(DEBUG=True):
@@ -977,6 +1151,11 @@ class AjaxResponseExceptionReporterFilter(ExceptionReportTestMixin, LoggingCaptu
         with self.settings(DEBUG=False):
             self.verify_unsafe_response(custom_exception_reporter_filter_view, check_for_vars=False)
 
+    @override_settings(DEBUG=True, ROOT_URLCONF='view_tests.urls')
+    def test_ajax_response_encoding(self):
+        response = self.client.get('/raises500/', HTTP_X_REQUESTED_WITH='XMLHttpRequest')
+        self.assertEqual(response['Content-Type'], 'text/plain; charset=utf-8')
+
 
 class HelperFunctionTests(SimpleTestCase):
 
